Package: genArise
Version: 1.89.0
Title: Microarray Analysis tool
Author: Ana Patricia Gomez Mayen <pgomez@ifc.unam.mx>,\\ Gustavo
        Corral Guille <gcorral@ifc.unam.mx>, \\ Lina Riego Ruiz
        <lriego@ifc.unam.mx>,\\ Gerardo Coello Coutino
        <gcoello@ifc.unam.mx>
Acknowledgements: Jorge Ramirez Salcedo and Microarray Unit IFC
Maintainer: IFC Development Team <info-genarise@ifc.unam.mx>
Depends: R (>= 1.7.1), locfit, tkrplot, methods
Imports: graphics, grDevices, methods, stats, tcltk, utils, xtable
Description: genArise is an easy to use tool for dual color microarray
        data. Its GUI-Tk based environment let any non-experienced
        user performs a basic, but not simple, data analysis just
        following a wizard. In addition it provides some tools for the
        developer.
biocViews: Microarray, TwoChannel, Preprocessing
License: file LICENSE
License_restricts_use: yes
URL: http://www.ifc.unam.mx/genarise
git_url: https://git.bioconductor.org/packages/genArise
git_branch: devel
git_last_commit: fcaf847
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: no
Packaged: 2026-05-18 22:07:10 UTC; biocbuild
Built: R 4.6.0; ; 2026-05-19 04:52:37 UTC; unix

import(methods)
import(tcltk)
importFrom(graphics, image, lines, par, plot, points)
importFrom(grDevices, col2rgb, dev.cur, dev.off, pdf, rgb)
importFrom(stats, fitted, lowess, sd)
importFrom(utils, read.csv, read.table, write.table)
importFrom(xtable, xtable)

exportClasses(Spot, DataSet)
export(a.arise, alter.unique, analysis.window, annotations, back.gui,
       bg.correct, create.project, cys.plot, filter.spot, genArise,
       genArise.init, genMerge, get.values, get.Zscore, global.norm,
       graphic.choose, grid.norm, help, i.arise, imageLimma, m.arise,
       ma.plot, make.swap, meanUnique, note, old.project, post.analysis,
       principal, projects.select, r.arise, read.dataset, read.spot,
       reset.history, ri.plot, set.grid.properties, set.history.project,
       set.path.project, set.project.properties, single.norm, spotUnique,
       swap.select, trim, write.dataSet, write.spot, write.zscore, Zscore,
       Zscore.plot, Zscore.points)

# Extract columns of interest from complete data frames

#Intensity log-ratio
m.arise <- function (mySpot){
	spot <- as.list(attr(mySpot, "spotData"))
	cys.index <- (spot$Cy3 > 0) & (spot$Cy5 > 0) & (!is.na(spot$Cy3)) &(!is.na(spot$Cy5)) & is.finite(spot$Cy3) & is.finite(spot$Cy5)
	cy3 <- spot$Cy3[cys.index]
	cy5 <- spot$Cy5[cys.index]
	log(cy5,2) - log(cy3, 2)
}

#Mean log intensity
a.arise <- function(mySpot) {
	spot <- attr(mySpot, "spotData")
	cys.index <- (spot$Cy3 > 0) & (spot$Cy5 > 0) & (!is.na(spot$Cy3)) &(!is.na(spot$Cy5)) & is.finite(spot$Cy3) & is.finite(spot$Cy5)
	cy3 <- spot$Cy3[cys.index]
	cy5 <- spot$Cy5[cys.index]
	(log(cy3, 2) + log(cy5, 2))/2
}

#Get the Ratios
r.arise <- function(mySpot){
	spot <- as.list(attr(mySpot, "spotData"))
	log(spot$Cy5,2) - log(spot$Cy3, 2)
}

#Get the Intensities
i.arise <- function(mySpot){
	spot <- as.list(attr(mySpot, "spotData"))
	log(spot$Cy3, 10) + log(spot$Cy5, 10)
}

# Miscelaneous functions to get information for the project to perform

# Just set the options for the GUI
set.project.properties <- function(envir){

  assign("graphic.type", 2, envir = envir)
  assign("icons.dir", file.path(system.file(),"..", "genArise", "doc"), envir = envir)

  if(.Platform$OS.type == "unix"){
    assign("Myhscale", 1 , envir = envir)
    assign("Myvscale", 1 , envir = envir)
    assign("barWidth", 70, envir=envir)
    assign("barHeight", 5, envir=envir)
    assign("height","550",envir=envir)
    assign("width", "55", envir = envir)
    assign("editor", "emacs", envir = envir)
  }else{
    assign("Myhscale", 1.3 , envir = envir)
    assign("Myvscale", 1.3 , envir = envir)
    assign("height","550",envir=envir)
    assign("barWidth", 50, envir=envir)
    assign("barHeight", 3, envir=envir)
    assign("width", "55", envir = envir)
    assign("editor", "notepad", envir = envir)
  }
}

# Set properties of the experiment
# nr: Rows, nc: Columns, nmr: Meta-rows, nmc: Meta-columns
set.grid.properties <- function(envir, name, nr, nc, nmr, nmc){
  assign("name.project", name, envir = envir)
  assign("nr", nr, envir = envir)
  assign("nc", nc, envir = envir)
  assign("nmr", nmr, envir = envir)
  assign("nmc", nmc, envir = envir)
}

# The path of the project's output 
set.path.project <-  function(path, results.file, graphics.file, envir){
  assign("path.project", path, envir = envir)
  assign("path.results", file.path(path, results.file), envir = envir)
  assign("path.graphics", file.path(path, graphics.file), envir = envir)
}

# Create the directory's hierarchy of the project
create.project <-  function(project.name, results.file = "Results", graphics.file = "Graphics"){
  if(length(unlist(strsplit(results.file, .Platform$file.sep))) > 1 || length(unlist(strsplit(graphics.file, .Platform$file.sep))) > 1 )
    stop("Argument 2 and 3 must be the file name without the path!..")
  path <-  unlist(strsplit(project.name, .Platform$file.sep))
  history.project <- file(paste(file.path(project.name), ".prj", sep = ""), "w")
  set.history.project(paste(file.path(project.name), ".prj", sep = ""), results.file, graphics.file)
  close(history.project)
  dir.create(project.name)
  dir.create(file.path(project.name, results.file))
  dir.create(file.path(project.name, graphics.file))
}

# Concatenate the complete text that appear in the history file of the project
set.history.project <- function(history.file, id.name, data.file){
  history.project <- file(history.file, "a")
  cat(paste(id.name, data.file, sep = "\t"), file = history.file, append = TRUE, sep = "\n")
  close(history.project)
}

# Select the type of graphic to be shown
graphic.choose <- function(spot.object, graphic.type){
  switch(graphic.type,
         cys.plot(spot.object),
         ri.plot(spot.object),
         ma.plot(spot.object))
}

#Triming a string
trim <- function(word){
  sub("^[ \t\n\r]*", "", sub("[ \t\n\r]*$", "", word))
}

# Update the information of the history file of the project
reset.history <- function(history.file, text){
  write.table(text, history.file, sep ="\t", quote = FALSE, col.names = FALSE, row.names = FALSE)
}

imageLimma <-  function(z, row, column, meta.row, meta.column, low = NULL, high = NULL){
  if (is.null(row) || is.null(column) || is.null(meta.row) || is.null(meta.column))
    stop("Layout needs to contain components ngrid.r, ngrid.c, nspot.r and spot.c")
  if (length(z) != row * column * meta.row * meta.column)
    stop("Number of image spots does not agree with layout dimensions")
  if (is.character(low))
    low <- col2rgb(low)/255
  if (is.character(high))
    high <- col2rgb(high)/255
  if (!is.null(low) && is.null(high))
    high <- c(1, 1, 1) - low
  if (is.null(low) && !is.null(high))
    low <- c(1, 1, 1) - high
  zr0 <- range(z, na.rm = TRUE)
  zr <- range(z, na.rm = TRUE, finite = TRUE)
  zmax <- max(abs(zr))
  zmin <- zr[1]
  zerocenter <- (zmin < 0)
  if (zerocenter) {
    if (is.null(low))
      low <- c(0, 1, 0)
    if (is.null(high))
      high <- c(1, 0, 0)
    zlim <- c(-zmax, zmax)
  }
  else {
    if (is.null(low))
      low <- c(1, 1, 1)
    if (is.null(high))
      high <- c(0, 0, 1)
      zlim <- c(zmin, zmax)
  }
  col <- rgb(seq(high[1], low[1], len = 123), seq(high[2], low[2], len = 123), seq(high[3], low[3], len = 123))
  dim(z) <- c(column, row, meta.column, meta.row)
  z <- aperm(z, perm = c(1, 3, 2, 4))
  dim(z) <- c(meta.column * column, meta.row * row)
  old.par <- par(mar = c(1, 1, 1, 1))
  on.exit(par(old.par))
  
    image(x = 0:(meta.column * column), y = 0:(meta.row * row), z = z, zlim = zlim, ylim = c(row * meta.row, 0), col = col, add = FALSE, xaxs = "i", yaxs = "i", xlab = "", ylab = "",axes = FALSE)
  
  for (igrid in 0:meta.row) lines(c(0, meta.column * column), rep(igrid * row,2))
  for (igrid in 0:meta.column) lines(rep(igrid * column, 2), c(0, meta.row * row))

}

# Just the graphic help of genArise

help <- function(){
  tclRequire("BWidget")
  tt <- tktoplevel()
  tkfocus(tt)
  tkfocus(tt)
  tkwm.title(tt,"genArise Help")
  upper.frame <- tkframe(tt)
  
  frameOverall <- tkframe(upper.frame)
  
  yscr <- tkscrollbar(frameOverall, command=function(...)tkyview(txt,...))
  txt <- tktext(frameOverall,bg="white",font="Helvetica 12",yscrollcommand=function(...)tkset(yscr,...))
  tkgrid(txt,yscr)
  tkgrid.configure(yscr, stick="nsw")

  
  tree.frame <- tkframe(upper.frame, relief = "groove", borderwidth=2)
  
  xScr       <- tkscrollbar(tree.frame,command=function(...)tkxview(treeWidget,...),orient="horizontal")
  yScr       <- tkscrollbar(tree.frame,command=function(...)tkyview(treeWidget,...))
  treeWidget <- tkwidget(tree.frame,"Tree",xscrollcommand=function(...)tkset(xScr,...),
                         yscrollcommand=function(...)tkset(yScr,...),width=30,height=20)
  tkgrid(treeWidget,yScr)
  tkgrid.configure(yScr,stick="nsw")
  tkgrid(xScr)
  tkgrid.configure(xScr,stick="new")

   tkinsert(treeWidget,"end","root", "About genArise", text = "About genArise")
  
  tkinsert(treeWidget,"end","root", "Project", text = "Project")
  tkinsert(treeWidget,"end",  "Project", "Creating a New Project", text = "Creating a New Project")
  tkinsert(treeWidget,"end",  "Creating a New Project", "File Format", text = "File Format")
  tkinsert(treeWidget,"end",  "Project", "Opening an Old Project", text = "Opening an Old Project")
  
  tkinsert(treeWidget,"end","root", "Diagnostic Plots", text = "Diagnostic Plots")
  tkinsert(treeWidget,"end",  "Diagnostic Plots", "Intensity Ratio Plot", text = "Intensity Ratio Plot")
  tkinsert(treeWidget,"end",  "Diagnostic Plots", "Background of Cy3 Plot", text = "Background of Cy3 Plot")
  tkinsert(treeWidget,"end",  "Diagnostic Plots", "Background of Cy5 Plot", text = "Background of Cy5 Plot")
  
  tkinsert(treeWidget,"end","root", "Microarray Analysis", text = "Microarray Analysis")
  tkinsert(treeWidget,"end",  "Microarray Analysis", "Background Correction", text = "Background Correction")  
  tkinsert(treeWidget,"end",  "Microarray Analysis", "Normalization", text = "Normalization")
  tkinsert(treeWidget,"end",  "Normalization", "By Grid", text = "By Grid")
  tkinsert(treeWidget,"end",  "Normalization", "Global", text = "Global")
  tkinsert(treeWidget,"end",  "Microarray Analysis", "Filter", text = "Filter")
  tkinsert(treeWidget,"end",  "Microarray Analysis", "Duplicates Analysis", text = "Duplicates Analysis")
  tkinsert(treeWidget,"end",  "Duplicates Analysis", "Mean Replicates Filter", text = "Mean Replicates Filter")
  tkinsert(treeWidget,"end",  "Duplicates Analysis", "Non-extreme Values Filter", text = "Non-extreme Values Filter")
  tkinsert(treeWidget,"end",  "Duplicates Analysis", "Geometric Mean Filter", text = "Geometric Mean Filter")

  tkinsert(treeWidget,"end","root", "Graphics", text = "Graphics")
  tkinsert(treeWidget,"end",  "Graphics", "Cy3 -vs- Cy5", text = "Cy3 -vs- Cy5")
  tkinsert(treeWidget,"end",  "Graphics", "R -vs- I", text = "R -vs- I")
  tkinsert(treeWidget,"end",  "Graphics", "M -vs- A", text = "M -vs- A")

  tkinsert(treeWidget,"end",  "root", "Write Output", text = "Write Output")

  tkinsert(treeWidget,"end","root", "Z-score", text = "Z-score")
  
  tkinsert(treeWidget,"end","root", "Post-Analysis", text = "Post-Analysis")

  tkinsert(treeWidget,"end",  "root", "genArise Keyboard Shortcuts", text = "genArise Keyboard Shortcuts")
  
  tkselection.set(treeWidget,"About genArise")
  topic <- as.character(tclvalue(tcl(treeWidget,"selection","get")))
  first <- substr(topic,1,1)
  last <- substr(topic,nchar(topic),nchar(topic))
  if(first =="{" && last == "}") topic <- substr(topic,2,nchar(topic)-1)
  line <- readLines(file.path(system.file(),"..","genArise","doc","help","contents"))
  title <- which(line == topic)
  lim <- which(line[title:length(line)] == "*")[1]
  text <- paste(line[title:((lim-1)+(title-1))],"",collapse="\n\t",sep="")
  tkinsert(txt,"0.0",text)
  events <- function(){
    tkconfigure(txt, state="normal")
    tkdelete(txt,"0.0","end")
    topic <- tclvalue(tcl(treeWidget,"selection","get"))
    first <- substr(topic,1,1)
    last <- substr(topic,nchar(topic),nchar(topic))
    if(first =="{" && last == "}") topic <- substr(topic,2,nchar(topic)-1)
    line <- readLines(file.path(system.file(),"..", "genArise","doc","help","contents"))
    title <- which(line == topic)
    lim <- which(line[title:length(line)] == "*")[1]
    text <- paste(line[title:((lim-1)+(title-1))],"",collapse="\n\t",sep="")
    tkinsert(txt,"0.0",text)
    tkconfigure(txt, state="disabled")
  }
      



  tkbind(tt, "<Button-1>",events)  
  tkgrid(tklabel(upper.frame,text = "Contents" ), padx = "10")
  tkgrid(tree.frame, frameOverall,padx = "10", pady = "10")
  tkgrid(upper.frame, pady = "10")
}

require(locfit)
bg.correct <- function(mySpot){
  spot <- as.list(attr(mySpot, "spotData"))
  spot$Cy3 <- spot$Cy3 - spot$BgCy3
  spot$Cy5 <- spot$Cy5 - spot$BgCy5
  spotName <- attr(mySpot, "name")
  if(is.null(spot$Symdesc)){
    SD <- list(Cy3 = spot$Cy3, Cy5 = spot$Cy5, BgCy3 = spot$BgCy3, BgCy5 = spot$BgCy5, Id = spot$Id)
  }
  else{
    SD <- list(Cy3 = spot$Cy3, Cy5 = spot$Cy5, BgCy3 = spot$BgCy3, BgCy5 = spot$BgCy5, Id = spot$Id, Symdesc = spot$Symdesc)
  }
  new("Spot", name = spotName, spotData= SD)
}

                                        # Filtering
# Intensity-based filtering of array elements

filter.spot <- function (mySpot){
  spot <- attr(mySpot, "spotData")
  cys <- (!is.na(spot$Cy3)) & (!is.na(spot$Cy5)) & (!is.na(spot$BgCy3)) & (!is.na(spot$BgCy5)) & (!is.na(spot$Id))
  spot$Cy3 <- spot$Cy3[cys]
  spot$Cy5 <- spot$Cy5[cys]
  spot$BgCy3 <- spot$BgCy3[cys]
  spot$BgCy5 <- spot$BgCy5[cys]
  spot$Id <- spot$Id[cys]
  condition <- spot$Cy3 > 2 * sd(spot$BgCy3) & spot$Cy5 > 2 * sd(spot$BgCy5)
  spot$Cy3 <- spot$Cy3[condition]
  spot$Cy5 <- spot$Cy5[condition]
  spot$BgCy3 <- spot$BgCy3[condition]
  spot$BgCy5 <- spot$BgCy5[condition]
  spot$Id <- spot$Id[condition]
  if(!is.null(spot$Symdesc)){
    spot$Symdesc <- spot$Symdesc[cys]
    spot$Symdesc <- spot$Symdesc[condition]
    newSpot <- list(Cy3 = spot$Cy3 , Cy5= spot$Cy5 , BgCy3 = spot$BgCy3 , BgCy5 = spot$BgCy5, Id = spot$Id, Symdesc = spot$Symdesc)
  }
  else{
    newSpot <- list(Cy3 = spot$Cy3 , Cy5= spot$Cy5 , BgCy3 = spot$BgCy3 , BgCy5 = spot$BgCy5, Id = spot$Id)
  }
  spotName <- attr(mySpot, "name")
  new("Spot", name = spotName, spotData= newSpot)
}

# Normalisation

# Normalisation by grid

grid.norm <- function(mySpot, nr, nc){
  spot <- attr(mySpot, "spotData")
  x <- spot$Cy3
  y <- spot$Cy5
  k <-(nc * nr)-1
  j <- 1
  Cy3 <- Cy5 <- BgCy3 <- BgCy5<- Id <- Symdesc <- list()
  bloque <- 1
  n <- length (x)
  while(j<(n+1)){
    a <- unlist(sapply(j : (j+k),function(x) spot$Cy3[x]))
    b <- unlist(sapply(j : (j+k),function(x) spot$Cy5[x]))
    c <- unlist(sapply(j : (j+k),function(x) spot$BgCy3[x]))
    d <- unlist(sapply(j : (j+k),function(x) spot$BgCy5[x]))
    e <- unlist(sapply(j : (j+k),function(x) spot$Id[x]))

    if(!is.null(spot$Symdesc)){
      f <- unlist(sapply(j : (j+k),function(x) spot$Symdesc[x]))
      index.temp <- a > 0 &  b > 0 & (!is.na(a)) & (!is.na(b)) & (!is.na(c)) & (!is.na(d)) & (!is.na(e)) & (!is.na(f))
      f <- f[index.temp]
    }
    else{
      index.temp <- a > 0 &  b > 0 & (!is.na(a)) & (!is.na(b)) & (!is.na(c)) & (!is.na(d)) & (!is.na(e))
    }
    a <- a[index.temp]
    b <- b[index.temp]
    c <- c[index.temp]
    d <- d[index.temp]
    e <- e[index.temp]

    if(length(a) > 0){
      R <- log(b,2) - log(a,2)
      I <- log(a,10) + log( b, 10)
      zero.index <- R == 0
      R <- R[!zero.index]
      I <- I[!zero.index]
      a <- a[!zero.index]
      b <- b[!zero.index]
      c <- c[!zero.index]
      d <- d[!zero.index]
      e <- e[!zero.index]
      if(!is.null(spot$Symdesc)){
        f <- f[!zero.index]
        Symdesc <- c(Symdesc, f, recursive = TRUE)
      }
      }
    tmp <- fitted(locfit(R~I,alpha = 0.1, deg = 1, ev=rbox(cut  = 0.9)))
    Cy5 <- c(Cy5, b * (1/2^tmp), recursive = TRUE)
    Cy3 <- c(Cy3, a, recursive = TRUE)
    BgCy3 <- c(BgCy3, c, recursive = TRUE)
    BgCy5 <- c(BgCy5, d, recursive = TRUE)
    Id <- c(Id, e, recursive = TRUE)
    j <- j + k + 1
  }
  Cy3 <- as.integer(Cy3)
  Cy5 <- as.integer(Cy5)
  BgCy5 <- as.integer(BgCy5)
  BgCy3 <- as.integer(BgCy3)
  if(is.null(spot$Symdesc)){
    SD <- list(Cy3 = Cy3, Cy5 = Cy5, BgCy3 = BgCy3, BgCy5 = BgCy5, Id = Id)
  }
  else{
    SD <- list(Cy3 = Cy3, Cy5 = Cy5, BgCy3 = BgCy3, BgCy5 = BgCy5, Id = Id, Symdesc = Symdesc)
  }
  new ("Spot", name = attr(mySpot, "name"), spotData = SD)
}

global.norm <- function(mySpot){
  spot <- attr(mySpot, "spotData")
  Cy3 <- Cy5 <- BgCy3 <- BgCy5<- Id <- Symdesc <- list()
  
  index.temp <- spot$Cy3 > 0 &  spot$Cy5 > 0
  a <- spot$Cy3[index.temp]
  b <- spot$Cy5[index.temp]
  c <- spot$BgCy3[index.temp]
  d <- spot$BgCy5[index.temp]
  e <- spot$Id[index.temp]

  if(!is.null(spot$Symdesc)){
    f <- spot$Symdesc[index.temp]
  }

  if(length(a) > 0){  
    R <- log(b,2) - log(a,2)
    I <- log(a,10) + log( b, 10)
    zero.index <- R == 0
    R <- R[!zero.index]
    I <- I[!zero.index]
    a <- a[!zero.index]
    b <- b[!zero.index]
    c <- c[!zero.index]
    d <- d[!zero.index]
    e <- e[!zero.index]
    if(!is.null(spot$Symdesc)){
      f <- f[!zero.index]
      Symdesc <- f
    }

    tmp <- fitted(locfit(R~I,alpha = 0.1, deg = 1, ev=rbox(cut  = 0.9) ))
    Cy5 <- b * (1/2^tmp)
    Cy3 <- a
    BgCy3 <- c
    BgCy5 <- d
    Id <- e
  }
  Cy3 <- as.integer(Cy3)
  Cy5 <- as.integer(Cy5)
  BgCy5 <- as.integer(BgCy5)
  BgCy3 <- as.integer(BgCy3)
  if(is.null(spot$Symdesc)){
    SD <- list(Cy3 = Cy3, Cy5 = Cy5, BgCy3 = BgCy3, BgCy5 = BgCy5, Id = Id)
  }
  else{
    SD <- list(Cy3 = Cy3, Cy5 = Cy5, BgCy3 = BgCy3, BgCy5 = BgCy5, Id = Id, Symdesc = Symdesc)
  }
  new ("Spot", name = attr(mySpot, "name"), spotData = SD)
}

# Duplicate Analysis

# Mean Replicate filtering 
meanUnique <- function(mySpot){
  spot <- attr(mySpot, "spotData")
  idHash <- new.env(hash=TRUE)
  n <- length(spot$Cy3)
  spot$Id <- toupper(spot$Id)
  for(i in 1:n){
    if(spot$Id[i] == ""){
      spot$Id[i] <- "EMPTY"
    }
    if(exists( spot$Id[i], envir = idHash)){
      idData <- get( spot$Id[i], envir = idHash)
      if(!is.null(spot$Symdesc)){
      assign(spot$Id[i], list(spotSymdesc = c(idData$spotSymdesc, spot$Symdesc[i], recursive = TRUE), spotCy3 = c( idData$spotCy3, spot$Cy3[i],recursive=TRUE) ,
                              spotCy5 = c( idData$spotCy5, spot$Cy5[i],recursive=TRUE),spotBgCy3 = c( idData$spotBgCy3, spot$BgCy3[i],recursive=TRUE),
                              spotBgCy5 = c( idData$spotBgCy5, spot$BgCy5[i],recursive=TRUE)), envir = idHash)
    }
      else{
         assign(spot$Id[i], list(spotCy3 = c( idData$spotCy3, spot$Cy3[i],recursive=TRUE) ,
                              spotCy5 = c( idData$spotCy5, spot$Cy5[i],recursive=TRUE),spotBgCy3 = c( idData$spotBgCy3, spot$BgCy3[i],recursive=TRUE),
                              spotBgCy5 = c( idData$spotBgCy5, spot$BgCy5[i],recursive=TRUE)), envir = idHash)
      }
    }
    
    else{
      if(!is.null(spot$Symdesc)){
        assign(spot$Id[i], list(spotSymdesc = spot$Symdesc[i], spotCy3 =  spot$Cy3[i] ,spotCy5 =  spot$Cy5[i],spotBgCy3 =  spot$BgCy3[i],
                                spotBgCy5 = spot$BgCy5[i]), envir = idHash)
    }
      else{
        assign(spot$Id[i], list(spotCy3 =  spot$Cy3[i] ,spotCy5 =  spot$Cy5[i],spotBgCy3 =  spot$BgCy3[i],
                                spotBgCy5 = spot$BgCy5[i]), envir = idHash) 
      }
    }
  }
  
  compara <- function(lista.cy3, lista.cy5){
    resultado <- TRUE
    R.list <- log2(lista.cy5)-log2(lista.cy3)
    n <- length(R.list)
    if(n > 1){
      for( i in 1:(length(R.list)-1)) {
        if((R.list[i] < 0 & R.list[i+1] > 0) | (R.list[i] > 0 & R.list[i+1] < 0)){
          return( FALSE )
        }
        
        if(abs(R.list[i]) > (abs(R.list[i+1]) * 1.2)){
          resultado <- FALSE
        }
      }
    }    
    resultado
  }
  ids <- ls(envir = idHash)
  replicates <- function( id ){
    mainList <- get(id, envir = idHash)
    if(compara(as.numeric(mainList$spotCy3), as.numeric(mainList$spotCy5)))
      if(!is.null(spot$Symdesc))
        list(Id = id, Symdesc = unique(mainList$spotSymdesc), Cy3 = mean(mainList$spotCy3), Cy5 = mean(mainList$spotCy5), BgCy3 = mainList$spotBgCy3[length(mainList$spotBgCy3)], BgCy5 = mainList$spotBgCy5[length(mainList$spotBgCy3)])
      else
        list(Id = id, Cy3 = mean(mainList$spotCy3), Cy5 = mean(mainList$spotCy5), BgCy3 = mainList$spotBgCy3[length(mainList$spotBgCy3)], BgCy5 = mainList$spotBgCy5[length(mainList$spotBgCy3)])
  }
  restCy3 <- restCy5 <- restBgCy3 <- restBgCy5 <- restId <- restSymdesc <- list()
  for(i in 1:length(ids)){
    rest <- replicates(ids[i])
    restCy3 <- c(restCy3,rest$Cy3,recursive=TRUE)
    restCy5 <- c(restCy5,rest$Cy5,recursive=TRUE)
    restBgCy3 <- c(restBgCy3,rest$BgCy3,recursive=TRUE)
    restBgCy5 <- c(restBgCy5,rest$BgCy5,recursive=TRUE)
    restId <- c(restId,rest$Id,recursive=TRUE)
    if(!is.null(spot$Symdesc))
      restSymdesc <- c(restSymdesc,rest$Symdesc,recursive=TRUE)
  }
    if(!is.null(spot$Symdesc))
      SD <- list(Cy3 = restCy3, Cy5 = restCy5, BgCy3 = restBgCy3, BgCy5 = restBgCy5, Id = restId, Symdesc = restSymdesc)
    else
      SD <- list(Cy3 = restCy3, Cy5 = restCy5, BgCy3 = restBgCy3, BgCy5 = restBgCy5, Id = restId)
    spotName <- attr(mySpot, "name")
    new("Spot", name = spotName, spotData = SD)
}

                                        # Non-extreme values replicate filtering

alter.unique <- function(mySpot){
  spot <- attr(mySpot, "spotData")
  myHash <- new.env(hash=TRUE)
  n <- length(spot$Cy3)
  spot$Id <- toupper(spot$Id)
  for(i in 1:n){
    if(spot$Id[i] == ""){
      spot$Id[i] <- "EMPTY"
    }
    if(exists( spot$Id[i], envir = myHash)){
      idData <- get( spot$Id[i], envir = myHash)
      if(!is.null(spot$Symdesc)){
        assign(spot$Id[i], list(spotSymdesc = c(idData$spotSymdesc, spot$Symdesc[i], recursive = TRUE), spotCy3 = c( idData$spotCy3, spot$Cy3[i],recursive=TRUE) ,
                                spotCy5 = c( idData$spotCy5, spot$Cy5[i],recursive=TRUE),spotBgCy3 = c( idData$spotBgCy3, spot$BgCy3[i],recursive=TRUE),
                                spotBgCy5 = c( idData$spotBgCy5, spot$BgCy5[i],recursive=TRUE)), envir = myHash)
      }
      else{
        assign(spot$Id[i], list(spotCy3 = c( idData$spotCy3, spot$Cy3[i]) ,
                                spotCy5 = c( idData$spotCy5, spot$Cy5[i]),spotBgCy3 = c( idData$spotBgCy3, spot$BgCy3[i]),
                                spotBgCy5 = c( idData$spotBgCy5, spot$BgCy5[i])), envir = myHash)
      }
    }    
    else{
      if(!is.null(spot$Symdesc)){
      assign(spot$Id[i], list(spotSymdesc = spot$Symdesc[i], spotCy3 =  spot$Cy3[i] ,spotCy5 =  spot$Cy5[i],spotBgCy3 =  spot$BgCy3[i],
                              spotBgCy5 = spot$BgCy5[i]), envir = myHash)
    }
      else{
        assign(spot$Id[i], list(spotCy3 =  spot$Cy3[i] ,spotCy5 =  spot$Cy5[i],spotBgCy3 =  spot$BgCy3[i],
                                spotBgCy5 = spot$BgCy5[i]), envir = myHash) 
      }
    }
  }
  
  ids <- ls(envir = myHash)
  Cy3.list <- Cy5.list <- BgCy3.list <- BgCy5.list <- Id.list <- Symdesc.list <- list()
  Cy3s <- Cy5s <- BgCy3s <- BgCy5s <- Ids <- Symdescs <- list()
  
  replicates2 <- function(id){
    mainList <- get(id, envir = myHash)
    Cy3.list <- mainList$spotCy3
    Cy5.list <- mainList$spotCy5
    BgCy3.list <- mainList$spotBgCy3
    BgCy5.list <- mainList$spotBgCy5
    if(!is.null(spot$Symdesc))
      Symdesc.list <- mainList$spotSymdesc
    R.list <- log2(Cy5.list)-log2(Cy3.list)
    x.prueba <- R.list[R.list < 0]
    Cy3s <- Cy5s <- BgCy3s <- BgCy5s <- ids <- Symdescs <- list()
    if(length(x.prueba) == 0 || length(x.prueba) == length(Cy3.list)){
      if(length(x.prueba) == 0){ # all positive
        bueno.index <- as.numeric(which(R.list == max(R.list)))
      }
      else{# all negative
        bueno.index <- as.numeric(which(R.list == min(R.list)))
      }
      Cy3s <- Cy3.list[bueno.index]
      Cy5s <- Cy5.list[bueno.index]
      BgCy3s <- BgCy3.list[bueno.index]
      BgCy5s <- BgCy5.list[bueno.index]
      if(!is.null(spot$Symdesc))
        Symdescs <- Symdesc.list[bueno.index]
      ids <- id
      if(!is.null(spot$Symdesc))
        list(Cy3 = Cy3s[1], Cy5 = Cy5s[1], BgCy3 = BgCy3s[1], BgCy5 = BgCy5s[1], Id = ids, Symdesc = Symdescs[1])
      else
        list(Cy3 = Cy3s[1], Cy5 = Cy5s[1], BgCy3 = BgCy3s[1], BgCy5 = BgCy5s[1], Id = ids)
    }
  }
  
  restCy3 <- restCy5 <- restBgCy3 <- restBgCy5 <- restId <- restSymdesc <- list()
  for(i in 1:length(ids)){
    rest <- replicates2(ids[i])
    restCy3 <- c(restCy3,rest$Cy3,recursive=TRUE)
    restCy5 <- c(restCy5,rest$Cy5,recursive=TRUE)
    restBgCy3 <- c(restBgCy3,rest$BgCy3,recursive=TRUE)
    restBgCy5 <- c(restBgCy5,rest$BgCy5,recursive=TRUE)
    restId <- c(restId,rest$Id,recursive=TRUE)
    if(!is.null(spot$Symdesc)){
      restSymdesc <- c(restSymdesc,rest$Symdesc,recursive=TRUE)
    }
  }
    if(!is.null(spot$Symdesc))
      SD <- list(Cy3 = restCy3, Cy5 = restCy5, BgCy3 = restBgCy3, BgCy5 = restBgCy5, Id = restId, Symdesc = restSymdesc)
    else
      SD <- list(Cy3 = restCy3, Cy5 = restCy5, BgCy3 = restBgCy3, BgCy5 = restBgCy5, Id = restId)
    spotName <- attr(mySpot, "name")
    new("Spot", name = spotName, spotData = SD)
}

# Remove replicated observations, geometric mean
spotUnique <- function(mySpot){
  spot <- attr(mySpot, "spotData")
  idHash <- new.env(hash=TRUE)
  n <- length(spot$Cy3)
  spot$Id <- toupper(spot$Id)
  for(i in 1:n){
    if(spot$Id[i] == ""){
      spot$Id[i] <- "EMPTY"
    }
    if(exists( spot$Id[i], envir = idHash)){
      idData <- get( spot$Id[i], envir = idHash)
      if(!is.null(spot$Symdesc)){
      assign(spot$Id[i], list(spotSymdesc = c(idData$spotSymdesc, spot$Symdesc[i], recursive = TRUE), spotCy3 = c( idData$spotCy3, spot$Cy3[i],recursive=TRUE) ,
                              spotCy5 = c( idData$spotCy5, spot$Cy5[i],recursive=TRUE),spotBgCy3 = c( idData$spotBgCy3, spot$BgCy3[i],recursive=TRUE),
                              spotBgCy5 = c( idData$spotBgCy5, spot$BgCy5[i],recursive=TRUE)), envir = idHash)
    }
      else{
        assign(spot$Id[i], list(spotCy3 = c( idData$spotCy3, spot$Cy3[i],recursive=TRUE) ,
                                spotCy5 = c( idData$spotCy5, spot$Cy5[i],recursive=TRUE),spotBgCy3 = c( idData$spotBgCy3, spot$BgCy3[i],recursive=TRUE),
                                spotBgCy5 = c( idData$spotBgCy5, spot$BgCy5[i],recursive=TRUE)), envir = idHash)
      }
    }
    
    else{
      if(!is.null(spot$Symdesc)){
      assign(spot$Id[i], list(spotSymdesc = spot$Symdesc[i], spotCy3 =  spot$Cy3[i] ,spotCy5 =  spot$Cy5[i],spotBgCy3 =  spot$BgCy3[i],
                              spotBgCy5 = spot$BgCy5[i]), envir = idHash)
    }
      else{
        assign(spot$Id[i], list(spotCy3 =  spot$Cy3[i] ,spotCy5 =  spot$Cy5[i],spotBgCy3 =  spot$BgCy3[i],
                                spotBgCy5 = spot$BgCy5[i]), envir = idHash)
      }
  }
  }
  
  
  geometric.mean <-  function(x){
    prod(x^(1/length(x)))
  }
  
  ids <- ls(envir = idHash)
  
  replicates3 <- function(id){
    mainList <- get(id, envir = idHash)
    Cy3.list <- mainList$spotCy3
    Cy5.list <- mainList$spotCy5
    BgCy3.list <- mainList$spotBgCy3
    BgCy5.list <- mainList$spotBgCy5
    Symdesc.list <- mainList$spotSymdesc
    Cy3 <- Cy5 <- BgCy3 <- BgCy5 <- Id <- Symdesc <- list()
    tmp1 <- geometric.mean(Cy3.list)
    tmp2 <- geometric.mean(Cy5.list)
    if(tmp1 != 0 && tmp2 != 0){
      Cy3 <- c(Cy3, round(tmp1,1), recursive=TRUE)
      Cy5 <- c(Cy5, round(tmp2,1), recursive=TRUE)
      BgCy3 <- c(BgCy3, BgCy3.list[length(mainList$spotBgCy3)],recursive=TRUE)
      BgCy5 <- c(BgCy5, BgCy5.list[length(mainList$spotBgCy5)],recursive=TRUE)
      Id <- c(Id, id, recursive=TRUE)
      Symdesc <- c(Symdesc, Symdesc.list[1], recursive=TRUE)
    }
    list(Cy3 = Cy3, Cy5 = Cy5, BgCy3 = BgCy3, BgCy5 = BgCy5, Id = Id, Symdesc = Symdesc)
  }
  
  restCy3 <- restCy5 <- restBgCy3 <- restBgCy5 <- restId <- restSymdesc <- list()
  for(i in 1:length(ids)){
    rest <- replicates3(ids[i])
    restCy3 <- c(restCy3,rest$Cy3,recursive=TRUE)
    restCy5 <- c(restCy5,rest$Cy5,recursive=TRUE)
    restBgCy3 <- c(restBgCy3,rest$BgCy3,recursive=TRUE)
    restBgCy5 <- c(restBgCy5,rest$BgCy5,recursive=TRUE)
    restId <- c(restId,rest$Id,recursive=TRUE)
    if(!is.null(spot$Symdesc))
      restSymdesc <- c(restSymdesc, rest$Symdesc, recursive=TRUE)
  }
  if(!is.null(spot$Symdesc))
    SD <- list(Cy3 = restCy3, Cy5 = restCy5, BgCy3 = restBgCy3, BgCy5 = restBgCy5, Id = restId, Symdesc = restSymdesc)
  else
    SD <- list(Cy3 = restCy3, Cy5 = restCy5, BgCy3 = restBgCy3, BgCy5 = restBgCy5, Id = restId)

  spotName <- attr(mySpot, "name")
  new("Spot", name = spotName, spotData = SD)
}

make.swap <- function(spot1, spot2, Cy3, Cy5, BgCy3, BgCy5, Id, Symdesc=NULL, header = FALSE, is.ifc = FALSE,envir,nr,nc){
  if(is.null(Symdesc)){
    spot.ori <- read.spot(spot1,Cy3,Cy5,BgCy3,BgCy5,Id, header = header,is.ifc = is.ifc,envir=envir)
    spot.swap <- read.spot(spot2,Cy3,Cy5,BgCy3,BgCy5,Id, header = header,is.ifc = is.ifc,envir=envir)
  }
  else{
    spot.ori <- read.spot(spot1,Cy3,Cy5,BgCy3,BgCy5,Id, Symdesc, header = header,is.ifc = is.ifc,envir=envir)
    spot.swap <- read.spot(spot2,Cy3,Cy5,BgCy3,BgCy5,Id, Symdesc, header = header,is.ifc = is.ifc,envir=envir)
  }
  if(is.ifc){
    nr <- get("nr.global",envir=envir)
    nc <- get("nc.global",envir=envir)
    nmr <- get("nmr.global",envir=envir)
    nmc <- get("nmc.global",envir=envir)
  }
  ori.c <- bg.correct(spot.ori)
  swap.c <- bg.correct(spot.swap)
  assign("ori.c", ori.c, envir = envir)
  assign("swap.c", swap.c, envir = envir)
  spot1.data <- attr(ori.c,"spotData")
  spot2.data <- attr(swap.c,"spotData")
  
  A <- log2(spot1.data$Cy5*spot2.data$Cy5)-log2(spot1.data$Cy3*spot2.data$Cy3)
  index <- spot1.data$Cy5 == 0 | spot2.data$Cy5 == 0 | spot1.data$Cy3 == 0 | spot2.data$Cy3 == 0
  A[index] <- 0
  A.sd <- sd(A,na.rm=TRUE)
  A.mean <- mean(A,na.rm=TRUE)
  cond <- A <= A.mean - (A.sd*2) | A >= A.mean + (A.sd*2)
  assign("cond.index",cond,envir = envir)
  conf.factor <- length(A[!cond])/length(A)
  assign("conf",paste("Confidence Factor: ",round(conf.factor*100,digits=2),sep=""),envir=envir)
  disp.factor <- length(A[cond])/length(A)
  assign("disp",paste("Dispersion Factor: ",round(disp.factor*100,digits=2),sep=""),envir=envir)
  x <- spot1.data$Cy3[!cond]*spot2.data$Cy5[!cond]
  y <- spot1.data$Cy5[!cond]*spot2.data$Cy3[!cond]
  z <- spot1.data$BgCy3[!cond]*spot2.data$BgCy5[!cond]
  w <- spot1.data$BgCy5[!cond]*spot2.data$BgCy3[!cond]
  R.file1 <- log2(spot1.data$Cy5)-log2(spot1.data$Cy3)
  R.file2 <- log2(spot2.data$Cy3)-log2(spot2.data$Cy5)
  assign("R1",R.file1,envir = envir)
  assign("R2",R.file2,envir = envir)
  geometric.mean <-  function(x){sqrt(x)}
  Cy3.new <- sapply(x, geometric.mean)
  Cy5.new <- sapply(y, geometric.mean)
  BgCy3.new <- sapply(z, geometric.mean)
  BgCy5.new <- sapply(w, geometric.mean)
  if(is.null(Symdesc))
    SD <- list(Cy3 = as.numeric(Cy3.new),Cy5 = as.numeric(Cy5.new),BgCy3 = as.numeric(BgCy3.new),BgCy5 = as.numeric(BgCy5.new),Id = as.vector(spot1.data$Id[!cond]))
  else
    SD <- list(Cy3 = as.numeric(Cy3.new),Cy5 = as.numeric(Cy5.new),BgCy3 = as.numeric(BgCy3.new),BgCy5 = as.numeric(BgCy5.new),Id = as.vector(spot1.data$Id[!cond]), Symdesc = as.vector(spot1.data$Symdesc[!cond]))
  assign("o.spot", new ("Spot", name = paste(attr(spot.ori, "name"),"vs",attr(spot.swap, "name"),sep=""), spotData = SD), envir = envir)
  write.spot(get("o.spot",envir=envir),"original")
  assign("a.spot",get("o.spot",envir=envir), envir = envir)
  assign("spot.name",paste(attr(spot.ori, "name"),"vs",attr(spot.swap, "name"),sep=""), envir = envir)
  assign("name.project", paste(attr(spot.ori, "name"),"vs",attr(spot.swap, "name"),sep=""), envir = envir)
}

single.norm <- function(envir){
  norm.alg <- get("norm.alg",envir = envir)
  ori.c <- get("ori.c",envir = envir)
  swap.c <- get("swap.c",envir = envir)
  nr <- get("nr.global",envir=envir)
  nc <- get("nc.global",envir=envir)
  switch(norm.alg,
         {ori.n <- grid.norm(ori.c,nr,nc)
          swap.n <- grid.norm(swap.c,nr,nc)},
         {ori.n <- global.norm(ori.c)
          swap.n <- global.norm(swap.c)})
  ori.f <- filter.spot(ori.n)
  swap.f <- filter.spot(swap.n)
  unique.alg <- get("unique.alg",envir = envir)
  switch(unique.alg,
         {ori.u <-  meanUnique(ori.f)
          swap.u <- meanUnique(swap.f)},
         {ori.u <- alter.unique(ori.f)
          swap.u <- alter.unique(swap.f)},
         {ori.u <- spotUnique(ori.f)
          swap.u <- spotUnique(swap.f)})
  ori.dif <- Zscore(ori.u)
  swap.dif <- Zscore(swap.u)
  assign("ori.dif",ori.dif, envir = envir)
  assign("swap.dif",swap.dif, envir = envir)
}

get.Zscore <- function( spot, name, Zscore.min=NULL, Zscore.max=NULL, all=FALSE, envir) {
  idHash <- new.env(hash=TRUE)
  tmp <-  unlist(strsplit(name, "\\."))
  name <- paste(tmp[-length(tmp)],sep=".",collapse=".")
  ext <- tmp[length(tmp)]
  zscore.ori <- attr(get("ori.dif",envir = envir), "dataSets")
  zscore.swap <- attr(get("swap.dif", envir = envir), "dataSets")
  zscore <- attr(spot, "dataSets")
  if(is.null(Zscore.min) && is.null(Zscore.max))
    stop("At least one range value must be different than null")
  if(is.null(Zscore.min)){
    if(!is.numeric(Zscore.max))
      stop("Zscore.max must be numeric")
    if(Zscore.max < 0 )
      stop("Zscore.max must be positive integer")
    Up.index <- abs(zscore$Zscore) <= Zscore.max
  }else{
    if(is.null(Zscore.max)){
      if(!is.numeric(Zscore.min))
        stop("Zscore.max must be numeric")
      if(Zscore.min < 0 )
        stop("Zscore.max must be positive integer")
      Up.index <-  zscore$Zscore > Zscore.min
      Down.index <- zscore$Zscore < (-Zscore.min)
    }else{
      if(Zscore.min > Zscore.max)
        stop("Invalid ranges")
      Up.index <-  (zscore$Zscore > Zscore.min) & (zscore$Zscore < Zscore.max)
      Down.index <- (zscore$Zscore < (-Zscore.min)) & (zscore$Zscore > (-Zscore.max))
    }
  }
  n <- length(zscore.ori$Id)
  zscore.swap.length <- length(zscore.swap$Id)
  zscoreIdUp <- zscore$Id[Up.index]
  zscoreZscoreUp <- zscore$Zscore[Up.index]
  if(!is.null(zscore$Symdesc)){
    zscoreSymUp <- zscore$Symdesc[Up.index]
    zscoreSymDown <- zscore$Symdesc[Down.index]
  }
  up.length <- length(zscoreIdUp)
  zscoreIdDown <- zscore$Id[Down.index]
  zscoreZscoreDown <- zscore$Zscore[Down.index]
  
  for(i in 1:n){
    if(!is.null(zscore.ori$Symdesc))
      assign(paste(zscore.ori$Id[i],zscore.ori$Symdesc[i],sep="\t"), list(ZscoreUp ="null" ,ZscoreDown = "null", Zscore.ori = zscore.ori$Zscore[i] , Zscore.swap =0),envir=idHash)
    else
      assign(zscore.ori$Id[i], list(ZscoreUp ="null" ,ZscoreDown = "null", Zscore.ori = zscore.ori$Zscore[i] , Zscore.swap =0),envir=idHash)
  }
  
  for(i in 1:zscore.swap.length){
    if(!is.null(zscore.ori$Symdesc)){
      if(exists(paste(zscore.swap$Id[i],zscore.swap$Symdesc[i],sep="\t"), envir = idHash)){
        idData <- get( paste(zscore.swap$Id[i],zscore.swap$Symdesc[i],sep="\t"), envir = idHash)
        assign(paste(zscore.swap$Id[i],zscore.swap$Symdesc[i],sep="\t"), list(ZscoreUp = idData$ZscoreUp, ZscoreDown= idData$ZscoreDown, Zscore.ori = idData$Zscore.ori, Zscore.swap =zscore.swap$Zscore[i]),envir=idHash)}
    }
    else{
      if(exists(zscore.swap$Id[i], envir = idHash)){
        idData <- get(zscore.swap$Id[i], envir = idHash)
        assign(zscore.swap$Id[i], list(ZscoreUp = idData$ZscoreUp, ZscoreDown= idData$ZscoreDown, Zscore.ori = idData$Zscore.ori, Zscore.swap =zscore.swap$Zscore[i]),envir=idHash)}
    }
  }

  for(i in 1:up.length){
    if(is.null(zscore$Symdesc)){
      if(exists( zscoreIdUp[i], envir = idHash)){
        idData <- get( zscoreIdUp[i], envir = idHash)
        assign(zscoreIdUp[i], list(ZscoreUp =zscoreZscoreUp[i], ZscoreDown = idData$ZscoreDown, Zscore.ori = idData$Zscore.ori, Zscore.swap = idData$Zscore.swap),envir=idHash)}
    }
    else{
      if(exists(paste(zscoreIdUp[i],zscoreSymUp[i],sep="\t"), envir = idHash)){
        idData <- get(paste(zscoreIdUp[i],zscoreSymUp[i],sep="\t"), envir = idHash)
        assign(paste(zscoreIdUp[i],zscoreSymUp[i],sep="\t"), list(ZscoreUp =zscoreZscoreUp[i], ZscoreDown = idData$ZscoreDown, Zscore.ori = idData$Zscore.ori, Zscore.swap = idData$Zscore.swap),envir=idHash)}   }
  }
  
  down.length <- length(zscoreIdDown)
  for(i in 1:down.length){
    if(is.null(zscore$Symdesc)){
       if(exists( zscoreIdDown[i], envir = idHash)){
        idData <- get( zscoreIdDown[i], envir = idHash)
        assign(zscoreIdDown[i], list(ZscoreUp = idData$ZscoreUp, ZscoreDown = zscoreZscoreDown[i], Zscore.ori = idData$Zscore.ori, Zscore.swap = idData$Zscore.swap),envir=idHash)}
     }
    else{
      if(exists(paste(zscoreIdDown[i],zscoreSymDown[i],sep="\t"), envir = idHash)){
      idData <- get(paste(zscoreIdDown[i],zscoreSymDown[i],sep="\t"), envir = idHash)
        assign(paste(zscoreIdDown[i],zscoreSymDown[i],sep="\t"), list(ZscoreUp = idData$ZscoreUp, ZscoreDown = zscoreZscoreDown[i], Zscore.ori = idData$Zscore.ori, Zscore.swap = idData$Zscore.swap),envir=idHash)}
    }
  }
  
  get.finalZ  <- function(id,range){
    zScore <- get(id, envir = idHash)
    if(range == "up"){
      if( zScore$ZscoreUp !="null"){
        if(!is.null(zscore$Symdesc)){
          id2 <- unlist(strsplit(id,split="\t"))[1]
          sym <-  unlist(strsplit(id,split="\t"))[2]
          list(Id=id2, Symdesc=sym, Zscore=zScore$ZscoreUp, Zscore.ori=zScore$Zscore.ori,Zscore.swap=zScore$Zscore.swap)
        }
        else
          list(Id=id, Zscore=zScore$ZscoreUp, Zscore.ori=zScore$Zscore.ori,Zscore.swap=zScore$Zscore.swap)
      }
    }else{
      if( zScore$ZscoreDown !="null"){
        if(!is.null(zscore$Symdesc)){
          id2 <- unlist(strsplit(id,split="\t"))[1]
          sym <-  unlist(strsplit(id,split="\t"))[2]
          list(Id=id2, Symdesc=sym, Zscore=zScore$ZscoreDown, Zscore.ori=zScore$Zscore.ori,Zscore.swap=zScore$Zscore.swap)
        }
        else
          list(Id=id, Zscore=zScore$ZscoreDown, Zscore.ori=zScore$Zscore.ori,Zscore.swap=zScore$Zscore.swap)
      }
    }
  }
  
  if(all){
    if(file.exists(paste(name,".",ext, sep="")))
      file.remove(paste(name,".",ext, sep=""))
    if(is.null(zscore.ori$Symdesc))
      write.table("Id\tSwapped Zscore\tZscore File1\tZscore File2", paste(name,"_.",ext, sep=""),col.names=FALSE,row.names=FALSE,quote=FALSE,sep="\t",append=FALSE)
    else
      write.table("Id\tSymbol or description\tSwapped Zscore\tZscore File1\tZscore File2", paste(name,"_.",ext, sep=""),col.names=FALSE,row.names=FALSE,quote=FALSE,sep="\t",append=FALSE)
    for(i in 1:n){
      if(!is.null(zscore.ori$Symdesc))
        id <- paste(zscore.ori$Id[i],zscore.ori$Symdesc[i],sep="\t")
      else
        id <- zscore.ori$Id[i]
      write.table(get.finalZ(id,"up"), paste(name,"_All.",ext, sep=""),col.names=FALSE,row.names=FALSE,quote=FALSE,sep="\t",append=TRUE)
      write.table(get.finalZ(id,"down"), paste(name,"_All.", ext, sep=""),col.names=FALSE,row.names=FALSE,quote=FALSE,sep="\t",append=TRUE)
    }
  }
  else{
    if(file.exists(paste(name,"_Up.", ext, sep="")))
      file.remove(paste(name,"_Up.", ext, sep=""))
    if(file.exists(paste(name,"_Down.", ext, sep="")))
      file.remove(paste(name,"_Down.", ext, sep=""))
    if(is.null(zscore.ori$Symdesc)){
      write.table("Id\tSwapped Zscore\tZscore File1\tZscore File2", paste(name,"_Up.",ext, sep=""),col.names=FALSE,row.names=FALSE,quote=FALSE,sep="\t",append=FALSE)
      write.table("Id\tSwapped Zscore\tZscore File1\tZscore File2", paste(name,"_Down.",ext, sep=""),col.names=FALSE,row.names=FALSE,quote=FALSE,sep="\t",append=FALSE)
    }
    else{
      write.table("Id\tSymbol or description\tSwapped Zscore\tZscore File1\tZscore File2", paste(name,"_Up.",ext, sep=""),col.names=FALSE,row.names=FALSE,quote=FALSE,sep="\t",append=FALSE)
      write.table("Id\tSymbol or description\tSwapped Zscore\tZscore File1\tZscore File2", paste(name,"_Down.",ext, sep=""),col.names=FALSE,row.names=FALSE,quote=FALSE,sep="\t",append=FALSE)
    }
    for(i in 1:n){
      if(!is.null(zscore.ori$Symdesc))
        id <- paste(zscore.ori$Id[i],zscore.ori$Symdesc[i],sep="\t")
      else
        id <- zscore.ori$Id[i]
      write.table(get.finalZ(id,"up"), paste(name,"_Up.", ext, sep=""),col.names=FALSE,row.names=FALSE,quote=FALSE,sep="\t",append=TRUE)
    }
    
    for(i in 1:n){
      if(!is.null(zscore.ori$Symdesc))
        id <- paste(zscore.ori$Id[i],zscore.ori$Symdesc[i],sep="\t")
      else
        id <- zscore.ori$Id[i]
      write.table(get.finalZ(id,"down"), paste(name,"_Down.", ext, sep=""),col.names=FALSE,row.names=FALSE,quote=FALSE,sep="\t",append=TRUE)
    }
  }  
}

# GenArise

# Write the spot to a file in txt format

write.spot <- function( spot, fileName, quote=FALSE,sep="\t",col.names=FALSE, row.names=FALSE ) {
  spotData<-attr(spot,"spotData")
  spotData$Cy3 <- format(spotData$Cy3,justify="left",trim=TRUE)
  spotData$Cy5 <- format(spotData$Cy5,justify="left",trim=TRUE)
  spotData$BgCy3 <- format(spotData$BgCy3,justify="left",trim=TRUE)
  spotData$BgCy5 <- format(spotData$BgCy5,justify="left",trim=TRUE)
  if(is.null(spotData$Symdesc))
    write.table("Cy3\tCy5\tBgCy3\tBgCy5\tId", fileName, quote = quote,sep = sep, col.names = col.names, row.names = row.names )
  else
    write.table("Cy3\tCy5\tBgCy3\tBgCy5\tId\tSymbol", fileName, quote = quote,sep = sep, col.names = col.names, row.names = row.names )
  write.table(spotData, fileName, quote = quote,sep = sep, col.names = col.names, row.names = row.names, append=TRUE )
}

write.dataSet <- function(dataSet.spot, fileName, quote = FALSE, col.names=FALSE, row.names=FALSE, Zscore.min = NULL, Zscore.max = NULL, sep = "\t"){
  tmp <-  unlist(strsplit(fileName, "\\."))
  fileName <- paste(tmp[-length(tmp)],sep=".",collapse=".")
  ext <- tmp[length(tmp)]
  
  if(is.null(Zscore.min) && is.null(Zscore.max))
    stop("At least one range value must be different than null")
  data.values <- attr(dataSet.spot, "dataSets")
  if(is.null(Zscore.min)){
    if(!is.numeric(Zscore.max))
      stop("Zscore.max must be numeric")
    if(Zscore.max < 0 )
      stop("Zscore.max must be positive integer")
    index <- abs(data.values$Zscore) <= Zscore.max
    if(is.null(data.values$Symdesc)){
      Center <- list(Cy3 = data.values$Cy3[index], Cy5 = data.values$Cy5[index],
                     Id = data.values$Id[index], Zscore = data.values$Zscore[index])
      write.table("Cy3\tCy5\tId\tZscore", paste(fileName,"_Center.", ext, sep = "") , quote = quote,sep = sep, col.names = col.names, row.names = row.names )
    }
    else{
      Center <- list(Cy3 = data.values$Cy3[index], Cy5 = data.values$Cy5[index],
                     Id = data.values$Id[index], Symdesc = data.values$Symdesc[index], Zscore = data.values$Zscore[index])
      write.table("Cy3\tCy5\tId\tSymbol\tZscore", paste(fileName,"_Center.", ext, sep = "") , quote = quote,sep = sep, col.names = col.names, row.names = row.names )
    }
    write.table(Center, paste(fileName,"_Center.", ext, sep = ""), quote = quote, sep = sep, col.names = col.names, row.names = row.names, append = TRUE)
  }else{
    if(is.null(data.values$Symdesc)){
      write.table("Cy3\tCy5\tId\tZscore", paste(fileName,"_Up.", ext, sep = "") , quote = quote,sep = sep, col.names = col.names, row.names = row.names )
      write.table("Cy3\tCy5\tId\tZscore", paste(fileName,"_Down.", ext, sep = "") , quote = quote,sep = sep, col.names = col.names, row.names = row.names )
    }
    else{
      write.table("Cy3\tCy5\tId\tSymbol\tZscore", paste(fileName,"_Up.", ext, sep = "") , quote = quote,sep = sep, col.names = col.names, row.names = row.names )
      write.table("Cy3\tCy5\tId\tSymbol\tZscore", paste(fileName,"_Down.", ext, sep = "") , quote = quote,sep = sep, col.names = col.names, row.names = row.names )
    }
     if(is.null(Zscore.max)){
      if(!is.numeric(Zscore.min))
        stop("Zscore.max must be numeric")
      if(Zscore.min < 0 )
        stop("Zscore.max must be positive integer")
      Up.index <-  data.values$Zscore > Zscore.min
      Down.index <- data.values$Zscore < (-Zscore.min)
      if(is.null(data.values$Symdesc)){
        Up <- list(Cy3 = data.values$Cy3[Up.index], Cy5 = data.values$Cy5[Up.index],
                   Id = data.values$Id[Up.index], Zscore = data.values$Zscore[Up.index])
        Down <- list(Cy3 = data.values$Cy3[Down.index], Cy5 = data.values$Cy5[Down.index],
                     Id = data.values$Id[Down.index], Zscore = data.values$Zscore[Down.index])
      }else{
        Up <- list(Cy3 = data.values$Cy3[Up.index], Cy5 = data.values$Cy5[Up.index],
                   Id = data.values$Id[Up.index], Symdesc = data.values$Symdesc[Up.index], Zscore = data.values$Zscore[Up.index])
        Down <- list(Cy3 = data.values$Cy3[Down.index], Cy5 = data.values$Cy5[Down.index],
                     Id = data.values$Id[Down.index], Symdesc = data.values$Symdesc[Down.index], Zscore = data.values$Zscore[Down.index])
      }
      write.table(Up, paste(fileName,"_Up.", ext, sep = ""), quote = quote, sep = sep, col.names = col.names, row.names = row.names,append=TRUE)
     
      write.table(Down, paste(fileName,"_Down.", ext, sep = ""), quote = quote, sep = sep, col.names = col.names, row.names = row.names,append=TRUE)
    }else{
      if(Zscore.min > Zscore.max)
        stop("Invalid ranges")
      Up.index <-  (data.values$Zscore > Zscore.min) & (data.values$Zscore < Zscore.max)
      Down.index <- (data.values$Zscore < (-Zscore.min)) & (data.values$Zscore > (-Zscore.max))
      if(is.null(data.values$Symdesc)){
        Up <- list(Cy3 = data.values$Cy3[Up.index], Cy5 = data.values$Cy5[Up.index],
                   Id = data.values$Id[Up.index], Zscore = data.values$Zscore[Up.index])
        Down <- list(Cy3 = data.values$Cy3[Down.index], Cy5 = data.values$Cy5[Down.index],
                     Id = data.values$Id[Down.index], Zscore = data.values$Zscore[Down.index])
      }
      else{
        Up <- list(Cy3 = data.values$Cy3[Up.index], Cy5 = data.values$Cy5[Up.index],
                   Id = data.values$Id[Up.index], Symdesc = data.values$Symdesc[Up.index], Zscore = data.values$Zscore[Up.index])
        Down <- list(Cy3 = data.values$Cy3[Down.index], Cy5 = data.values$Cy5[Down.index],
                     Id = data.values$Id[Down.index], Symdesc = data.values$Symdesc[Down.index], Zscore = data.values$Zscore[Down.index])
      }
      write.table(Up, paste(fileName,"_Up.", ext, sep = ""), quote = quote, sep = sep, col.names = col.names, row.names = row.names,append=TRUE)
      write.table(Down, paste(fileName,"_Down.", ext, sep = ""), quote = quote, sep = sep, col.names = col.names, row.names = row.names, append=TRUE)
    }
  }
}

write.zscore <- function(dataSet.spot, fileName, sep = "\t"){
  data.values <- attr(dataSet.spot, "dataSets")
  type <- attr(dataSet.spot, "type")
  if(is.null(data.values$Symdesc))
    zscore <- list(Cy3 = data.values$Cy3, Cy5 = data.values$Cy5,
                   Id = data.values$Id, Zscore = data.values$Zscore)
  else
    zscore <- list(Cy3 = data.values$Cy3, Cy5 = data.values$Cy5,
                   Id = data.values$Id, Symdesc = data.values$Symdesc, Zscore = data.values$Zscore)
  unlink(fileName)
  object <- file(fileName,"w")
  cat("\t\t\t",type,"\n",file=fileName)
  write.table(zscore, fileName, quote = FALSE, sep = sep, col.names = FALSE, row.names = FALSE, append=TRUE)
  close(object)
}

# GenArise

# Read all file, but only extract the interested columns and create a Spot object
# The numnber of columns are the IFC format 

read.spot <- function( file.name, cy3, cy5, bg.cy3, bg.cy5, ids, symdesc, header = FALSE, sep = "\t", is.ifc = FALSE,envir){
  if(missing(envir)){
    envir <- globalenv()
  }
  tmp <- unlist(strsplit(file.name, "\\/"))
  spot.name <- unlist(strsplit(tmp[length(tmp)], "\\."))[1]
  if(is.ifc){
    temp <- read.csv(file.name, header = FALSE, sep = "\t", comment.char = "", quote="")
    temp <- temp[-1,] # we delete the header
    row1 <- temp[,1]
    flag <- which(rev(row1) == "Mean value") # regresa un solo valor
    empty.row <- length(row1[row1==""])
    total.rows <-  length(temp[,1])
    temp <- temp[-((total.rows-(flag-1 + empty.row)):total.rows),]
    total.rows <- length(temp[,1])
    temp <- apply(temp, 2, as.vector)
    ans <- paste(max(as.integer(temp[,2]),na.rm=TRUE), max(as.integer(temp[,3]),na.rm=TRUE),
                 max(as.integer(temp[,4]),na.rm=TRUE), max(as.integer(temp[,5]),na.rm=TRUE), sep = " ")
#    print(paste("The dimensions are ", max(as.integer(temp[,2]),na.rm=TRUE), max(as.integer(temp[,3]),na.rm=TRUE), max(as.integer(temp[,4]),na.rm=TRUE), max(as.integer(temp[,5]),na.rm=TRUE), "Is it right? [y/n]",sep = "  "))
 #   resp <- readline()
  #  if(tolower(resp) == "n"| tolower(resp) == "no"){
   #   print("Enter the right dimensions? ")
    #  ans <- readline()
     #                                   }
    assign("nr.global",max(as.integer(temp[,4])),envir = envir)
    assign("nc.global",max(as.integer(temp[,5])),envir = envir)
    assign("nmr.global",max(as.integer(temp[,2])),envir = envir)
    assign("nmc.global",max(as.integer(temp[,3])),envir = envir)
    dimensiones <-  unlist(strsplit(ans, " "))
    dimensiones <- as.numeric(dimensiones)
    
    
    ubications <- function(mr,mc,r,c){
      lista <- seq(1:mr)
      lista <- sapply( lista, paste, seq(1:mc))
      lista <- as.vector(lista)
      lista <- sapply( lista, paste, seq(1:r))
      lista <- as.vector(lista)
      lista <- sapply(lista, paste, seq(1:c))
      lista <- as.vector(lista)
      lista
    }
    
    lista <-  ubications(dimensiones[1],dimensiones[2],dimensiones[3],dimensiones[4])
    final <- matrix(ncol = 5)
    conf <-  paste(temp[,2],temp[,3],temp[,4],temp[,5], sep = " ")
    index <- is.element(lista,conf)
    if(!missing(symdesc)){
      result.list <-  matrix(ncol = 6, nrow=length(lista))
      result.list[!index,6] <- "empty"
    }
    else
      result.list <-  matrix(ncol = 5, nrow=length(lista))
    result.list[!index,1:4] <- as.integer(0)
    result.list[!index,5] <- "empty"

    koala <- match(conf,lista)
    if(missing(symdesc)){
      result.list[koala,] <- c(temp[koala, cy3], temp[koala,cy5], temp[koala, bg.cy3], temp[koala, bg.cy5], temp[koala, ids])
    return(new ("Spot", name = spot.name,
                spotData = list(Cy3 = as.numeric(result.list[,1]), Cy5 = as.numeric(result.list[,2]),
                  BgCy3 = as.numeric(result.list[,3]), BgCy5 = as.numeric(result.list[,4]), Id = as.vector(result.list[,5]))))
    }
    else{
      result.list[koala,] <- c(temp[koala, cy3], temp[koala,cy5], temp[koala, bg.cy3], temp[koala, bg.cy5], temp[koala, ids],temp[koala, symdesc])
      return(new ("Spot", name = spot.name,
                  spotData = list(Cy3 = as.numeric(result.list[,1]), Cy5 = as.numeric(result.list[,2]),
                    BgCy3 = as.numeric(result.list[,3]), BgCy5 = as.numeric(result.list[,4]), Id = as.vector(result.list[,5]), Symdesc = as.vector(result.list[,6]))))
    }
  }
  else{
    spot <- read.csv( file.name, header = header, sep = sep)
    spot[,ids] <-  as.vector(spot[,ids])
    if(missing(symdesc))
      return(new ("Spot", name = spot.name ,spotData = list(Cy3=spot[,cy3],
                                              Cy5=spot[,cy5], BgCy3=spot[,bg.cy3], BgCy5=spot[,bg.cy5], Id = as.vector(spot[,ids]))))
    else{
      return(new ("Spot", name = spot.name ,spotData = list(Cy3=spot[,cy3],
                                              Cy5=spot[,cy5], BgCy3=spot[,bg.cy3], BgCy5=spot[,bg.cy5], Id = as.vector(spot[,ids]), Symdesc=as.vector(spot[,symdesc]))))
    }
  }
}

read.dataset <- function( file.name, cy3 = 1, cy5 = 2, ids = 3, symdesc=NULL, zscore = 4, type = 6, header = FALSE, sep = "\t"){
  if(!is.null(symdesc)){
    zscore <- 5
    symdesc <- 4
  }
  tmp <- unlist(strsplit(file.name, "\\/"))
  dataset.name <- unlist(strsplit(tmp[length(tmp)], "\\."))[1]
  dataset <- read.csv( file.name, header = header, sep = sep)
  type <- trim(as.character(dataset[1,4]))
  dataset <- dataset[-1,]
  if(is.null(symdesc)){
    new ("DataSet", name = dataset.name ,dataSets = list(Cy3 = as.numeric(dataset[,cy3]),
                                         Cy5 = as.numeric(dataset[,cy5]), Id = as.vector(dataset[,ids]),
                                         Zscore = as.numeric(as.vector(dataset[,zscore]))), type= type)
}
  else{
      new ("DataSet", name = dataset.name ,dataSets = list(Cy3 = as.numeric(dataset[,cy3]),
                                         Cy5 = as.numeric(dataset[,cy5]), Id = as.vector(dataset[,ids]), Symdesc = as.vector(dataset[,symdesc]),
                                         Zscore = as.numeric(as.vector(dataset[,zscore]))), type= type)
    }
  
}

# Windows for diagnostic plots and the analysis
require(tkrplot)
require(xtable)

back.gui <- function(envir){
  flag <- 1
  answer <- as.character(tkmessageBox(message = "All the performed operations will be deleted from this project. Are you sure?",
                                      icon = "question", type = "yesnocancel", default = "yes"))
  if( answer == "yes" ){
    proj.file <- read.table(get("history.project", envir = envir),header=FALSE,sep="\t")
    proj.tmp <- proj.file[1:(length(proj.file[,1])-get("op.counter",envir = envir)),]
    assign("op.counter",0,envir = envir)
    reset.history(get("history.project", envir = envir),proj.tmp)
    flag <- 0
  }
  flag
}

principal <-  function(envir,swap=0){
  tclRequire("BWidget")
  tt <- tktoplevel()
  tkfocus(tt)
  o.spot <- get("o.spot", envir = envir)
  datos <- attr(o.spot, "spotData")
  M <- log(datos$Cy5, 2) - log(datos$Cy3, 2)
  tkwm.geometry(tt,"+50+50")
  tkwm.title(tt,"GenArise Microarray Analyzer")
  
  lower.menu <- tkframe(tt, relief = "groove", borderwidth=2)
  
  image.menu.frame <- tkframe(lower.menu)
  
  tcl("image","create","photo","analysis",file=file.path(get("icons.dir", envir = envir),"icons/analysis.png"))
  analysis.button <- tkbutton(image.menu.frame,image="analysis", command = function(){ follow.wizard()})
  tkconfigure(analysis.button, heigh = 16, width = 16)
  
  tcl("image","create","photo","acroread",file= file.path(get("icons.dir", envir = envir),"icons/acroread.png"))
  acroread.button <- tkbutton(image.menu.frame,image="acroread", command = function(){save.as.pdf()}) 
  tkconfigure(acroread.button, heigh = 16, width = 16)
  
  tcl("image","create","photo","editor",file=file.path(get("icons.dir", envir = envir),"icons/editor.png"))
  editor.button <- tkbutton(image.menu.frame,image="editor", command = function() note(envir))
  tkconfigure(editor.button, heigh = 16, width = 16)
  
  tcl("image","create","photo","back",file=file.path(get("icons.dir", envir = envir),"icons/regresar.png"))
  back.button <- tkbutton(image.menu.frame,image="back",command =  function(){
    genArise(); tkdestroy(tt)
    unlink(paste(get("path.project", envir = envir),".prj",sep=""))
    unlink(get("path.project", envir = envir), recursive = TRUE)})
  tkconfigure(back.button, heigh = 16, width = 16)
  
  tcl("image","create","photo","help",file=file.path(get("icons.dir", envir = envir),"icons/help.png"))
  help.button <- tkbutton(image.menu.frame,image="help", command = function() help())
  tkconfigure(help.button, heigh = 16, width = 16)

  tcl("image","create","photo","exit",file=file.path(get("icons.dir", envir = envir),"icons/logout.png"))
  exit.button <- tkbutton(image.menu.frame,image="exit", command = function(){
    tkdestroy(tt)})
  tkconfigure(exit.button, heigh = 16, width = 16)
  
  tkgrid(analysis.button, acroread.button, editor.button, back.button, help.button, exit.button, sticky = "nw")
  
  etiqueta <- tklabel(lower.menu, text = "")
  tkconfigure(etiqueta, text = "", width = "90") 
  tkgrid(image.menu.frame, etiqueta, sticky = "w")
  tkgrid(lower.menu)
  
  label.function <-  function(texto.etiqueta) tkconfigure(etiqueta, text = texto.etiqueta, width = "90")
  
  
  tkbind(analysis.button, "<Enter>", function() label.function("Make analysis data"))
  tkbind(analysis.button, "<Leave>", function() label.function(""))
  
  tkbind(acroread.button, "<Enter>", function() label.function("Save graphic as PDF"))
  tkbind(acroread.button, "<Leave>", function() label.function(""))
  
  tkbind(editor.button, "<Enter>", function() label.function("Notes about this experiment"))
  tkbind(editor.button, "<Leave>", function() label.function(""))
  
  tkbind(help.button, "<Enter>", function() label.function("Display the genArise's help"))
  tkbind(help.button, "<Leave>", function() label.function(""))

  tkbind(exit.button, "<Enter>", function() label.function("Exit GenArise GUI"))
  tkbind(exit.button, "<Leave>", function() label.function(""))

  
  follow.wizard <- function(){
    respuesta <- as.character(tkmessageBox(message = "Do you want to follow the wizard?",
                                           icon = "question", type = "yesnocancel", default = "yes"))
    
    if(respuesta == "yes"){ 
      tkconfigure(tt,cursor="watch")
      tkentryconfigure(topMenu, "0", state ="disable" )
      tkentryconfigure(topMenu, "1", state ="disable" )
      tkentryconfigure(topMenu, "2", state ="disable" )
      tkentryconfigure(topMenu, "3", state ="disable" )
      tkentryconfigure(topMenu, "4", state ="disable" )
      analysis.window("", follow.wizard = TRUE, envir,swap)
      tkdestroy(tt)
      
    }else{
      if(respuesta == "no"){
        tkconfigure(tt,cursor="watch")
        analysis.window(tclvalue(tkget(txt,"0.0","end")), follow.wizard = FALSE, envir,swap)
        tkdestroy(tt)
      }
    }
  }
  
  topMenu <- tkmenu(tt)
  tkconfigure(tt,menu=topMenu)
  fileMenu <- tkmenu(topMenu,tearoff=FALSE)
  tkadd(fileMenu,"command",label="Quit", command=function(){
    tkdestroy(tt)})
  tkadd(topMenu,"cascade",label="File", menu=fileMenu)
  tkadd(topMenu,"command",label="Back", command = function(){
    tkdestroy(tt)
    genArise()
  })
  
  tkadd(topMenu,"command",label="Analyze", command = function(){
    follow.wizard()
  })
  
  save.as.pdf <-  function(){
    name <- tclvalue(tkgetSaveFile(initialdir = get("path.graphics", envir = envir),
                                   initialfile=get("spot.name",envir=envir),filetypes="{{PDF Files} {.pdf}} {{All files} *}"))
    name <- unlist(strsplit(name, "\\."))[1]
    if(swap == 0){
      if (!nchar(name))
        tkmessageBox(parent = tt,  message= "You must write a name of file!", icon = "error", default = "ok")
      else{
        rbVal <- as.character(tclvalue(rbValue))
        if(rbVal == "Red & Green"){
          pdf(paste(name, "R&G.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "R&G.pdf", sep = "_"))
          imageLimma(log(datos$Cy5, 2) - log(datos$Cy3, 2), row = get("nr", envir = envir), column = get("nc", envir = envir),
                   meta.row = get("nmr", envir = envir),meta.column = get("nmc", envir = envir))
          dev.off(dev.cur())
        }
        else{
          if(rbVal == "Red"){
            pdf(paste(name, "BgCy5.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "BgCy5.pdf", sep = "_"))
            imageLimma(log(datos$BgCy5, 2), get("nr", envir = envir), get("nc", envir = envir),
                       get("nmr", envir = envir),
                       get("nmc", envir = envir), low = "white", high = "red")
            dev.off(dev.cur())
          }
          else{
            pdf(paste(name, "BgCy3.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "BgCy3.pdf", sep = "_"))
            imageLimma(log(datos$BgCy3, 2),get("nr", envir = envir), get("nc", envir = envir),
                       get("nmr", envir = envir),
                       get("nmc", envir = envir), low = "white", high = "green")
            dev.off(dev.cur())
          }
        }
      }
    }
    else if(swap == 1){
      if (!nchar(name))
        tkmessageBox(parent = tt,  message= "You must write a name of file!", icon = "error", default = "ok")
      else{
        pdf(paste(name,"Swap.pdf",sep="_"),horiz=F, height=8,width=8,title=paste(name, "Swap.pdf", sep = "_"))
        swap.plot()
        dev.off(dev.cur())
      }
    }
  }
  
  optionsMenu <- tkmenu(topMenu,tearoff=FALSE)
  tkadd(optionsMenu,"command",label="Save graphic as PDF",command = function(){
    save.as.pdf()})
  tkadd(optionsMenu,"command",label="Note",command = function(){
    note(envir)
  })
  
  tkadd(topMenu,"cascade",label="Options", menu =optionsMenu)
  helpMenu <- tkmenu(topMenu,tearoff=FALSE)
  tkadd(helpMenu,"command",label="About GenArise",command = function() help())
  tkadd(topMenu,"cascade",label="Help",menu = helpMenu)
  upper.frame <- tkframe(tt)
  swap.plot <- function(){
    cond <- get("cond.index",envir=envir)
    R1 <- get("R1",envir=envir)
    R2 <- get("R2",envir=envir)
    par( bg = "black", col.axis = "white",col.lab = "white", fg = "white")
    plot(R1,R2, xlab = "log2(Cy5/Cy3) File1", ylab = "log2(Cy3/Cy5) File2", pch = ".",col = "white")
    points(R1[!cond],R2[!cond],col="blue",pch=".")
  }
  if(swap==0){
    img <- tkrplot(upper.frame, fun = function() imageLimma(M, get("nr", envir = envir), get("nc", envir = envir),
                                  get("nmr", envir = envir),
                                  get("nmc", envir = envir)), hscale= get("Myhscale", envir = envir),vscale = get("Myvscale",
                                                                                                       envir = envir))    
    frameOverall <- tkframe(upper.frame)
    frameUpper <- tkframe(frameOverall,relief="groove",borderwidth=2)
    tkgrid(tklabel(frameUpper, text = "Spot Features", font = "Helvetica 12"), pady = "7")
    tkgrid(tklabel(frameUpper, text = paste("Name:   ", get("spot.name", envir = envir))), sticky = "w", pady = "2")
    tkgrid(tklabel(frameUpper, text = paste("Rows:           ", get("nr", envir = envir))), sticky = "w", pady = "2")
    tkgrid(tklabel(frameUpper, text = paste("Columns:       ", get("nc", envir = envir))), sticky = "w", pady = "2")
    tkgrid(tklabel(frameUpper, text = paste("MetaRows  :     ", get("nmr", envir = envir))), sticky = "w", pady = "2")
    tkgrid(tklabel(frameUpper, text = paste("MetaColumns:  ", get("nmc", envir = envir))), sticky = "w", pady = "2")
    tkgrid(frameUpper, padx  = "10", pady = "10")
    
  
    radio.frame <- tkframe(frameOverall,relief="groove",borderwidth=2)
    
# diagnostic plots
    rbValue <- tclVar("Red & Green")
    rb1 <- tkradiobutton(radio.frame, text = "Cy5/Cy3", value = "Red & Green", variable = rbValue,
                         command = function() {tkrreplot(img, fun = function() imageLimma(M,
                                                                get("nr", envir = envir),
                                                                get("nc", envir = envir),
                                                                get("nmr", envir = envir),
                                                                get("nmc", envir = envir)))})
    rb2 <- tkradiobutton(radio.frame, text = "Background Cy5", value = "Red", variable = rbValue,
                         command = function() {tkrreplot(img, fun = function()imageLimma(log(datos$BgCy5,2),
                                                                get("nr", envir = envir),
                                                                get("nc", envir = envir),
                                                                get("nmr", envir = envir),
                                                                get("nmc", envir = envir), low = "white", high = "red"))})
    rb3 <- tkradiobutton(radio.frame, text = "Background Cy3", value = "Green", variable = rbValue,
                         command = function() {tkrreplot(img, fun = function()imageLimma(log(datos$BgCy3, 2),
                                                                get("nr", envir = envir),
                                                                get("nc", envir = envir),
                                                                get("nmr", envir = envir),
                                                                get("nmc", envir = envir), low = "white", high = "green"))})
    tkgrid(rb1, sticky = "w", pady = "2",padx="10")
    tkgrid(rb2, sticky = "w", pady = "2",padx="10")
    tkgrid(rb3, sticky = "w", pady = "2",padx="10")
    tkgrid(frameUpper, padx  = "20", pady = "120")
#Label Diagnostic Plots
    tkgrid(tklabel(upper.frame, text = "Diagnostic Plots", font = "Helvetica 12"))  
    tkgrid(radio.frame, padx  = "20", pady = "10")
    tkgrid(img, frameOverall, sticky = "nw")
    tkgrid(tklabel(upper.frame, text = "This plots allow to look at the variation of R and Background values over the array.", font = "Helvetica 10"))  
    tkgrid(upper.frame, pady = "10")
  }
  else if(swap==1){
    img <- tkrplot(upper.frame, fun = function() swap.plot())
    frameOverall <- tkframe(upper.frame)
    frameUpper <- tkframe(frameOverall,relief="groove",borderwidth=2)
    tkgrid(tklabel(frameUpper, text = "Spot Features", font = "Helvetica 12"), pady = "7")
    tkgrid(tklabel(frameUpper, text = get("spot.name", envir = envir)), sticky = "w", pady = "2")
    tkgrid(tklabel(frameUpper, text = paste("Rows:           ", get("nr.global", envir = envir))), sticky = "w", pady = "2")
    tkgrid(tklabel(frameUpper, text = paste("Columns:       ", get("nc.global", envir = envir))), sticky = "w", pady = "2")
    tkgrid(tklabel(frameUpper, text = paste("MetaRows  :     ", get("nmr.global", envir = envir))), sticky = "w", pady = "2")
    tkgrid(tklabel(frameUpper, text = paste("MetaColumns:  ", get("nmc.global", envir = envir))), sticky = "w", pady = "2")
    tkgrid(frameUpper, padx  = "10", pady = "10")

    radio.frame <- tkframe(frameOverall,relief="groove",borderwidth=2)
    tkgrid(tklabel(radio.frame, text = get("conf", envir = envir)), sticky = "w", pady = "2")
    tkgrid(tklabel(radio.frame, text = get("disp", envir = envir)), sticky = "w", pady = "2")
    tkgrid(radio.frame, padx  = "20", pady = "10")
    tkgrid(img, frameOverall, sticky = "nw")
    tkgrid(upper.frame, pady = "10")
  }
  area.frame <- tkframe(tt,relief="groove")
  yscr <- tkscrollbar(area.frame, repeatinterval = 5,
                      command=function(...)tkyview(txt,...))
  txt <- tktext(area.frame,bg="white",font="courier",width =get("barWidth",envir=envir),height=get("barHeight",envir=envir),yscrollcommand=function(...)tkset(yscr,...))
  tkgrid(txt, yscr, sticky = "w")
  tkgrid.configure(yscr,rowspan=4,sticky="ns")
  tkinsert(txt,"end","GenArise Microarray Analyzer\n")
  tkinsert(txt,"end","Institute of Cellular Physiology UNAM")
  tkconfigure(txt, state="disabled")
  tkgrid(area.frame, padx = "10", pady = "10", sticky = "w")
}

# Auxiliar function to call the Zscore.plot function in the GUI
Zscore.points <-  function(type="ri",text, envir,swap){
  tt <- tktoplevel()
  tkfocus(tt)
  tkwm.title(tt,"GenArise Microarray Analyzer")
  topMenu <- tkmenu(tt)
  tkconfigure(tt,menu=topMenu)
  fileMenu <- tkmenu(topMenu,tearoff=FALSE)
  upper.frame <- tkframe(tt)
  sd.frame <- tkframe(upper.frame, borderwidth=2)
  ratio.buttons.frame <- tkframe(sd.frame,relief="groove",borderwidth=2)
  dist  <- tclVar(5)
  
  radio1 <- tkradiobutton(ratio.buttons.frame, text="Zscore < 1",
                          value=1, variable=dist, command=function(){
                            tkrreplot(img, fun = function()
                                      Zscore.plot(get("Zscore.spot", envir = envir), all = FALSE, Zscore.max = 1, col = "green"))})
  radio2 <- tkradiobutton(ratio.buttons.frame, text="1 < Zscore < 1.5",
                          value=2, variable=dist,command = function(){
                            tkrreplot(img, fun = function() Zscore.plot(get("Zscore.spot", envir = envir),
                                             Zscore.max = 1.5, Zscore.min = 1, all = FALSE, col = "blue"))})
  radio3 <- tkradiobutton(ratio.buttons.frame, text="1.5 < Zscore < 2",
                          value=3, variable=dist,command = function(){
                            tkrreplot(img, fun = function() Zscore.plot(get("Zscore.spot", envir = envir),
                                             Zscore.max = 2, Zscore.min = 1.5, all = FALSE, col = "cyan"))})
  radio4 <- tkradiobutton(ratio.buttons.frame, text="Zscore > 1.5",
                          value=4, variable=dist,command = function(){
                            tkrreplot(img, fun = function() Zscore.plot(get("Zscore.spot", envir = envir),
                                             Zscore.min = 1.5, all = FALSE, col = "yellow"))})
  radio5 <- tkradiobutton(ratio.buttons.frame, text="Zscore > 2",
                          value=5, variable=dist, command=function(){
                            tkrreplot(img, fun = function() Zscore.plot(get("Zscore.spot", envir = envir),
                                             Zscore.min = 2, all = FALSE, col = "snow"))})
  radio6 <- tkradiobutton(ratio.buttons.frame, text="All",
                          value=6, variable=dist, command=function(){
                            tkrreplot(img, fun = function() Zscore.plot(get("Zscore.spot", envir = envir)))})
  
  tkgrid(radio1, sticky="w")
  tkgrid(radio2, sticky="w")
  tkgrid(radio3, sticky="w")
  tkgrid(radio4, sticky="w")
  tkgrid(radio5, sticky="w")
  tkgrid(radio6, sticky="w")

  # get the information for the annotation function
  annotations.but <- tkbutton(sd.frame,text = "Annotations", command=function(){
    choose.specie <- tktoplevel()
    tkfocus(choose.specie)
    tkwm.title(choose.specie,"Specie Selector")
    tkgrid(tklabel(choose.specie,text="Choose the specie"))
    species <- c("Mus musculus","Saccharomyces Cerevisiae","Rattus norvegicus", "NCBI")
    comboBox <- tkwidget(choose.specie, "ComboBox",editable=FALSE,values=species)
    tkgrid(comboBox)
    
    OnOK <- function()
      {
        datos <- attr(get("Zscore.spot", envir = envir), "dataSets")
        val <- as.numeric(tclvalue(dist))
        if(val != 6){
          Zscore <- datos$Zscore
          if(val == 1){
            
            indexUp <- Zscore > 1
            indexDown <- Zscore < -1
            name <- "_less_1SD"
          }
          if(val == 2){
            indexUp <- Zscore > 1 & Zscore < 1.5
            indexDown <- Zscore < -1 & Zscore > -1.5
            name <- "_1-1.51SD"
          }
          if(val == 3){
            indexUp <- Zscore > 1.5 & Zscore < 2
            indexDown <- Zscore < -1.5 & Zscore > -2
            name <- "_1_5-2SD"
          }
          if(val == 4){
            indexUp <- Zscore > 1.5
            indexDown <- Zscore < -1.5
            name <- "_greater_1_5SD"
          }
          if(val == 5){
          indexUp <- Zscore > 2
          indexDown <- Zscore < -2
          name <- "_greater_2SD"
        }
          if(!is.null(datos$Symdesc)){
            Up <- list(Cy3 = datos$Cy3[indexUp], Cy5 = datos$Cy5[indexUp],
                       Id = datos$Id[indexUp], Symdesc = datos$Symdesc[indexUp], Zscore = datos$Zscore[indexUp])
            Down <- list(Cy3 = datos$Cy3[indexDown], Cy5 = datos$Cy5[indexDown],
                         Id = datos$Id[indexDown], Symdesc = datos$Symdesc[indexDown], Zscore = datos$Zscore[indexDown])
          }
          else{
            Up <- list(Cy3 = datos$Cy3[indexUp], Cy5 = datos$Cy5[indexUp],
                       Id = datos$Id[indexUp], Zscore = datos$Zscore[indexUp])
            Down <- list(Cy3 = datos$Cy3[indexDown], Cy5 = datos$Cy5[indexDown],
                         Id = datos$Id[indexDown], Zscore = datos$Zscore[indexDown])
          }
          kk <- as.data.frame(Up)
          kk2 <- as.data.frame(Down)
          kk <- apply(kk, 2, as.vector)
          kk2 <- apply(kk2, 2, as.vector)
          la.especie <- as.numeric(tclvalue(tcl(comboBox,"getvalue")))
          if(!is.null(datos$Symdesc)){
            annotations(kk, species[(la.especie + 1)], 3, symbol=4, file.path(get("path.project", envir = envir), paste("annotationsUp",name,".htm",sep="")))
            annotations(kk2, species[(la.especie + 1)], 3, symbol=4, file.path(get("path.project", envir = envir), paste("annotationsDown",name,".htm",sep="")))
          }
          else{
            annotations(kk, species[(la.especie + 1)], 3, symbol=NULL, file.path(get("path.project", envir = envir), paste("annotationsUp",name,".htm",sep="")))
            annotations(kk2, species[(la.especie + 1)], 3, symbol=NULL,file.path(get("path.project", envir = envir), paste("annotationsDown",name,".htm",sep="")))
          }
          tkdestroy(choose.specie)
         }
        else{
          kk <- as.data.frame(datos)
          kk <- apply(kk, 2, as.vector)
          la.especie <- as.numeric(tclvalue(tcl(comboBox,"getvalue")))
          if(!is.null(datos$Symdesc))
            annotations(kk, species[(la.especie + 1)], 3, 4, file.path(get("path.project", envir = envir), "annotationsAll.htm"))
          else
            annotations(kk, species[(la.especie + 1)], 3,symbol=NULL, file.path(get("path.project", envir = envir), "annotationsAll.htm"))
          tkdestroy(choose.specie)
         }
      }
    
    OnCancel <-  function(){
      tkdestroy(choose.specie)
    }
    
    OK.but <-tkbutton(choose.specie,text="   OK   ",command=OnOK)
    Cancel.but <-tkbutton(choose.specie,text="   Cancel   ",command=OnCancel)
    tkgrid(OK.but, Cancel.but, padx = "20", pady = "10")
    tkfocus(tt)
  })
  
  tkgrid(ratio.buttons.frame, sticky="w", pady = "20")
  tkgrid(annotations.but, sticky="w", pady = "10")
  
  tkselect(radio6)
  tkadd(fileMenu,"command",label="Close",command=function(){
    tkdestroy(tt)
    genArise.init(envir)})
  tkadd(fileMenu,"command",label="Exit",command=function(){
    tkdestroy(tt)})
  tkadd(topMenu,"cascade",label="File",menu=fileMenu)
  tkadd(topMenu,"command",label="Back", command = function(){
    sure <- back.gui(envir)
    if(sure == 0){
      tkdestroy(tt)
      if(swap== 0){
        principal(envir,0)
      }
      else if(swap == 1){
        principal(envir,1)
      }
  }})
  
  optionsMenu <- tkmenu(topMenu,tearoff=FALSE)
  
  make.file.name <- function(file.type, ext){
    switch(as.integer(tclvalue(dist)),
           suggested.name <- paste(get("spot.name",envir=envir), "_lower_1.", ext, sep = ""),
           suggested.name <- paste(get("spot.name",envir=envir), "_1-1_5.", ext, sep = ""),
           suggested.name <- paste(get("spot.name",envir=envir), "_1_5-2.", ext, sep = ""),
           suggested.name <- paste(get("spot.name",envir=envir), "_greater_1_5.", ext, sep = ""),
           suggested.name <- paste(get("spot.name",envir=envir), "_greater_2.", ext, sep = ""),
           suggested.name <- paste(get("spot.name",envir=envir), "_all.", ext, sep = ""),
           )
    if(ext == "pdf"){
      salida <- "path.graphics"
    }else{
      salida <-  "path.results"
    }
    tclvalue(tkgetSaveFile(initialdir = get(salida, envir = envir),
                           initialfile = suggested.name, filetypes = file.type))


    
  }    
  tkadd(optionsMenu,"command",label="Save graphic as PDF",command = function() {
    name <- make.file.name(file.type = "{{PDF Files} {.pdf}} {{All files} *}", ext ="pdf")
    name <- unlist(strsplit(name, "\\."))[1]
    if (!nchar(name))
      tkmessageBox(parent = tt,  message= "You must write a name of file!", icon = "error", default = "ok")
    else{
      pdf(paste(name,".pdf",sep=""), horiz = FALSE, height = 8, width = 8, title = name)
      switch(as.integer(tclvalue(dist)),
             Zscore.plot(get("Zscore.spot", envir = envir), all = FALSE, Zscore.max = 1, col = "green"),
             Zscore.plot(get("Zscore.spot", envir = envir), Zscore.max = 1.5, Zscore.min = 1, all = FALSE, col = "blue"),
             Zscore.plot(get("Zscore.spot", envir = envir), Zscore.max = 2, Zscore.min = 1.5, all = FALSE, col = "cyan"),
             Zscore.plot(get("Zscore.spot", envir = envir), Zscore.min = 1.5, all = FALSE, col = "yellow"),
             Zscore.plot(get("Zscore.spot", envir = envir), Zscore.min = 2, all = FALSE, col = "snow"),
             Zscore.plot(get("Zscore.spot", envir = envir)))
      dev.off(dev.cur())
    }
  })

  # save the Zscore for the data set in a file
  tkadd(optionsMenu,"command",label="Write as outputfile",command = function() {
    name <- make.file.name(file.type = "{{TXT Files} {.txt}} {{All files} *}", ext="txt")
    if (!nchar(name))
      tkmessageBox(parent = tt,  message= "You must write a name of file!", icon = "error", default = "ok")
    else{
      val <- as.integer(tclvalue(dist))
      if(swap==0){
        if( val == 6){
          if(is.null(attr(get("Zscore.spot", envir = envir),"dataSets")$Symdesc))
            write.table("Cy3\tCy5\tId\tZscore",name, quote = FALSE,sep = "\t", col.names = FALSE, row.names = FALSE )
          else
            write.table("Cy3\tCy5\tId\tSymbol\tZscore",name, quote = FALSE,sep = "\t", col.names = FALSE, row.names = FALSE )
          write.table(attr(get("Zscore.spot", envir = envir), "dataSets"),
                      name, quote = FALSE, col.names = FALSE, row.names = FALSE,sep="\t",append=TRUE)
        }else if(val == 5){
          write.dataSet(get("Zscore.spot", envir = envir), name, Zscore.min = 2)
        }
        else if(val == 4){
          write.dataSet(get("Zscore.spot", envir = envir), name, Zscore.min = 1.5)
        }
        else if(val == 3){
          write.dataSet(get("Zscore.spot", envir = envir), name, Zscore.min = 1.5, Zscore.max = 2)
        }
        else if(val == 2){
          write.dataSet(get("Zscore.spot", envir = envir), name, Zscore.min = 1, Zscore.max = 1.5)
        }
        else if(val == 1){
          write.dataSet(get("Zscore.spot", envir = envir), name, Zscore.max = 1)
        }
      }
      else{
        warn <- tktoplevel()
        tkfocus(warn)
        message <- tklabel(warn,text="This operation may take a few minutes")
        tkwm.title(warn,"Note")
        tkgrid(message)
        tkfocus(warn)
        Sys.sleep(1)
        if(get("ind.norm", envir = envir) == 0){
          single.norm(envir)
          assign("ind.norm",1,envir=envir)
        }
        Zscore.spot <- get("Zscore.spot", envir = envir)
        if( val == 6){
            get.Zscore(Zscore.spot, name, Zscore.min = 0, all=TRUE, envir = envir)
        }else if(val == 5){
            get.Zscore(Zscore.spot, name, Zscore.min = 2, envir = envir)
        }
        else if(val == 4){
            get.Zscore(Zscore.spot, name, Zscore.min = 1.5, envir = envir)
        }
        else if(val == 3){
            get.Zscore(Zscore.spot, name, Zscore.min = 1.5, Zscore.max = 2, envir = envir)
        }
        else if(val == 2){
            get.Zscore(Zscore.spot, name, Zscore.min = 1, Zscore.max = 1.5, envir = envir)
        }
        else if(val == 1){
            write.dataSet(Zscore.spot, name, Zscore.max = 1)
        }
        tkdestroy(warn)
      }
    }
  })
  
  tkadd(optionsMenu,"command",label="Notes",command = function()note(envir))
  tkadd(topMenu,"cascade",label="Options",menu =optionsMenu)
  
  helpMenu <- tkmenu(topMenu,tearoff=FALSE)
  tkadd(helpMenu,"command",label="About genArise...", command=function() help())
  tkadd(topMenu,"cascade",label="Help",menu = helpMenu)
  
  
  frame.label <- tkframe(tt,relief="groove",borderwidth=2)

  #######################

  lower.menu <- tkframe(tt, relief = "groove", borderwidth=2)
  
  image.menu.frame <- tkframe(lower.menu)
  
  tcl("image","create","photo","acroread",file= file.path(get("icons.dir", envir = envir),"icons/acroread.png"))
  acroread.button <- tkbutton(image.menu.frame,image="acroread", command = function(){
    name <- make.file.name(file.type = "{{PDF Files} {.pdf}} {{All files} *}", ext ="pdf")
    name <- unlist(strsplit(name, "\\."))[1]
    if (!nchar(name))
      tkmessageBox(parent = tt,  message= "You must write a name of file!", icon = "error", default = "ok")
    else{
      pdf(paste(name,".pdf",sep=""), horiz = FALSE, height = 8, width = 8, title = name)
      switch(as.integer(tclvalue(dist)),
             Zscore.plot(get("Zscore.spot", envir = envir), all = FALSE, Zscore.max = 1, col = "green"),
             Zscore.plot(get("Zscore.spot", envir = envir), Zscore.max = 1.5, Zscore.min = 1, all = FALSE, col = "blue"),
             Zscore.plot(get("Zscore.spot", envir = envir), Zscore.max = 2, Zscore.min = 1.5, all = FALSE, col = "cyan"),
             Zscore.plot(get("Zscore.spot", envir = envir), Zscore.min = 1.5, all = FALSE, col = "yellow"),
             Zscore.plot(get("Zscore.spot", envir = envir), Zscore.min = 2, all = FALSE, col = "snow"),
             Zscore.plot(get("Zscore.spot", envir = envir)))
      dev.off(dev.cur())
    }
  }) 
  tkconfigure(acroread.button, heigh = 16, width = 16)
  
  tcl("image","create","photo","editor",file=file.path(get("icons.dir", envir = envir),"icons/editor.png"))
  editor.button <- tkbutton(image.menu.frame,image="editor", command = function() note(envir))
  tkconfigure(editor.button, heigh = 16, width = 16)
  
  tcl("image","create","photo","back",file=file.path(get("icons.dir", envir = envir),"icons/regresar.png"))
  back.button <- tkbutton(image.menu.frame,image="back",command =  function(){
     sure <- back.gui(envir)
    if(sure == 0){
      tkdestroy(tt)
      if(swap== 0){
        principal(envir,0)
      }
      else if(swap == 1){
        principal(envir,1)
      }
  }})
  tkconfigure(back.button, heigh = 16, width = 16)
  
  tcl("image","create","photo","help",file=file.path(get("icons.dir", envir = envir),"icons/help.png"))
  help.button <- tkbutton(image.menu.frame,image="help", command = function() help())
  tkconfigure(help.button, heigh = 16, width = 16)

  tcl("image","create","photo","exit",file=file.path(get("icons.dir", envir = envir),"icons/logout.png"))
  exit.button <- tkbutton(image.menu.frame,image="exit", command = function(){
    tkdestroy(tt)})
  tkconfigure(exit.button, heigh = 16, width = 16)
  
  tkgrid(acroread.button, editor.button, back.button, help.button, exit.button, sticky = "nw")
  
  etiqueta <- tklabel(lower.menu, text = "")
  tkconfigure(etiqueta, text = "", width = "90") 
  tkgrid(image.menu.frame, etiqueta, sticky = "w")
  tkgrid(lower.menu)


  #######################
  tkfocus(tt)
  img <- tkrplot(upper.frame,fun = function()Zscore.plot(get("Zscore.spot", envir = envir),type),
                 hscale= get("Myhscale", envir = envir),vscale = get("Myvscale",envir = envir))
  tkgrid(img,sd.frame, padx = "10", pady = "10")
  tkgrid(upper.frame, pady = "10")
  area.frame <- tkframe(tt,relief="groove")
  yscr <- tkscrollbar(area.frame, repeatinterval = 5,
                      command=function(...)tkyview(txt,...))
  
 
  txt <- tktext(area.frame,bg="white",font="courier",width =get("barWidth",envir=envir),height=get("barHeight",envir=envir),yscrollcommand=function(...)tkset(yscr,...))
  tkgrid(txt, yscr)
  tkgrid.configure(yscr,rowspan=4,sticky="nsw")
  tkinsert(txt,"end",text)
  tkinsert(txt,"end","\n\nZscore Done!!")
  tkconfigure(txt, state="disabled")
  tkgrid(area.frame, padx = "10", sticky = "w")
}

# Window for the analysis
analysis.window <-  function(texto, follow.wizard = FALSE, envir,swap=0){
if(swap==1){
  assign("ind.norm",0, envir=envir)
}else if(swap==0){
  assign("corrected", 0, envir = envir)
}
  # check if the background correction will be done
  bg.question <- function(){
    answer <-tkmessageBox(message="Do you want to make background correction?",icon="question",type="yesnocancel",default="yes")
    ans <- as.character(answer)
    op.counter <<- op.counter + 1
    write.spot(get("o.spot", envir = envir), paste(get("path.results", envir = envir),
                               .Platform$file.sep, "original.txt",sep = ""))
    set.history.project(get("history.project", envir = envir), "Original Spot", "original.txt")
    
    if(ans == "yes"){
      c.spot <- bg.correct(get("a.spot", envir = envir))
      op.counter <<- op.counter + 1
      tkconfigure(txt, state="normal")
      tkinsert(txt,"end","\n\nBackground correction done.............")
      tkconfigure(txt, state="disable")
      assign("c.spot", c.spot, envir = envir)
      write.spot(get("c.spot", envir = envir), paste(get("path.results", envir = envir),
                                 .Platform$file.sep, "corrected.txt",sep = ""))
#      print(attr(get("c.spot", envir = envir),"spotData")$Symdesc)
      set.history.project(get("history.project", envir = envir), "Corrected Spot", "corrected.txt")
      assign("a.spot", c.spot, envir = envir)
      tkpack(corr.radio,anchor="w",padx="10")
      tkselect(corr.radio)
      assign("corrected",1, envir = envir)
    }
    else {
      if(ans == "no"){
        assign("corrected",1, envir = envir)
      }
      else{
        assign("corrected",2, envir = envir)      }
    }
  }

  # GUI behavior after the data normalization
  normalized.gui <- function(type = 1, history.entry, analysis.plot){ # 1 by grid, 2 global
    if(swap==0){
    if(get("corrected",envir=envir) == 0) bg.question()
    if(get("corrected",envir=envir) == 2) assign("corrected",0,envir=envir)
    else{
      if(get("corrected",envir = envir) == 1){
        tkconfigure(tt,cursor="watch")
        switch(type,
               {n.spot <- grid.norm(get("a.spot", envir = envir),
                                    get("nr", envir = envir), get("nc", envir = envir))
                op.counter <<- op.counter + 1
                history.text <- "\n\nNormalization by grid done!..."},
               {n.spot <- global.norm(get("a.spot", envir = envir))
                history.text <- "\n\nGlobal normalization done!..."})
        tkconfigure(history.entry, state="normal")
        tkinsert(history.entry,"end",history.text)
        tkconfigure(history.entry, state="disable")
        assign("n.spot", n.spot, envir = envir)
        write.spot(get("n.spot", envir = envir), paste(get("path.results", envir = envir),
                                   .Platform$file.sep, "normalized.txt",sep = ""))
        set.history.project(get("history.project", envir = envir), "Normalized Spot", "normalized.txt")
        assign("a.spot", n.spot, envir = envir)
        tkrreplot(analysis.plot,fun=function()graphic.choose(get("a.spot", envir = envir),
                                  get("graphic.type", envir = envir)))
        tkpack(normal.radio,anchor="w",padx="10")
        tkselect(normal.radio)
        tkentryconfigure(topMenu, "4", state ="disable" )
        tkconfigure(normal.button, state = "disable")
        tkconfigure(tt,cursor="arrow")
      }
    }
  }
    else if(swap==1){
 tkconfigure(tt,cursor="watch")
        switch(type,
               {n.spot <- grid.norm(get("a.spot", envir = envir),
                                    get("nr.global", envir = envir), get("nc.global", envir = envir))
                op <- get("op.counter", envir = envir) + 1
                assign("op.counter", op, envir = envir)
                history.text <- "\n\nNormalization by grid done!..."},
               {n.spot <- global.norm(get("a.spot", envir = envir))
                history.text <- "\n\nGlobal normalization done!..."})
        tkconfigure(history.entry, state="normal")
        tkinsert(history.entry,"end",history.text)
        tkconfigure(history.entry, state="disable")
        assign("n.spot", n.spot, envir = envir)
        write.spot(get("n.spot", envir = envir), paste(get("path.results", envir = envir),
                                   .Platform$file.sep, "normalized.txt",sep = ""))
        set.history.project(get("history.project", envir = envir), "Normalized Spot", "normalized.txt")
        assign("a.spot", n.spot, envir = envir)
        tkrreplot(analysis.plot,fun=function()graphic.choose(get("a.spot", envir = envir),
                                  get("graphic.type", envir = envir)))
        tkpack(normal.radio,anchor="w",padx="10")
        tkselect(normal.radio)
        tkentryconfigure(topMenu, "4", state ="disable" )
        tkconfigure(normal.button, state = "disable")
        tkconfigure(tt,cursor="arrow")
}
  }

  # GUI behavior after the data filtering
  filter.gui <-  function(history.text, analysis.plot){
    if(swap==0){
    if(get("corrected",envir=envir) == 0) bg.question()
    if(get("corrected",envir=envir) == 2) assign("corrected",0,envir=envir)
       else{
         tkconfigure(tt,cursor="watch")
         f.spot <- filter.spot(get("a.spot", envir = envir))
         op.counter <<- op.counter + 1
         tkconfigure(history.text, state="normal")
                                        #    tkinsert(history.text,"end","\n\nIn some cases, the correlation of a signal between log background and log signal is observed to be larger than that of the other signal in the experiment. If this is the case, you could see a strange effect after filter by intensity.\n\nIntensity-based filtering done.............")
         tkinsert(history.text,"end","\n\nIn some cases, the standard deviation of the background of a signal is observed to be much larger than that of the other signal in the experiment. If this is the case, you could see a strange effect in the plot after filter by intensity.\n\nIntensity-based filtering done.............")
         tkconfigure(history.text, state="disable")
         assign("f.spot", f.spot, envir = envir)
         write.spot(get("f.spot", envir = envir), paste(get("path.results", envir = envir),
                                    .Platform$file.sep, "filter.txt",sep = ""))
         set.history.project(get("history.project", envir = envir), "Filtered Spot", "filter.txt")
         assign("a.spot", f.spot, envir = envir)
         tkrreplot(analysis.plot,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))
         tkpack(fil.radio,anchor="w",padx="10")
         tkselect(fil.radio)
         tkentryconfigure(filterMenu, "0", state ="disable" )
         tkconfigure(filter.button, state = "disable")
         count.filter <<- count.filter +1
         tkentryconfigure(normalMenu, "0", state ="disable" )
         if(count.filter == 2){
           tkentryconfigure(topMenu, "3", state ="disable" )}
         tkconfigure(tt,cursor="arrow")
       }
  }
    else if(swap==1){
 tkconfigure(tt,cursor="watch")
         f.spot <- filter.spot(get("a.spot", envir = envir))
         op <- get("op.counter", envir = envir) + 1
         assign("op.counter", op, envir = envir)
         tkconfigure(history.text, state="normal")
                                        #    tkinsert(history.text,"end","\n\nIn some cases, the correlation of a signal between log background and log signal is observed to be larger than that of the other signal in the experiment. If this is the case, you could see a strange effect after filter by intensity.\n\nIntensity-based filtering done.............")
         tkinsert(history.text,"end","\n\nIn some cases, the standard deviation of the background of a signal is observed to be much larger than that of the other signal in the experiment. If this is the case, you could see a strange effect in the plot after filter by intensity.\n\nIntensity-based filtering done.............")
         tkconfigure(history.text, state="disable")
         assign("f.spot", f.spot, envir = envir)
         write.spot(get("f.spot", envir = envir), paste(get("path.results", envir = envir),
                                    .Platform$file.sep, "filter.txt",sep = ""))
         set.history.project(get("history.project", envir = envir), "Filtered Spot", "filter.txt")
         assign("a.spot", f.spot, envir = envir)
         tkrreplot(analysis.plot,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))
         tkpack(fil.radio,anchor="w",padx="10")
         tkselect(fil.radio)
         tkentryconfigure(filterMenu, "0", state ="disable" )
         tkconfigure(filter.button, state = "disable")
         count.filter <<- count.filter +1
         tkentryconfigure(normalMenu, "0", state ="disable" )
         if(count.filter == 2){
           tkentryconfigure(topMenu, "3", state ="disable" )}
         tkconfigure(tt,cursor="arrow")
}
  }
     

  
  # GUI behavior after remove replicates
  # 1 mean replicate filtering, 2 non-extreme values replicate, 3 geometric mean
  remove.duplicates.gui <-  function(type = 1, history.entry, analysis.plot){
    if(swap==0){
    if(get("corrected",envir=envir) == 0) bg.question()
    if(get("corrected",envir=envir) == 2) assign("corrected",0,envir=envir)
    else{
      if(get("corrected",envir=envir) == 1){
        tkconfigure(tt,cursor="watch")
        switch(type,
               {u.spot <- meanUnique(get("a.spot", envir = envir))
                op.counter <<- op.counter + 1
                history.text <-  "\n\nMean replicates filtering done!..."
                history.project.text <- "Mean replicates filter"},
               {u.spot <- alter.unique(get("a.spot", envir = envir))
                op.counter <<- op.counter + 1
                history.text <- "\n\nNon-extreme values replicate"
                history.project.text <- "Non-extreme filter dup"},
               {u.spot <- spotUnique(get("a.spot", envir = envir))
                op.counter <<- op.counter + 1
                history.text <- "\n\nGeometric mean filtering done!..."
                history.project.text <- "Geometric mean filter dup"})
        tkconfigure(history.entry, state="normal")
        tkinsert(history.entry,"end", history.text)
        tkconfigure(history.entry, state="disable")
        assign("u.spot", u.spot, envir = envir)
        write.spot(get("u.spot", envir = envir), paste(get("path.results", envir = envir),
                                   .Platform$file.sep, "withoutDup.txt",sep = ""))
        set.history.project(get("history.project", envir = envir), history.project.text, "withoutDup.txt")
        assign("a.spot", u.spot, envir = envir)
        tkrreplot(analysis.plot,fun=function()graphic.choose(get("a.spot", envir = envir),
                                  get("graphic.type", envir = envir)))
        tkpack(uniq.radio,anchor="w",padx="10")
        tkselect(uniq.radio)
        tkentryconfigure(filterMenu, "1", state ="disable" )
        count.filter <- count.filter + 1
        tkentryconfigure(normalMenu, "0", state ="disable" )
        if(count.filter == 2){
          tkentryconfigure(topMenu, "3", state ="disable" )}}
      tkconfigure(tt,cursor="arrow")
    }
  }
    else if(swap==1){
        tkconfigure(tt,cursor="watch")
        switch(type,
               {u.spot <- meanUnique(get("a.spot", envir = envir))
                op <- get("op.counter", envir = envir) + 1
                assign("op.counter", op, envir = envir)
                history.text <-  "\n\nMean replicates filtering done!..."
                history.project.text <- "Mean replicates filter"},
               {u.spot <- alter.unique(get("a.spot", envir = envir))
                op <- get("op.counter", envir = envir) + 1
                assign("op.counter", op, envir = envir)
                history.text <- "\n\nNon-extreme values replicate"
                history.project.text <- "Non-extreme filter dup"},
               {u.spot <- spotUnique(get("a.spot", envir = envir))
                op <- get("op.counter", envir = envir) + 1
                assign("op.counter", op, envir = envir)
                history.text <- "\n\nGeometric mean filtering done!..."
                history.project.text <- "Geometric mean filter dup"})
        tkconfigure(history.entry, state="normal")
        tkinsert(history.entry,"end", history.text)
        tkconfigure(history.entry, state="disable")
        assign("u.spot", u.spot, envir = envir)
        write.spot(get("u.spot", envir = envir), paste(get("path.results", envir = envir),
                                   .Platform$file.sep, "withoutDup.txt",sep = ""))
        set.history.project(get("history.project", envir = envir), history.project.text, "withoutDup.txt")
        assign("a.spot", u.spot, envir = envir)
        tkrreplot(analysis.plot,fun=function()graphic.choose(get("a.spot", envir = envir),
                                  get("graphic.type", envir = envir)))
        tkpack(uniq.radio,anchor="w",padx="10")
        tkselect(uniq.radio)
        tkentryconfigure(filterMenu, "1", state ="disable" )
        count.filter <- count.filter + 1
        tkentryconfigure(normalMenu, "0", state ="disable" )
        if(count.filter == 2){
          tkentryconfigure(topMenu, "3", state ="disable" )}
      tkconfigure(tt,cursor="arrow")
      }
  }

  # GUI behavior after perform Zscore
  zscore.gui <- function(){
    tkconfigure(tt,cursor="watch")
    assign("Zscore.spot", Zscore(get("a.spot",envir=envir)), envir = envir)
    write.zscore(get("Zscore.spot", envir = envir), file.path(get("path.results", envir = envir), "zscore.txt"))
    set.history.project(get("history.project", envir = envir), "RI Zscore", "zscore.txt")
    Zscore.points(type="ri",text=tclvalue(tkget(txt,"0.0","end")), envir,swap)
    tkdestroy(tt)
  }
  
  count.filter <-  0
  tt <- tktoplevel()
tkfocus(tt)
  tkwm.title(tt,"GenArise Microarray Analyzer")
  topMenu <- tkmenu(tt)
  tkconfigure(tt,menu=topMenu)
  fileMenu <- tkmenu(topMenu,tearoff=FALSE)
  upper.frame <- tkframe(tt)
  frame1 <- tkframe(upper.frame,relief="groove",borderwidth=2)
  dist  <- tclVar(1)
  ori.radio <- tkradiobutton(frame1, text="Original Spot",
                             value=1, variable=dist, command=function(){
                               assign("a.spot", get("o.spot", envir = envir), envir = envir)
                               tkrreplot(img,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))})
  corr.radio <- tkradiobutton(frame1, text="Corrected Spot",
                              value=2, variable=dist,command = function(){
                                assign("a.spot", get("c.spot", envir = envir), envir = envir)
                                tkrreplot(img,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))})
  
  normal.radio <- tkradiobutton(frame1, text="Normal Spot",
                                value=3, variable=dist,command = function(){
                                  assign("a.spot", get("n.spot", envir = envir), envir = envir)
                                  tkrreplot(img,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))})
  
  fil.radio <- tkradiobutton(frame1, text="Filter Spot",
                             value=4, variable=dist,command = function(){
                               assign("a.spot", get("f.spot", envir = envir), envir = envir)
                               tkrreplot(img,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))})
  uniq.radio <- tkradiobutton(frame1, text="Without duplicates",
                              value=5, variable=dist,command = function(){
                                assign("a.spot", get("u.spot", envir = envir), envir = envir)
                                tkrreplot(img,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))})
  tkpack(ori.radio,anchor="w",padx="10")
  
  tkadd(fileMenu,"command",label="Close",command=function(){
    tkdestroy(tt)
    genArise.init(envir) })
  tkadd(fileMenu,"command",label="Exit",command=function(){
    tkdestroy(tt)})
  tkadd(topMenu,"cascade",label="File",menu=fileMenu)

  tkadd(topMenu,"command",label="Back", command = function(){
    sure <- back.gui(envir)
    if(sure == 0){
      tkdestroy(tt)
      if(swap== 0){
        principal(envir,0)
      }
      else if(swap == 1){
        principal(envir,1)
      }
  }})
  filterMenu <- tkmenu(topMenu,tearoff=FALSE)
  
  tkadd(filterMenu,"command",label="By Intensity", state ="active",command = function() {
    filter.gui(txt, img)})
#  tkentryconfigure(filterMenu, "0", state ="disable" ) #Inactive temporal
  duplicatesMenu <- tkmenu(filterMenu,tearoff=FALSE)
if(swap==0){
  tkadd(duplicatesMenu,"command",label="Mean Replicate Filtering", command = function(){
    remove.duplicates.gui(1,txt,img)})
  tkadd(duplicatesMenu,"command",label="Non-extreme values replicate filtering", command = function(){
    remove.duplicates.gui(2, txt, img)})
  tkadd(duplicatesMenu,"command",label="Geometric Mean Filter", command = function(){
    remove.duplicates.gui(3, txt, img)})

  tkadd(filterMenu,"cascade",label="Duplicates Analysis", state ="active",menu= duplicatesMenu)
  
  tkadd(topMenu,"cascade",label="Filter",menu =filterMenu)
  
  normalMenu <- tkmenu(topMenu,tearoff=FALSE)
  
  tkadd(normalMenu,"command",label="By grid", state ="active", command=function(){
    normalized.gui(1,txt, img)
  })
  tkadd(normalMenu,"command",label="Global", state ="active", command=function(){
    normalized.gui(2,txt, img)
  })
}
else if(swap==1){
   tkadd(duplicatesMenu,"command",label="Mean Replicate Filtering", command = function(){
    remove.duplicates.gui(1,txt,img); assign("unique.alg", 1, envir = envir)})
  tkadd(duplicatesMenu,"command",label="Non-extreme values replicate filtering", command = function(){
    remove.duplicates.gui(2, txt, img); assign("unique.alg", 2, envir = envir)})
  tkadd(duplicatesMenu,"command",label="Geometric Mean Filter", command = function(){
    remove.duplicates.gui(3, txt, img); assign("unique.alg",3, envir = envir)})

    tkadd(filterMenu,"cascade",label="Duplicates Analysis", state ="active",menu= duplicatesMenu)
  
  tkadd(topMenu,"cascade",label="Filter",menu =filterMenu)
  
  normalMenu <- tkmenu(topMenu,tearoff=FALSE)
  
  tkadd(normalMenu,"command",label="By grid", state ="active", command=function(){
    normalized.gui(1,txt, img)
    assign("norm.alg", 1, envir = envir)
  })
  tkadd(normalMenu,"command",label="Global", state ="active", command=function(){
    normalized.gui(2,txt, img)
    assign("norm.alg", 2, envir = envir)
  })
 }
  

  cys.plot <-  function(){
    assign("graphic.type", 1, envir = envir)
    tkrreplot(img,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))}

  tkadd(topMenu,"cascade",label="Normalize",menu = normalMenu)
  graphicMenu <- tkmenu(topMenu,tearoff=FALSE)
  tkadd(graphicMenu,"command",label="log(Cy3) vs log(Cy5)",command = function() {
    cys.plot()})
  tkadd(graphicMenu,"command",label="R vs I",command = function() {
    assign("graphic.type", 2, envir = envir)
    tkrreplot(img,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))})
  tkadd(graphicMenu,"command",label="M vs A",command = function() {
    assign("graphic.type", 3, envir = envir)
    tkrreplot(img,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))})
  tkadd(topMenu,"cascade",label="Graphics",menu =graphicMenu)
  
  ZscoreMenu <- tkmenu(topMenu,tearoff=FALSE)
  tkadd(topMenu,"cascade",label="Zscore",menu = ZscoreMenu)
  tkadd(ZscoreMenu,"command",label="R vs I", state="active", command=function(){
    zscore.gui()
  })
  tkadd(ZscoreMenu,"command",label="M vs A", state="active",command=function(){
    assign("Zscore.spot", Zscore(get("a.spot",envir=envir), type = "ma"), envir = envir)
    write.zscore(get("Zscore.spot", envir = envir),file.path(get("path.results", envir = envir), "zscore.txt"))
    set.history.project(get("history.project", envir = envir), "MA Zscore", "zscore.txt")
    Zscore.points(type="ma",text=tclvalue(tkget(txt,"0.0","end")), envir,swap)
    tkdestroy(tt)
  })
  
  optionsMenu <- tkmenu(topMenu,tearoff=FALSE)
  tkadd(optionsMenu,"command",label="Save graphic as PDF",command = function(){
    name <- tclvalue(tkgetSaveFile(initialdir = get("path.graphics", envir = envir),
                                   initialfile=get("spot.name",envir=envir),
                                   filetypes="{{PDF Files} {.pdf}} {{All files} *}"))
    name <- unlist(strsplit(name, "\\."))[1]
    if (!nchar(name))
      tkmessageBox(parent = tt,  message= "You must write a name of file!", icon = "error", default = "ok")
    else{
      rbVal <- as.integer(tclvalue(dist))
      gt <- get("graphic.type", envir = envir)
      switch(rbVal,
             {if(gt == 1){
               pdf(paste(name, "OriginalCy3vsCy5.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "OriginalCy3vsCy5.pdf", sep = "_"))}
              else if(gt == 2){
                pdf(paste(name, "OriginalRvsI.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "OriginalRvsI.pdf", sep = "_"))}
              else{
                pdf(paste(name, "OriginalMvsA.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "OriginalMvsA.pdf", sep = "_"))};
              graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir))
              dev.off(dev.cur())},
             {if(gt == 1){
               pdf(paste(name, "CorrectedCy3vsCy5.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "CorrectedCy3vsCy5.pdf", sep = "_"))}
              else if(gt == 2){
                pdf(paste(name, "CorrectedRvsI.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "CorrectedRvsI.pdf", sep = "_"))}
              else{
                pdf(paste(name, "CorrectedMvsA.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "CorrectedMvsA.pdf", sep = "_"))};
              graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir))
              dev.off(dev.cur())},
             {if(gt == 1){
               pdf(paste(name, "NormalCy3vsCy5.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "NormalCy3vsCy5.pdf", sep = "_"))}
              else if(gt == 2){
                pdf(paste(name, "NormalRvsI.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "NormalRvsI.pdf", sep = "_"))}
              else{
                pdf(paste(name, "NormalMvsA.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "NormalMvsA.pdf", sep = "_"))};
              graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir))
              dev.off(dev.cur())},
             {if(gt == 1){
               pdf(paste(name, "FilterCy3vsCy5.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "FilterCy3vsCy5.pdf", sep = "_"))}
              else if(gt == 2){
                pdf(paste(name, "FilterRvsI.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "FilterRvsI.pdf", sep = "_"))}
              else{
                pdf(paste(name, "FilterMvsA.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "FilterMvsA.pdf", sep = "_"))};
              graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir))
              dev.off(dev.cur())},
             {if(gt == 1){
               pdf(paste(name, "NoDuplicatesCy3vsCy5.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "NoDuplicatesCy3vsCy5.pdf", sep = "_"))}
              else if(gt == 2){
                pdf(paste(name, "NoDuplicatesRvsI.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "NoDuplicatesRvsI.pdf", sep = "_"))}
              else{
                pdf(paste(name, "NoDuplicatesMvsA.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "NoDuplicatesMvsA.pdf", sep = "_"))};
              graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir))
              dev.off(dev.cur())})
    }
  })
  tkadd(optionsMenu,"command",label="Notes",command = function() note(envir))
  tkadd(topMenu,"cascade",label="Options",menu =optionsMenu)
    
  
  helpMenu <- tkmenu(topMenu,tearoff=FALSE)
  tkadd(helpMenu,"command",label="About genArise...", command=function() help())
  tkadd(topMenu,"cascade",label="Help",menu = helpMenu)

  sub.menu.frame <- tkframe(tt, relief = "groove", border = "2")
  
  image.menu.frame <- tkframe(sub.menu.frame, relief = "groove", border = "2")
  
  tcl("image","create","photo","filter",file=file.path(get("icons.dir", envir = envir),"icons/filter.png"))
  filter.button <- tkbutton(image.menu.frame,image="filter", command = function(){filter.gui(txt, img)})
  tkconfigure(filter.button, heigh = 16, width = 16)
  
  tcl("image","create","photo","normal",file=file.path(get("icons.dir", envir = envir),"icons/norm.png"))
  normal.button <- tkbutton(image.menu.frame,image="normal", command = function(){normalized.gui(1, txt, img)
                                                                                if(swap==1){
                                                                                  assign("norm.alg", 1, envir = envir)}
                                                                                }) 
  tkconfigure(normal.button, heigh = 16, width = 16)
  
  tcl("image","create","photo","zscore",file=file.path(get("icons.dir", envir = envir),"icons/zscore.png"))
  zscore.button <- tkbutton(image.menu.frame,image="zscore", command = function() zscore.gui())
  tkconfigure(zscore.button, heigh = 16, width = 16)
  
  tcl("image","create","photo","cys",file=file.path(get("icons.dir", envir = envir),"icons/cy3vscy5.png"))
  cys.button <- tkbutton(image.menu.frame,image="cys", command = function() cys.plot())
  tkconfigure(cys.button, heigh = 16, width = 16)

  tcl("image","create","photo","acroread",file=file.path(get("icons.dir", envir = envir),"icons/acroread.png"))
  acroread.button <- tkbutton(image.menu.frame,image="acroread", command = function(){
    name <- tclvalue(tkgetSaveFile(initialdir = get("path.graphics", envir = envir),
                                   initialfile=get("spot.name",envir=envir),
                                   filetypes="{{PDF Files} {.pdf}} {{All files} *}"))
    name <- unlist(strsplit(name, "\\."))[1]
    if (!nchar(name))
      tkmessageBox(parent = tt,  message= "You must write a name of file!", icon = "error", default = "ok")
    else{
      rbVal <- as.integer(tclvalue(dist))
      gt <- get("graphic.type", envir = envir)
      switch(rbVal,
             {if(gt == 1){
               pdf(paste(name, "OriginalCy3vsCy5.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "OriginalCy3vsCy5.pdf", sep = "_"))}
              else if(gt == 2){
                pdf(paste(name, "OriginalRvsI.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "OriginalRvsI.pdf", sep = "_"))}
              else{
                pdf(paste(name, "OriginalMvsA.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "OriginalMvsA.pdf", sep = "_"))};
              graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir))
              dev.off(dev.cur())},
             {if(gt == 1){
               pdf(paste(name, "CorrectedCy3vsCy5.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "CorrectedCy3vsCy5.pdf", sep = "_"))}
              else if(gt == 2){
                pdf(paste(name, "CorrectedRvsI.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "CorrectedRvsI.pdf", sep = "_"))}
              else{
                pdf(paste(name, "CorrectedMvsA.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "CorrectedMvsA.pdf", sep = "_"))};
              graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir))
              dev.off(dev.cur())},
             {if(gt == 1){
               pdf(paste(name, "NormalCy3vsCy5.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "NormalCy3vsCy5.pdf", sep = "_"))}
              else if(gt == 2){
                pdf(paste(name, "NormalRvsI.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "NormalRvsI.pdf", sep = "_"))}
              else{
                pdf(paste(name, "NormalMvsA.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "NormalMvsA.pdf", sep = "_"))};
              graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir))
              dev.off(dev.cur())},
             {if(gt == 1){
               pdf(paste(name, "FilterCy3vsCy5.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "FilterCy3vsCy5.pdf", sep = "_"))}
              else if(gt == 2){
                pdf(paste(name, "FilterRvsI.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "FilterRvsI.pdf", sep = "_"))}
              else{
                pdf(paste(name, "FilterMvsA.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "FilterMvsA.pdf", sep = "_"))};
              graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir))
              dev.off(dev.cur())},
             {if(gt == 1){
               pdf(paste(name, "NoDuplicatesCy3vsCy5.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "NoDuplicatesCy3vsCy5.pdf", sep = "_"))}
              else if(gt == 2){
                pdf(paste(name, "NoDuplicatesRvsI.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "NoDuplicatesRvsI.pdf", sep = "_"))}
              else{
                pdf(paste(name, "NoDuplicatesMvsA.pdf", sep = "_"),horiz=F, height=8,width=8,title=paste(name, "NoDuplicatesMvsA.pdf", sep = "_"))};
              graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir))
              dev.off(dev.cur())})
    }
  }) 
  tkconfigure(acroread.button, heigh = 16, width = 16)
  
  tcl("image","create","photo","editor",file=file.path(get("icons.dir", envir = envir),"icons/editor.png"))
  editor.button <- tkbutton(image.menu.frame,image="editor", command = function() note(envir))
  tkconfigure(editor.button, heigh = 16, width = 16)
  
  tcl("image","create","photo","help",file=file.path(get("icons.dir", envir = envir),"icons/help.png"))
  help.button <- tkbutton(image.menu.frame,image="help",command = function() help())
  tkconfigure(help.button, heigh = 16, width = 16)

  tcl("image","create","photo","back",file=file.path(get("icons.dir", envir = envir),"icons/regresar.png"))
  back.button <- tkbutton(image.menu.frame,image="back",command =  function(){
    sure <- back.gui(envir)
    if(sure == 0){
      tkdestroy(tt)
      principal(envir,0)
    }
  } )
  tkconfigure(back.button, heigh = 16, width = 16)
  
  etiqueta <- tklabel(sub.menu.frame, text = "")
  tkconfigure(etiqueta, text = "", width = "80") 
  
  
  tkgrid(filter.button, normal.button, zscore.button, cys.button, acroread.button, editor.button, back.button, help.button, sticky = "nw")
  tkgrid(image.menu.frame, etiqueta, sticky = "w")
  tkgrid(sub.menu.frame)
  
  label.function <-  function(texto.etiqueta) tkconfigure(etiqueta, text = texto.etiqueta, width = "80")
  
  
  tkbind(filter.button, "<Enter>", function() label.function("Filter spots by intensity"))
  tkbind(filter.button, "<Leave>", function() label.function(""))
  
  tkbind(normal.button, "<Enter>", function()  label.function("Grid normalized spot"))
  tkbind(normal.button, "<Leave>", function() label.function(""))
  
  tkbind(zscore.button, "<Enter>", function() label.function("Obtain the z-score value"))
  tkbind(zscore.button, "<Leave>", function() label.function(""))

  tkbind(cys.button, "<Enter>", function() label.function("Plot log(Cy5) vs log(Cy3) values"))
  tkbind(cys.button, "<Leave>", function() label.function(""))
  
  tkbind(acroread.button, "<Enter>", function() label.function("Save graphic as PDF"))
  tkbind(acroread.button, "<Leave>", function() label.function(""))
  
  tkbind(editor.button, "<Enter>", function() label.function("Notes about this experiment"))
  tkbind(editor.button, "<Leave>", function() label.function(""))
 
  tkbind(back.button, "<Enter>", function() label.function("Back to the last window"))
  tkbind(back.button, "<Leave>", function() label.function(""))
  
  tkbind(help.button, "<Enter>", function() label.function("Help about genArise"))
  tkbind(help.button, "<Leave>", function() label.function(""))

  
  
  frame.label <- tkframe(tt,relief="groove",borderwidth=2)
  tkfocus(tt)
  assign("a.spot", get("o.spot", envir = envir), envir = envir)
  img <- tkrplot(upper.frame,fun = function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)), hscale= get("Myhscale", envir = envir),vscale = get("Myvscale",
                                                                                                                                                                        envir = envir))
  tkgrid(img,frame1,padx="10",pady="10")
  tkgrid(upper.frame, pady = "10")
  
  area.frame <- tkframe(tt,relief="groove")
  yscr <- tkscrollbar(area.frame, repeatinterval = 5, command=function(...)tkyview(txt,...))
  txt <- tktext(area.frame,bg="white",font="courier",width =get("barWidth",envir=envir),height=get("barHeight",envir=envir),yscrollcommand=function(...)tkset(yscr,...))
  tkgrid(txt, yscr)
  tkgrid.configure(yscr,rowspan=4,sticky="nsw")
  tkinsert(txt,"end",texto)
  tkconfigure(txt, state="disabled")
  tkgrid(area.frame, padx = "10", pady = "10", sticky = "ew")
  if(follow.wizard){
    if(swap==0){
      op.counter <<- op.counter + 1
      write.spot(get("o.spot", envir = envir), paste(get("path.results", envir = envir),
                                 .Platform$file.sep, "original.txt",sep = ""))
      set.history.project(get("history.project", envir = envir), "Original Spot", "original.txt")
      c.spot <- bg.correct(get("o.spot", envir = envir))
      op.counter <<- op.counter + 1
      tkconfigure(txt, state="normal")
      tkinsert(txt,"end","\n\nBackground correction done.............")
      tkconfigure(txt, state="disable")
      assign("c.spot", c.spot, envir = envir)
      write.spot(get("c.spot", envir = envir), paste(get("path.results", envir = envir),
                                 .Platform$file.sep, "corrected.txt",sep = ""))
      set.history.project(get("history.project", envir = envir), "Corrected Spot", "corrected.txt")
      assign("a.spot", c.spot, envir = envir)
      tkpack(corr.radio,anchor="w",padx="10")
      tkselect(corr.radio)
      assign("corrected",1, envir = envir)
      normalized.gui(1, txt, img)
      filter.gui(txt, img)   #disabled temp
      remove.duplicates.gui(3, txt, img)
    }
    else if(swap==1){
      op <- get("op.counter", envir = envir) + 1
      assign("op.counter", op, envir = envir)
      write.spot(get("o.spot", envir = envir), paste(get("path.results", envir = envir),
                                 .Platform$file.sep, "original.txt",sep = ""))
          set.history.project(get("history.project", envir = envir), "Original Spot", "original.txt")
      normalized.gui(1, txt, img)
      assign("norm.alg", 1, envir = envir)
      filter.gui(txt, img)   #disabled temp
      remove.duplicates.gui(3, txt, img)
      assign("unique.alg", 3, envir = envir)
    }
  }
}

genMerge <- function(gene.association, description, population.genes, study.genes, output.file = "GenMerge.txt"){

	gene.association.file <- read.csv(gene.association, header = FALSE, sep = "\t")
	des.file <- read.csv(description, header = FALSE, sep = "\t")
	population.file <- read.csv(population.genes, header = FALSE, sep = "\t")
	study.genes.file <- read.csv(study.genes, header = FALSE, sep = "\t")

	gene.association.file$V1 <- as.vector(gene.association.file$V1)
	gene.association.file$V2 <- as.vector(gene.association.file$V2)

	des.file$V1 <-  as.vector(des.file$V1)
	des.file$V2 <-  as.vector(des.file$V2)

	population.file$V1 <- as.vector(population.file$V1)
	total.no.detected.genes <- length(population.file$V1)
	GMRGgenomehash <- new.env(hash = TRUE)
	descriptionHash <- new.env(hash = TRUE)
	id.hash <- new.env(hash = TRUE)

	for(i in 1:total.no.detected.genes){
		assign(gene.association.file$V1[i], list( GO = gene.association.file$V2[i]), env = GMRGgenomehash)
	}

	for(i in 1:length(des.file$V1)){
		assign(des.file$V1[i], list( Description = des.file$V2[i]), env = descriptionHash)
	}

	key.gene.association <- ls(env = GMRGgenomehash)

	GMRGs.line.by.line <- list()

	for(i in 1:total.no.detected.genes){
		if(exists(population.file$V1[i], envir = GMRGgenomehash)){
			big.line <- get( population.file$V1[i], env = GMRGgenomehash)
			indivGMRGs <-  strsplit(big.line, ";")[[1]]
			GMRGs.line.by.line <- c(indivGMRGs, GMRGs.line.by.line, recursive = TRUE)
		}
	}

	tmp <- GMRGs.line.by.line[sort.list(GMRGs.line.by.line)]

	unique.list <- rle(tmp)

	output.hash <- new.env( hash = TRUE)

	for( i in 1:length(unique.list$values)){
		assign(unique.list$values[i], list(count = unique.list$lengths[i], frequence = unique.list$lengths[i]/total.no.detected.genes), env = output.hash)
	}

	study.genes.file$V1 <- as.vector(study.genes.file$V1)
	total.no.updown.genes <- length(study.genes.file$V1)
	updownGMRGs.line.by.line <- list()
	genes.not.found.in.ontology <- list()

	for(i in 1:total.no.updown.genes){
		if(exists(study.genes.file$V1[i], envir = GMRGgenomehash)){
		bigline <- get(study.genes.file$V1[i], env = GMRGgenomehash)
			updownGMRGs <-  strsplit(bigline, ";")[[1]]
			for(k in 1:length(updownGMRGs)){
				if(exists(updownGMRGs[k], envir = id.hash)){
					unique.list1 <- get( updownGMRGs[k], env = id.hash)
					unique.list <- paste(unique.list1, study.genes.file$V1[i], sep = " ")
					assign(updownGMRGs[k], unique.list,  env = id.hash)
				}
				else{
					assign(updownGMRGs[k], study.genes.file$V1[i], env = id.hash)
				}
			}
			updownGMRGs.line.by.line <- c(updownGMRGs, updownGMRGs.line.by.line, recursive = TRUE)
		}
		else{
			genes.not.found.in.ontology <- c(study.genes.file$V1[i], genes.not.found.in.ontology, recursive = TRUE)
		}
	}

	tmp <- updownGMRGs.line.by.line[sort.list(updownGMRGs.line.by.line)]

	unique.list <- rle(tmp)

	output2 <- list(uniqueUpDownGMRGIDs = unique.list$values , frequence = unique.list$lengths)

	BCr <- 0
	frequences <- list()
	count.list <- list()

	for(i in 1:length(output2$uniqueUpDownGMRGIDs)){
		if(exists(output2$uniqueUpDownGMRGIDs[i], output.hash)){
			tmp <- get(output2$uniqueUpDownGMRGIDs[i], env = output.hash)
			frequences <- c(frequences,tmp$frequence, recursive = TRUE)
			count.list <- c( count.list, tmp$count, recursive = TRUE)
		}
	}

BCr <- length(frequences[frequences > (1/total.no.detected.genes)])

hyper <- function(n,p,k,r){
	q <- (1-p)
	np <- floor(n*p + 0.5)
	nq <- floor(n*q + 0.5)
	log.n.choose.k <- lchoose(n,k)
	top <- k
	if(np < k){
		top <- np
	}
	lfoo <- lchoose(np, top) + lchoose(n*(1-p),k-top)
	sum <- 0
	i <- top
	while(i >= r){
		sum <- sum + exp(lfoo - log.n.choose.k)
		if(i > r){
			lfoo <- lfoo + log(i/(np-i+1)) + log((nq-k+i)/(k-i+1))
		}
		i <- i - 1
	}
	sum
}

p.value <- list()
bonferroni <- list()

for(s in 1:length(frequences)){
	if(output2$frequence[s] != 1){
		p.v <- hyper(total.no.detected.genes,frequences[s] ,total.no.updown.genes,output2$frequence[s])
		p.value <- c(p.value, p.v)
		p.value.corrected <- p.v * BCr
		if(p.value.corrected >= 1){
			bonferroni <- c( bonferroni, 1, recursive = TRUE)
		}
		else{
			bonferroni <- c( bonferroni, p.value.corrected, recursive = TRUE)
		}
	}
	else{
		p.value<- c(p.value, NA, recursive = TRUE)
		bonferroni <- c(bonferroni, NA, recursive = TRUE)
	}
}

description.list <- list()

for( k in 1:length(frequences)){
if(exists(output2$uniqueUpDownGMRGIDs[k], envir = descriptionHash)){
	description.list <- c(description.list, get(output2$uniqueUpDownGMRGIDs[k], env = descriptionHash), recursive = TRUE)
}
}
# write output file

listita <- list()
for(k in 1:length(output2$uniqueUpDownGMRGIDs)){
	listita <- c(listita, get(output2$uniqueUpDownGMRGIDs[k], env = id.hash), recursive = TRUE)
}

pop.fac <- paste(count.list, "/", total.no.detected.genes, sep = "")
std.fac <- paste(output2$frequence, "/", total.no.updown.genes, sep = "")

tablota <- list( output2$uniqueUpDownGMRGIDs, frequences, pop.fac, std.fac, p.value, bonferroni, description.list, listita)

#Crear archivo de salida:
unlink(output.file)

objeto <- file( output.file, "w")

cat( paste("genMerge; ", output.file, sep = "  "), file = output.file, append = TRUE, sep = " ")
cat("\n", paste("Gene Association File: ", gene.association, sep = " "),  paste("Description File: ", description, sep = " "), paste("Population File: ", population.genes, sep = " "), paste("Study File: ", study.genes, sep = " "), "\n", "\n", file = output.file, append = TRUE, sep = "\n")

cat("GMRG_Term", "Pop_freq", "Pop_frac", "Study_frac", "Raw_es", "e-score", "Description", "Contributing_genes", append = TRUE, file = output.file, sep = "\t")

cat("\n", append = TRUE, file = output.file, sep = "")

write.table(tablota, output.file, col.names = FALSE, row.names = FALSE, append = TRUE,quote = FALSE, sep = "\t")
cat("\n", append = TRUE, file = output.file, sep = "")

cat (paste("Total number of genes:", total.no.detected.genes, sep = " "), append = TRUE,file = output.file)
cat("\n", file = output.file, append = TRUE,sep = "")
cat (paste("Total number of Study genes:", total.no.updown.genes, sep = " "), append = TRUE,file = output.file)
cat("\n", append = TRUE,file = output.file, sep = "")
cat (paste("Total number of Study gene GMRG terms (pop non-singletons):", length(output2$uniqueUpDownGMRGIDs),"(", BCr, ")",  sep = " "), append = TRUE,file = output.file)
cat("\n", append = TRUE, file = output.file, sep = "")

cat(paste("Genes with GMRG information:", total.no.updown.genes - length(genes.not.found.in.ontology), sep = " "), file = output.file,  append = TRUE)
cat("\n", append = TRUE, file = output.file, sep = "")
cat(paste("Genes with no GMRG information:", length(genes.not.found.in.ontology), sep = " "), file = output.file,  append = TRUE)
cat("\n", append = TRUE, file = output.file, sep = "")
cat("These are", append = TRUE,file = output.file, sep = "")
cat("\n", append = TRUE, file = output.file, sep = "")
cat(genes.not.found.in.ontology, file = output.file, append = TRUE)
close(objeto)
}

# Miscelaneous apps of genArise
require(xtable)

# Get the annotations for a given data set
annotations <-  function(specie.data, specie, column, symbol=NULL, output.file = "annotations.html"){
  species <- c("Mus musculus","Saccharomyces Cerevisiae","Rattus norvegicus", "NCBI")
  if(! is.element(specie,species))
    stop("The specie option must be one of the following: Mus musculus, Saccharomyces Cerevisiae, Rattus norvegicus or NCBI")

  the.specie <- which(specie == species)

  # send to the corresponding database
  switch(the.specie,
         {specie.data[,column] <- paste('<a href="http://www.informatics.jax.org/javawi2//servlet/WIFetch?page=searchTool&query=',
                                specie.data[,column],'&selectedQuery=Genes+and+Markers">', specie.data[,column], sep = "")
          if(!is.null(symbol))
          specie.data[,symbol] <- paste('<a href="http://www.informatics.jax.org/javawi2//servlet/WIFetch?page=searchTool&query=',specie.data[,symbol],'&selectedQuery=Genes+and+Markers">',specie.data[,symbol],sep="")
       },
         {specie.data[,column] <- paste('<a href="http://db.yeastgenome.org/cgi-bin/locus.pl?locus=',specie.data[,column],'">',
                                       specie.data[,column], sep = "")
          if(!is.null(symbol))
            specie.data[,symbol] <- paste('<a href="http://db.yeastgenome.org/cgi-bin/locus.pl?locus=',specie.data[,symbol],'">',specie.data[,symbol], sep = "")},
         {specie.data[,column] <-  paste('<a href="http://rgd.mcw.edu/generalSearch/RgdSearch.jsp?quickSearch=1&searchKeyword=',
                                         specie.data[,column],'">', specie.data[,column], sep = "")
          if(!is.null(symbol))
            specie.data[,symbol] <- paste('<a href="http://rgd.mcw.edu/generalSearch/RgdSearch.jsp?quickSearch=1&searchKeyword=', specie.data[,symbol],'">', specie.data[,symbol], sep = "")},
         {specie.data[,column] <- paste('<a href="http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?CMD=search&DB=gene&term=',
                                specie.data[,column],'">', specie.data[,column], sep = "")
          if(!is.null(symbol))
            specie.data[,symbol] <- paste('<a href="http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?CMD=search&DB=gene&term=',
                  specie.data[,symbol],'">', specie.data[,symbol], sep = "")})
  
  specie.data <- data.frame(specie.data)
  
                                        # construct the table with the html references
  annotation.table <- xtable(specie.data)
  print.xtable(annotation.table, type = "html", file = output.file)
}

# Create the file notes.txt for any important note of the researcher
note <-  function(envir){
  file.note <- file.path(get("path.project", envir = envir), "notes.txt")
  if(!is.element("notes.txt", get("path.project", envir = envir))){
    con <- file(file.note, "a")
    close(con)
  }
  system(paste(get("editor", envir = envir),  file.note, sep = " "))
}

                                   # Post analysis configuration Window
projects.select <- function(envir, nombre = "sets-analysis"){
  if(.Platform$OS.type == "unix")
    nombre <- paste(Sys.getenv("HOME"),nombre,sep=.Platform$file.sep)
  else{ if(.Platform$OS.type == "windows")
    nombre <- paste(Sys.getenv("R_USER"),nombre,sep=.Platform$file.sep)}
  tt <- tktoplevel()
  tkfocus(tt)
  tkwm.title(tt,"File Selector")
  upper.frame <- tkframe(tt)
  frameOverall <- tkframe(upper.frame)
  framerange <- tkframe(frameOverall,relief="groove",borderwidth=2)
  frameupdown <- tkframe(frameOverall,relief="groove",borderwidth=2)
  range  <- tclVar(1)
  updown <- tclVar(1)
  output <- tkentry(frameOverall,width="30", bg = "white", textvariable=tclVar(nombre))

  radio1 <- tkradiobutton(framerange, value=1, variable=range, command=function(){
    tkconfigure(output, state="normal")
    tkdelete(output,"0","end")
    switch(as.integer(tclvalue(updown)),
               
           {
                tkinsert(output,"end",paste(nombre,"1-1_5sd","Up",sep=""))
              },
           {
                 tkinsert(output,"end",paste(nombre,"1-1_5sd","Down",sep=""))
               })})
  radio2 <- tkradiobutton(framerange, value=2, variable=range,command=function(){
    tkconfigure(output, state="normal")
    tkdelete(output,"0","end")
    switch(as.integer(tclvalue(updown)),
           {
                tkinsert(output,"end",paste(nombre,"1_5-2sd","Up",sep=""))
              },
           {
                 tkinsert(output,"end",paste(nombre,"1_5-2sd","Down",sep=""))
               })})
  radio4 <- tkradiobutton(framerange, value=4, variable=range,command=function(){
    tkconfigure(output, state="normal")
    tkdelete(output,"0","end")
    switch(as.integer(tclvalue(updown)),
           {
             tkinsert(output,"end",paste(nombre,"Greater1.5sd","Up",sep=""))
           },
           {
             tkinsert(output,"end",paste(nombre,"Greater1.5sd","Down",sep=""))
           })})
  radio3 <- tkradiobutton(framerange, value=3, variable=range,command=function(){
    tkconfigure(output, state="normal")
    tkdelete(output,"0","end")
    switch(as.integer(tclvalue(updown)),
           {
             tkinsert(output,"end",paste(nombre,"Greater2sd","Up",sep=""))
           },
           {
             tkinsert(output,"end",paste(nombre,"Greater2sd","Down",sep=""))
           })})
    
  tkgrid(tklabel(framerange,text="Select the range"),pady = "2",padx="10")
  tkgrid(tklabel(framerange,text="1 - 1.5 sd"),radio1,pady = "2",padx="10")
  tkgrid(tklabel(framerange,text="1.5 - 2 sd"),radio2,pady = "2",padx="10")
  tkgrid(tklabel(framerange,text="more than 1.5 sd"),radio4,pady = "2",padx="10")
  tkgrid(tklabel(framerange,text="more than 2 sd"),radio3,pady = "2",padx="10")
  radioup <- tkradiobutton(frameupdown, value=1, variable=updown, command=function(){
    tkconfigure(output, state="normal")
    tkdelete(output,"0","end")
    switch(as.integer(tclvalue(range)),
           {
                tkinsert(output,"end",paste(nombre,"1-1_5sd","Up",sep=""))
              },
           {
                 tkinsert(output,"end",paste(nombre,"1_5-2sd","Up",sep=""))
               },
            {
                tkinsert(output,"end",paste(nombre,"Greater1_5sd","Up",sep=""))
              },
           {
                 tkinsert(output,"end",paste(nombre,"Greater2sd","Up",sep=""))
               }
           )})
  radiodown <- tkradiobutton(frameupdown, value=2, variable=updown, command=function(){
    tkconfigure(output, state="normal")
    tkdelete(output,"0","end")
    switch(as.integer(tclvalue(range)),
           {
                tkinsert(output,"end",paste(nombre,"1-1_5sd","Down",sep=""))
              },
           {
                 tkinsert(output,"end",paste(nombre,"1_5-2sd","Down",sep=""))
               },
           {
                tkinsert(output,"end",paste(nombre,"Greater1_5sd","Down",sep=""))
              },
           {
                 tkinsert(output,"end",paste(nombre,"Greater2sd","Down",sep=""))
               }
           )})
  tkgrid(tklabel(frameupdown,text="Select the desired option"),pady = "2")
  tkgrid(tklabel(frameupdown,text="Up-regulated"),radioup,pady = "2")
  tkgrid(tklabel(frameupdown,text="Down-regulated"),radiodown,pady = "2")
  tkgrid(tklabel(frameOverall,text="Where you want to save the output"), padx = "10", pady = "10")
  
  out.function <-function(){
    dirname <- tclvalue(tkgetSaveFile())
    if (!nchar(dirname))
      tkmessageBox(message="No file was selected!",icon = "error", default = "ok")
    else{
      tkconfigure(output,textvariable = tclVar(dirname))
      nombre <<- dirname
      }
  }
  
  out.but <- tkbutton(frameOverall,text="Browse",command=function() out.function())

  tkgrid(output,out.but, padx = "5")
  
  left.frame <-  tkframe(upper.frame,relief = "groove", borderwidth=2)
  
  tree.frame <- tkframe(left.frame, relief = "groove", borderwidth=2)
  xscr <- tkscrollbar(tree.frame, repeatinterval=5, command = function(...)tkxview(files.list,...),orient="horizontal")
  yScr <- tkscrollbar(tree.frame, command = function(...)tkyview(files.list,...), orient = "vertical")
  files.list <- tklistbox(tree.frame, background = "white", xscrollcommand=function(...)tkset(xscr,...),
                          yscrollcommand=function(...)tkset(yScr,...))
  tkconfigure(files.list, width = "30", height = "10")
  
  tkgrid(files.list,yScr)
  tkgrid.configure(yScr,stick="nsw")
  tkgrid(xscr)
  tkgrid.configure(xscr,stick="new")
  files.frame <- tkframe(tree.frame)
  
  onDelete <-  function(){
    selected.file <-  tkcurselection(files.list)
    if(length(as.character(selected.file)) > 0)
      tkdelete(files.list, as.integer(selected.file))
  }
  
  onAdd <-  function(){
    tempo <- tclvalue(tkgetOpenFile(filetypes = "{{PRJ Files} {.prj}} {{All files} *}",multiple = "1"))
    tempo <- unlist(strsplit(tempo," "))
    if(length(tempo)!= 0){
      for(i in 1:length(tempo)){ 
        if(nchar(tempo[i])){
          tkinsert(files.list,"end",tempo[i])
        }
      }
    }
  }
  
  files.button.frame <- tkframe(left.frame)
  
  delete.button <-  tkbutton(files.button.frame, text = "Delete file", command = onDelete)
  
  add.button <-  tkbutton(files.button.frame, text = "Add file", command = onAdd)
  
  tkgrid(tklabel(left.frame,text = "Select all the .prj files with which you \nwill perform the combinatorial analysis"), padx = "10", pady = "10")
  tkgrid(tree.frame, pady = "10")
  tkgrid(add.button, delete.button, padx = "20")
  tkgrid(files.button.frame, pady= "10")

  button.frame <- tkframe(tt, borderwidth=2)
  
  operacion <- function(){
    values <- get.paths()
    if (length(values) < 1)
      tkmessageBox(message="No file was selected!",icon = "error", default = "ok")
    else{
      switch(as.integer(tclvalue(range)),
              { minimo <- 1; maximo <- 1.5
              },
             {minimo <- 1.5; maximo <- 2
            },
             {minimo <- 2; maximo <- NULL
            },
             {minimo <- 1.5; maximo <- NULL
            }
             )
      if(as.integer(tclvalue(updown)) == 1){
        up.down <- "up"
      }else{
        up.down <- "down"
      }
      dirname <- substr(tclvalue(tkget(output)),0,nchar(tclvalue(tkget(output))))
      post.analysis(values,
                    minimo, maximo, up.down = up.down, output = dirname)


    }
  }

  ok.but <- tkbutton(button.frame, text ="OK", command = function(){
    tkconfigure(tt, cursor="watch")
    operacion()
    tkconfigure(tt, cursor="arrow")
    tkmessageBox(title = "Post-Analysis", icon="info", message=paste("The Post-Analysis has done.\n The results files are in this directory:", tclvalue(tkget(output))), type="ok")
  })
  
  cancel.but <- tkbutton(button.frame, text ="Cancel", command = function(){tkdestroy(tt)
                                                                            genArise.init(envir)})
  
  tkgrid(framerange,pady="10")
  tkgrid(frameupdown,pady="10")
  tkgrid(left.frame, frameOverall,padx = "10", pady = "10")
  tkgrid(ok.but, cancel.but, padx = "20")
  tkgrid(upper.frame, pady = "10")
  tkgrid(button.frame)
  
  get.paths <- function(){
    list.projects <-  list()
    tkselection.set(files.list,"0", "end")
    selected.file <-  tkcurselection(files.list)
    paths <- unlist(strsplit(tclvalue(selected.file), " "))
    for(i in 1:length(paths)){
      list.projects <-  c(list.projects, tclvalue(tkget(files.list, paths[as.numeric(i)])), recursive = TRUE)
    }
    list.projects
  }
}

# Post analysis configuration Window

swap.select <- function(envir){
  tt <- tktoplevel()
  tkfocus(tt)
  active.options <- function(is.ifc = TRUE){
    if(is.ifc){
      state <- "disable"
    }else{
      state <- "normal"
    }
    tkconfigure(titulo2, state= state)
    tkconfigure(rows.label, state= state)
    tkconfigure(columns.label, state= state)
    tkconfigure(meta.rows.label, state= state)
    tkconfigure(meta.columns.label, state= state)
    tkconfigure(rows.entry, state= state)
    tkconfigure(columns.entry, state= state)
    tkconfigure(meta.rows.entry, state= state)
    tkconfigure(meta.columns.entry, state= state)
  }
  tkwm.resizable(tt,0,0)
  tkwm.title(tt,"File Selector")
  upper.frame <- tkframe(tt)
  right.frame <-  tkframe(upper.frame,relief = "groove")
  frameOverall <- tkframe(right.frame)
  left.frame <-  tkframe(upper.frame,relief = "groove", borderwidth=2)
  tree.frame <- tkframe(left.frame, relief = "groove", borderwidth=2)
  button.frame <- tkframe(right.frame, borderwidth=2)
  
  xscr <- tkscrollbar(tree.frame, repeatinterval=5, command = function(...)tkxview(files.list,...),orient="horizontal")
  yScr <- tkscrollbar(tree.frame, command = function(...)tkyview(files.list,...), orient = "vertical")
  files.list <- tklistbox(tree.frame, background = "white", xscrollcommand=function(...)tkset(xscr,...),
                          yscrollcommand=function(...)tkset(yScr,...))
  tkconfigure(files.list, width = "30", height = "10")
  tkgrid(files.list,yScr)
  tkgrid.configure(yScr,sticky="nsw")
  tkgrid(xscr)
  tkgrid.configure(xscr,sticky="new")
  rbValue <- tclVar("IFC")
  
  spot.frame <- tkframe(frameOverall, relief = "groove", borderwidth=2)
  radio.frame <- tkframe(spot.frame)
  
  rb1 <- tkradiobutton(radio.frame, text = "Foreign", value = "Foreign", variable = rbValue, command = function(){active.options(FALSE)})
  rb2 <- tkradiobutton(radio.frame, text = "IFC", value = "IFC", variable = rbValue, command = function(){active.options(TRUE)})
  tkgrid(tklabel(spot.frame,text="Input"), padx = "10", pady = "2")
  tkgrid(rb1, rb2, sticky = "w", pady = "2",padx="70")
  tkgrid(radio.frame,padx="5", pady = "10")
  tkgrid(tklabel(spot.frame, text = "Fill the text entries with the correct columns in your file"), columnspan=11)
  columns.frame <- tkframe(spot.frame)
  symbol.frame <- tkframe(spot.frame)
  Cy3 <- tclVar("")
  Cy5 <- tclVar("")
  BgCy3 <- tclVar("")
  BgCy5 <- tclVar("")
  Id <- tclVar("")
  Symbol <- tclVar("")
  tkgrid(tklabel(columns.frame,text="Cy3"),tkentry(columns.frame,width="5", textvariable=Cy3, bg = "white"),
         tklabel(columns.frame,text="Cy5"),tkentry(columns.frame,width="5", textvariable=Cy5, bg = "white"),
         tklabel(columns.frame,text="BgCy3"),tkentry(columns.frame,width="5", textvariable=BgCy3, bg = "white"),
         tklabel(columns.frame,text="BgCy5"),tkentry(columns.frame,width="5", textvariable=BgCy5, bg = "white"),
         tklabel(columns.frame,text="Id"),tkentry(columns.frame,width="5", textvariable=Id ,bg = "white"),pady=20,padx="4")
  tkgrid(tklabel(symbol.frame,text="Symbol"),tkentry(symbol.frame,width="5",textvariable=Symbol, bg = "white"), tklabel(symbol.frame,text="[optional]"))
  tkgrid(columns.frame)
  tkgrid(symbol.frame)
  
    Row <- tclVar("")
    Col <- tclVar("")
    MR <- tclVar("")
    MC <- tclVar("")

  rc.frame <- tkframe(spot.frame,borderwidth=2)
  titulo2 <- tklabel(spot.frame, text = "Especify the grid dimensions")
  
  rows.label <- tklabel(rc.frame,text="Rows")
  columns.label <- tklabel(rc.frame,text="Columns")
  meta.rows.label <- tklabel(rc.frame,text="Meta-rows")
  meta.columns.label <- tklabel(rc.frame,text="Meta-columns")
  rows.entry <- tkentry(rc.frame,width="5",textvariable=Row, bg = "white")
  columns.entry <- tkentry(rc.frame,width="5",textvariable=Col, bg = "white")
  meta.rows.entry <- tkentry(rc.frame,width="5",textvariable=MR, bg = "white")
  meta.columns.entry <- tkentry(rc.frame,width="5",textvariable=MC, bg = "white")
  
  tkgrid(titulo2, columnspan=11,pady=10)
  tkgrid(rows.label, rows.entry, columns.label, columns.entry, meta.rows.label,meta.rows.entry, meta.columns.label, meta.columns.entry)
  active.options(TRUE)
  tkgrid(rc.frame,padx="10", padx = "10")
  tkgrid(spot.frame,padx="5", pady = "10")
  
  lower.frame <- tkframe(frameOverall,relief="groove",borderwidth=2)
  tkgrid(tklabel(lower.frame,text="Output"), padx = "10", pady = "10")
  output.frame <- tkframe(lower.frame)
  project.entry <- tkentry(output.frame,width="30", bg = "white", textvariable= tclVar(""))
#  name.project <-  tclvalue(tkget(project.entry))
 
  browse2.function <- function(){
    fileName <- tclvalue(tkgetSaveFile())
    if (!nchar(fileName))
      tkmessageBox(message="No file was selected!",icon = "error", default = "ok")
    else{
      tkconfigure(project.entry, textvariable = tclVar(fileName))
    }
  }
  
  project.but <- tkbutton(output.frame,text="browse...",    command=function() browse2.function())
  tkgrid(tklabel(output.frame,text="Project Name:"), project.entry, project.but, padx = "5", sticky = "w", pady = "2")
  tkbind(project.but, "<Return>", browse2.function)
  
  graphic.file <- tclVar("Plots")
  output.file <- tclVar("Results")
  
  graphic.entry <- tkentry(output.frame,width="10", bg = "white", textvariable = graphic.file)
  output.entry  <- tkentry(output.frame,width="10", bg = "white", textvariable = output.file)
  
  
  tkgrid(tklabel(output.frame,text="Location for Plots:"), graphic.entry, sticky = "w", padx = "5", pady = "2")
  tkgrid(tklabel(output.frame,text="Location for Results:"), output.entry,  sticky = "w", padx = "5", pady = "2")
  tkgrid(output.frame, padx="5")
  tkgrid(lower.frame, padx="5")
  
  
  files.frame <- tkframe(tree.frame)
  
  onDelete <-  function(){
    selected.file <-  tkcurselection(files.list)
    if(length(as.character(selected.file)) > 0)
      tkdelete(files.list, as.integer(selected.file))
  }
  
  onAdd <-  function(){
    tempo <- tclvalue(tkgetOpenFile(filetypes = "{{CSV Files} {.csv}} {{TXT Files} {.txt}} {{All files} *}"))
   # tempo <- unlist(strsplit(tempo," "))
    for(i in 1:length(tempo)){    
      if(nchar(tempo[i])){
        tkinsert(files.list,"end",tempo[i])
      }
      else
        tkmessageBox(message="No file was selected!",icon = "error", default = "ok")
    }
  }
  
  files.button.frame <- tkframe(left.frame)
  delete.button <-  tkbutton(files.button.frame, text = "Delete file", command = onDelete)
  add.button <-  tkbutton(files.button.frame, text = "Add file", command = onAdd)
  tkgrid(tklabel(left.frame,text = "Select the swap dye data file pair with \nwhich you will perform the analysis"), padx = "10", pady = "10")
  tkgrid(tree.frame, pady = "10")
  tkgrid(add.button, delete.button, padx = "20")
  tkgrid(files.button.frame, pady= "10")
  ok.but <- tkbutton(button.frame, text ="OK", command = function(){
    tkconfigure(tt, cursor="watch")
    operacion()
    tkconfigure(tt, cursor="arrow")
    tkdestroy(tt)
  })

  cancel.but <- tkbutton(button.frame, text ="Cancel", command = function(){
    tkdestroy(tt)
    genArise()})
  tkgrid(ok.but, cancel.but, padx = "20")
  tkgrid(frameOverall,padx = "10", pady = "10")

  tkgrid(left.frame,right.frame, pady = "10")
  tkgrid(upper.frame)
  tkgrid(button.frame,pady="30")
  get.paths <- function(){
    list.projects <-  list()
    tkselection.set(files.list,"0", "end")
    selected.file <-  tkcurselection(files.list)
    paths <- unlist(strsplit(tclvalue(selected.file), " "))
    for(i in 1:length(paths)){
      list.projects <-  c(list.projects, tclvalue(tkget(files.list, paths[as.numeric(i)])), recursive = TRUE)
    }  
    list.projects
  }
  
  ##############################################################

  ##############################################################
  operacion <- function(){
    values <- get.paths()
    if(!nchar(tclvalue(tkget(project.entry))) ||
       !nchar(tclvalue(graphic.file)) || !nchar(tclvalue(output.file))){
      tkmessageBox(message= "No output file was specified! ", icon = "error", default = "ok")
    }
    else{
      name.project <-  tclvalue(tkget(project.entry))
      assign("path", values[1], envir = envir)
      name.tmp <- unlist(strsplit(values[1], paste("\\", .Platform$file.sep, sep = "")))
      tmp <- unlist(strsplit(name.tmp[length(name.tmp)], "\\."))
      name.tmp <- paste(tmp[-length(tmp)],sep=".",collapse=".")
      assign("spot.name", name.tmp, envir = envir)
      set.path.project(name.project, tclvalue(output.file),
                       tclvalue(graphic.file), envir)
      
      assign("op.counter",0,envir = envir)
      if(as.character(tclvalue(rbValue)) == "IFC"){
        if(as.character(tclvalue(Symbol)) == ""){
          make.swap(values[1],values[2],as.numeric(tclvalue(Cy3)),as.numeric(tclvalue(Cy5)),as.numeric(tclvalue(BgCy3)),as.numeric(tclvalue(BgCy5)),as.numeric(tclvalue(Id)), Symdesc=NULL, header = TRUE, is.ifc = TRUE,envir=envir,0,0)
        }
        else{
          make.swap(values[1],values[2],as.numeric(tclvalue(Cy3)),as.numeric(tclvalue(Cy5)),as.numeric(tclvalue(BgCy3)),as.numeric(tclvalue(BgCy5)),as.numeric(tclvalue(Id)), as.numeric(tclvalue(Symbol)), header = TRUE, is.ifc = TRUE,envir=envir,0,0)
        }
      }else{
        if(as.character(tclvalue(Symbol)) == ""){
          make.swap(values[1],values[2],as.numeric(tclvalue(Cy3)),as.numeric(tclvalue(Cy5)),as.numeric(tclvalue(BgCy3)),as.numeric(tclvalue(BgCy5)),as.numeric(tclvalue(Id)), Symdesc=NULL, header = FALSE, is.ifc = FALSE,envir=envir,as.numeric(tclvalue(Row)),as.numeric(tclvalue(Col)))
        }
        else{
          make.swap(values[1],values[2],as.numeric(tclvalue(Cy3)),as.numeric(tclvalue(Cy5)),as.numeric(tclvalue(BgCy3)),as.numeric(tclvalue(BgCy5)),as.numeric(tclvalue(Id)), as.numeric(tclvalue(Symbol)), header = FALSE, is.ifc = FALSE,envir=envir,as.numeric(tclvalue(Row)),as.numeric(tclvalue(Col)))
        }
      }
      create.project(name.project,tclvalue(output.file), tclvalue(graphic.file))
      history.project <- paste(name.project,".prj", sep = "")
      set.history.project(history.project, "Name:", get("spot.name", envir = envir))
      set.history.project(history.project, "Rows", get("nr.global", envir = envir))
      set.history.project(history.project, "Columns", get("nc.global", envir = envir))
      set.history.project(history.project, "MetaRows", get("nmr.global", envir = envir))
      set.history.project(history.project, "MetaColumns", get("nmc.global", envir = envir))
      assign("history.project",history.project, envir = envir)
      write.spot(get("o.spot", envir = envir), paste(get("path.results", envir = envir),
                                 .Platform$file.sep, "original.txt",sep = ""))
      set.history.project(get("history.project", envir = envir), "Original Spot", "original.txt")
      principal(envir,1)
    }
  }
}

# Main Window of genArise
# Contains menus for all the options in the system
genArise.init <- function(envir){ 
if(.Platform$pkgType == "mac.binary")
  addTclPath("/Library/Tcl")
  tclRequire("BWidget")
  tclRequire("Img")
  op.counter <<- 0  
  tt <- tktoplevel()
  tkfocus(tt)
  tkwm.geometry(tt,"+50+50")
  tkwm.maxsize(tt, get("height", envir = envir), get("width", envir = envir))
  tkwm.minsize(tt, get("height", envir = envir), get("width", envir = envir))
  tkwm.title(tt,"GenArise Microarray Analyzer")
  topMenu <- tkmenu(tt)
  tkconfigure(tt, menu = topMenu)
  fileMenu <- tkmenu(topMenu,tearoff=FALSE)
  optMenu <- tkmenu(topMenu,tearoff=FALSE)
  helpMenu <- tkmenu(topMenu,tearoff=FALSE)
  project.menu <- tkmenu(topMenu,tearoff=FALSE)

  # open a project already performed
  abre.project <- function(){
    file <- tclvalue(tkgetOpenFile(filetypes = "{{PRJ Files} {.prj}} {{All files} *}"))
    if(nchar(file)){
      old.project(file, envir,tt)
      tkdestroy(tt)
    }
  }

  # open a new project to be performed
  nuevo.project <-  function(){
    tkdestroy(tt)
#    new.project(envir)}
    select.experiments(envir)}

  # select the projects to perform the post-analysis
  projectos.select <- function() {
    tkdestroy(tt)
    projects.select(envir = envir)
  }

  swap.analysis <- function(){
    tkdestroy(tt)
    swap.select(envir = envir)
  }
  
  destroy <- function(){
    tkdestroy(tt)
  }
  
  tkadd(project.menu,"command",label="New Project     (Ctrl-N)", command=function(){
    nuevo.project()
  })
  tkadd(project.menu,"command",label="Open Project    (Ctrl-O)", command=function(){
    abre.project()
  })
  tkadd(fileMenu,"cascade",label="Project",menu=project.menu)
  tkadd(fileMenu,"command",label="Swap Analysis   (Ctrl-S)",command=function(){
    swap.analysis()})
  tkadd(fileMenu,"command",label="Post-analysis     (Ctrl-P)", command = function(){
    projectos.select()})
  tkadd(fileMenu,"command",label="Quit                   (Ctrl-X)",command=function() destroy())
  tkadd(topMenu,"cascade",label="File",menu=fileMenu)
  helpMenu <- tkmenu(topMenu,tearoff=FALSE)
  tkadd(helpMenu,"command",label="About genArise...", command=function() help())
  tkadd(topMenu,"cascade",label="Help",menu = helpMenu)
  lower.frame <- tkframe(tt)
  
  butt.frame <- tkframe(lower.frame)
  label.frame <- tkframe(lower.frame)
  etiqueta <- tklabel(label.frame, text = "")
  tkconfigure(etiqueta, text = "", width = "100")

  # shortcuts
  tkbind(tt, "<Control-O>",abre.project)   
  tkbind(tt, "<Control-o>",abre.project)
  tkbind(tt, "<Control-N>",nuevo.project)
  tkbind(tt, "<Control-n>",nuevo.project)
  tkbind(tt, "<Control-S>",swap.analysis)
  tkbind(tt, "<Control-s>",swap.analysis)   
  tkbind(tt, "<Control-P>",projectos.select)
  tkbind(tt, "<Control-p>",projectos.select)   
  tkbind(tt, "<Control-X>",destroy)
  tkbind(tt, "<Control-x>",destroy)
  openButton <- .Tcl(paste("button",.Tk.subwin(butt.frame),"-image [Bitmap::get open]",
                           .Tcl.args(command = abre.project))) 
  newButton <- .Tcl(paste("button",.Tk.subwin(butt.frame),"-image [Bitmap::get new]",
                          .Tcl.args(command = nuevo.project)))

  # icons in the bar menu
  tcl("image","create","photo","help",file=file.path(get("icons.dir", envir = envir),"icons/help.png"))
  help.button <- tkbutton(butt.frame,image="help", command = function() help())
  tkconfigure(help.button, heigh = 16, width = 16)

  tcl("image","create","photo","swap",file=file.path(get("icons.dir", envir = envir),"icons/swap.png"))
  swap.button <- tkbutton(butt.frame,image="swap", command = function() swap.analysis())
  tkconfigure(swap.button, heigh = 16, width = 16)
  
  tcl("image","create","photo","exit",file=file.path(get("icons.dir", envir = envir),"icons/logout.png"))
  exit.button <- tkbutton(butt.frame,image="exit", command = function(){
    tkdestroy(tt)})
  tkconfigure(exit.button, heigh = 16, width = 16)
                  
  tkpack(etiqueta, anchor = "e")
  tkgrid(newButton, openButton, swap.button, help.button, exit.button, sticky = "nw")
  tkgrid(butt.frame, label.frame)

  
  label.function <-  function(texto.etiqueta) tkconfigure(etiqueta, text = texto.etiqueta, width = "100", fg = "red")

  tkbind(openButton, "<Enter>", function() label.function("Open a project"))
  tkbind(openButton, "<Leave>", function() label.function(""))

  tkbind(newButton, "<Enter>", function()  label.function("Create a  new project"))
  tkbind(newButton, "<Leave>", function() label.function(""))

  tkbind(swap.button, "<Enter>", function() label.function("Dye-Swap Analysis"))
  tkbind(swap.button, "<Leave>", function() label.function(""))

  tkbind(help.button, "<Enter>", function() label.function("Display genArise's Help"))
  tkbind(help.button, "<Leave>", function() label.function(""))

  tkbind(exit.button, "<Enter>", function() label.function("Exit of genArise GUI"))
  tkbind(exit.button, "<Leave>", function() label.function(""))
  
  tkpack(lower.frame, padx = "0", anchor = "w")
}
  

genArise <- function(){
  genArise.env <- new.env()
  set.project.properties(genArise.env)
  assign("info.message",
         "This package was made in the Institute of cellular Physiology UNAM\nmailto: genArise@ifc.unam.mx",
         envir = genArise.env)
  genArise.init(genArise.env)
}

# GENARISE 
# Graphics and statistics operations

# Plot the signals: Cy3 vs Cy5 (obtained from the Spot object)
cys.plot <- function(mySpot, col = "green"){
  spot <- attr(mySpot, "spotData")
  par( bg = "black", col.axis = "white", col.lab = "white", fg = "white")
  plot(log2(spot$Cy3), log2(spot$Cy5),xlab = "log2(Cy3)", ylab= "log2(Cy5)",  pch = ".", col = col)
  lines(log2(spot$Cy5),log2(spot$Cy5),col="turquoise")
}

# Plot mean log intensity versus intensity log-ratio: M vs A
ma.plot <- function (mySpot, col = "green"){
  spot <- attr(mySpot, "spotData")
  cys.index <- (spot$Cy3 > 0) & (spot$Cy5 > 0) & (!is.na(spot$Cy3)) &(!is.na(spot$Cy5)) & is.finite(spot$Cy3) & is.finite(spot$Cy5)
  cy3 <- spot$Cy3[cys.index]
  cy5 <- spot$Cy5[cys.index]
  bgcy3 <- spot$BgCy3[cys.index]
  bgcy5 <- spot$BgCy5[cys.index]
  M <- log(cy5,2) - log(cy3, 2)
  A <- (log(cy3, 2) + log(cy5, 2))/2
  spot$Id <- spot$Id[cys.index]
  par( bg = "black", col.axis = "white",col.lab = "white", fg = "white")
  plot(A,M, xlab = "A", ylab = "M", pch = ".",col = col)
  lines(lowess(M~A),col="turquoise",lwd=1)
}

# log2(Cy5/Cy3) ratio as a function of the log10(Cy5*Cy3) product intensities
ri.plot <- function(mySpot, col = "green"){
  spot <- attr(mySpot, "spotData")
  cys.index <- (spot$Cy3 > 0) & (spot$Cy5 > 0) & (!is.na(spot$Cy3)) &(!is.na(spot$Cy5)) & is.finite(spot$Cy3) & is.finite(spot$Cy5)
  cy3 <- spot$Cy3[cys.index]
  cy5 <- spot$Cy5[cys.index]
  R <- log(cy5,2) - log(cy3, 2)
  I <- log(cy3, 10) + log(cy5, 10)
  spot$Id <- spot$Id[cys.index]
  par( bg = "black", col.axis = "white", col.lab = "white",fg = "white")
  plot(I,R, xlab = "I", ylab = "R", pch = ".",col = col )
  lines(lowess(R~I),col="turquoise",lwd=1)
}

# In this plot , array elements are color-coded depending on wether they are less than
# 1 standard deviation from the mean (green), between 1 and 1.5 standard deviations (blue)
# between 1.5 and 2 standard deviations (turquoise), or more than 2 standard deviations (white)
Zscore.plot <-  function(dataSet.spot, Zscore.min = NULL, Zscore.max = NULL, all = TRUE, col = "green"){
  values <- attr(dataSet.spot, "dataSets")
  type <- attr(dataSet.spot, "type")
  Zscore <- values$Zscore
  if(type == "ri"){
    x.axis <- log(values$Cy3, 10) + log(values$Cy5, 10)    
  }else{
    x.axis <- (log(values$Cy3, 2) + log(values$Cy5, 2))/2
  }
  y.axis <- log(values$Cy5, 2)  - log(values$Cy3, 2)
  if(all){
    par(bg = "black", col.axis = "white", col.lab = "white",fg="white")
    # get the ranges for the color-code
    blue <- abs(Zscore) < 1
    green <- (abs(Zscore) >= 1) & (abs(Zscore) < 1.5)
    red <- (abs(Zscore) >= 1.5) & (abs(Zscore) <= 2)
    #yellow <- abs(Zscore) > 1.5
    orange <- abs(Zscore) > 2
    # the plot will be R vs I or M vs A
    if(type == "ri"){    
      plot(x.axis, y.axis, col = "black", pch =3, xlab = "I", ylab = "R")
    }else{
      plot(x.axis, y.axis, col = "black", pch =3, xlab = "A", ylab = "M")
    }
    # draw the points
    points(x.axis[orange], y.axis[orange], col = "snow", pch =3)      
    points(x.axis[blue], y.axis[blue], col = "green", pch = 3)
    points(x.axis[green], y.axis[green], col = "blue",pch= 3)
#    points(x.axis[yellow], y.axis[yellow], col="yellow", pch= 3)
    points(x.axis[red], y.axis[red], col = "turquoise", pch = 3)
  }
  else{
    if(is.null(Zscore.min) && is.null(Zscore.max)){
      stop("At least one value must be non null")
    }
    if(!is.null(Zscore.max) && !is.null(Zscore.min) && Zscore.max < Zscore.min){
      stop("Invalid ranges")
    }
    par(bg = "black", col.axis = "white", col.lab = "white", fg = "white")
    if(is.null(Zscore.min)){
      green <- abs(Zscore) < Zscore.max
    }else{
      if(is.null(Zscore.max)){
        green <- abs(Zscore) > Zscore.min
      }else{
        green <- (abs(Zscore) > Zscore.min) & (abs(Zscore) < Zscore.max)
      }
    }
    if(type == "ri"){
      plot(x.axis, y.axis, col = "black", pch =3, xlab = "I", ylab = "R")      
      points(x.axis[green], y.axis[green], col = col, pch = 3)
    }else{
      plot(x.axis, y.axis, col = "black", pch =3, xlab = "A", ylab = "M")
      points(x.axis[green], y.axis[green], col = col, pch = 3)
    }
  }
}

require(tkrplot)
old.project <-  function(project.name, envir, parent){
  selected.original <-  function(state = TRUE){
    
  }
  selected.zscore <-  function(Zscore.min = NULL, Zscore.max = NULL, all = TRUE, col = "green"){
    tkentryconfigure(topMenu,"2",state="disabled")
    tkentryconfigure(topMenu,"3",state="normal")
    tkrreplot(img, fun = function() Zscore.plot(get("Zscore.spot", envir = envir),
                     Zscore.min = Zscore.min, Zscore.max = Zscore.max, all = all, col = col))
  }
  save.pdf <- function(name,Zscore.min = NULL, Zscore.max = NULL, all = TRUE, col = "green"){
    pdf(name,horiz=F, height=8,width=8,title=paste(name, sep = "_"))
    Zscore.plot(get("Zscore.spot", envir = envir),
                     Zscore.min = Zscore.min, Zscore.max = Zscore.max, all = all, col = col)
    dev.off(dev.cur())
  }

  imageLimma.plot <-  function(datos, low = NULL, high = NULL){
    tkrreplot(img, fun = function() imageLimma(datos, as.numeric(as.vector(spots[3,2])), as.numeric(as.vector(spots[4,2])),
                     as.numeric(as.vector(spots[5,2])), as.numeric(as.vector(spots[6,2])),low =low, high = high))
    tkentryconfigure(topMenu,"2",state="disabled")
    tkentryconfigure(topMenu,"3",state="disabled")
  }
  
  assign("graphic.type", 2, envir = envir)
  spots <- read.table(project.name, header = FALSE, sep = "\t")
  if(length(spots[,1]) < 7){
    tkmessageBox(parent = parent,  message= "This project doesn't have operations", icon = "error", default = "ok")
    stop("This project doesn't have operations")
  }
  else{
    tt <- tktoplevel()
    tkfocus(tt)
    tkwm.geometry(tt,"+50+50")
    tkwm.title(tt,"GenArise Microarray Analyzer")
    tmp <- unlist(strsplit(project.name, .Platform$file.sep))
    tmp[length(tmp)] <- unlist(strsplit(tmp[length(tmp)], "\\."))[1]
    noext <- paste(tmp, collapse = .Platform$file.sep)    
    results.file  <-  spots[1,1]
    graphics.file <-  spots[1,2]
    
    topMenu <- tkmenu(tt)
    tkconfigure(tt,menu=topMenu)
    fileMenu <- tkmenu(topMenu,tearoff=FALSE)
    upper.frame <- tkframe(tt)
    frameOverall <- tkframe(upper.frame)
    frameimage <- tkframe(upper.frame, relief="groove",borderwidth=2)
    frameFeatures <-  tkframe(frameOverall, relief="groove")
    tkgrid(tklabel(frameFeatures, text = "Spot Features"), pady = "7")
#Array geometry
    spot.name <- unlist(strsplit(as.character(spots[2,2]), " "))
    tkgrid(tklabel(frameFeatures, text = paste(spots[2,1], spot.name[1], sep = "\t")), padx = "0",pady = "2", sticky = "w")
    if(is.na(spot.name[3]))
      symdesc <- NULL
    else
      symdesc <- 1
    for(i in 3:6){
      tkgrid(tklabel(frameFeatures, text = paste(spots[i,1], spots[i,2], sep = "\t")), padx = "0",pady = "2", sticky = "w")
    }
#original data
    assign("a.spot1", read.spot(paste(noext,.Platform$file.sep,results.file,.Platform$file.sep,spots[7,2],sep=""), header=TRUE,cy3=1,cy5=2,bg.cy3=3,bg.cy5=4,ids=5,envir=envir),
           envir =  envir)
    datos <- attr(get("a.spot1",envir=envir), "spotData")
    M <- log(datos$Cy5, 2) - log(datos$Cy3, 2)
    prjlength <- length(spots[,1])
    name <- as.character(as.vector(spots[2,2]))
    
    frame1 <- tkframe(frameOverall,relief="groove",borderwidth=2)
    frame2 <- tkframe(frameOverall,relief="groove",borderwidth=2)
    dist  <- tclVar(1)
    
    otra.funcion <-  function(i){
      if(is.element(paste("a.spot",i, sep =""), ls(envir = envir))){
        assign("a.spot", get(paste("a.spot",i, sep =""), envir = envir), envir = envir)
        tkrreplot(img,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))        
      }else{
        assign(paste("a.spot",as.numeric(tclvalue(dist)),sep=""),
               read.spot(paste(noext,.Platform$file.sep,results.file,.Platform$file.sep,spots[as.numeric(tclvalue(dist)),2],sep=""),header=TRUE,cy3=1,cy5=2,bg.cy3=3,bg.cy5=4,ids=5,envir=envir),
               envir =  envir)
        assign("a.spot", get(paste("a.spot",i, sep =""), envir = envir), envir = envir)
        tkrreplot(img,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))        
      }
      tkentryconfigure(topMenu,"2",state="normal")
      tkentryconfigure(topMenu,"3",state="normal")
    }
    #Operations
    tkpack(tklabel(frame1, text = "OPERATIONS"), pady = "7")
    if(spots[(length(spots[,1])),1] == "RI Zscore" || spots[(length(spots[,1])),1] == "MA Zscore" ){
      radio.original <- tkradiobutton(frame1, text="Original Spot", value=7, variable=dist,command=function(){
          otra.funcion(as.numeric(tclvalue(dist)))
        })
      tkpack(radio.original,anchor="w", pady = "2",padx="10")
      
      for(i in 8:(length(spots[,1])-1)){
        radio <- tkradiobutton(frame1, text=as.vector(spots[i,1]), value=i, variable=dist,command=function(){
          otra.funcion(as.numeric(tclvalue(dist)))
        })
        tkpack(radio,anchor="w", pady = "2",padx="10")
      }
      
      frame2 <- tkframe(frameOverall,relief="groove",borderwidth=2)
      tkpack(tklabel(frame2, text = "Z-SCORE"), pady = "7")
        assign("Zscore.spot", read.dataset(paste(noext,.Platform$file.sep,results.file,.Platform$file.sep,spots[(length(spots[,1])),2],sep=""),symdesc=symdesc),
               envir =  envir)

      zscore1 <- tkradiobutton(frame2, text="Zscore < 1", value=1, variable=dist,command=function(){
        selected.zscore(all = FALSE, Zscore.max = 1, col = "green")
      })
      tkpack(zscore1,anchor="w", pady = "2",padx="10")
      
      zscore2 <- tkradiobutton(frame2, text="1 < Zscore < 1.5", value=2, variable=dist,command=function(){
        selected.zscore(Zscore.max = 1.5, Zscore.min = 1, all = FALSE, col = "blue")})
      tkpack(zscore2,anchor="w", pady = "2",padx="10")
      
      zscore3 <- tkradiobutton(frame2, text="1.5 < Zscore < 2", value=3, variable=dist,command=function(){
        selected.zscore(Zscore.max = 2, Zscore.min = 1.5, all = FALSE, col = "cyan")})
      tkpack(zscore3,anchor="w", pady = "2",padx="10")
      
      zscore4 <- tkradiobutton(frame2, text="Zscore > 1.5", value=4, variable=dist,command=function(){
        selected.zscore(Zscore.min = 1.5, all = FALSE, col = "yellow")})
      tkpack(zscore4,anchor="w", pady = "2",padx="10")
      
      zscore5 <- tkradiobutton(frame2, text="Zscore > 2", value=5, variable=dist,command=function(){
        selected.zscore(Zscore.min = 2, all = FALSE, col = "snow")})
      tkpack(zscore5,anchor="w", pady = "2",padx="10")
      
      zscore6 <- tkradiobutton(frame2, text="All", value=6, variable=dist,command=function(){
        selected.zscore()})
      tkpack(zscore6,anchor="w", pady = "2",padx="10")
    }
    else{
      radio.original <- tkradiobutton(frame1, text="Original Spot", value=7, variable=dist,command=function(){
          otra.funcion(as.numeric(tclvalue(dist)))
        })
      tkpack(radio.original,anchor="w", pady = "2",padx="10")
      for(i in 8:(length(spots[,1]))){
        radio <- tkradiobutton(frame1, text=as.vector(spots[i,1]), value=i, variable=dist,command=function(){
          otra.funcion(as.numeric(tclvalue(dist)))
        })
        tkpack(radio,anchor="w", pady = "2",padx="10")
      }
    }
    redgreenimg <- tkradiobutton(frameimage, text="Red & Green", value=12, variable=dist,command=function(){
      imageLimma.plot(M)})
    
    redimg <- tkradiobutton(frameimage, text="Background Cy5", value=13, variable=dist,command = function(){
      imageLimma.plot(log(datos$BgCy5, 2),"white","red")})
    
    greenimg <- tkradiobutton(frameimage, text="Background Cy3", value=14, variable=dist,command=function(){
      imageLimma.plot(log(datos$BgCy3, 2),"white","green")})
    
    tkgrid(redgreenimg,redimg,greenimg, pady = "2",padx="10")  
    
    assign("a.spot1",read.spot(paste(noext,.Platform$file.sep,results.file,.Platform$file.sep,spots[7,2],sep=""),header=TRUE,cy3=1,cy5=2,bg.cy3=3,bg.cy5=4,ids=5, envir=envir), envir = envir)
    assign("a.spot", get("a.spot1",envir = envir),envir=envir)
    img <-  tkrplot(upper.frame, fun = function() graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))
    tkadd(fileMenu,"command",label="Back to main", command=function(){ tkdestroy(tt); genArise() })
    tkadd(fileMenu,"command",label="Exit", command=function() tkdestroy(tt))

    tkadd(topMenu,"cascade",label="File",menu=fileMenu)
    
    graphicMenu <- tkmenu(topMenu,tearoff=FALSE)
    tkadd(graphicMenu,"command",label="Cy3 vs Cy5", command = function() {
      assign("graphic.type", 1, envir = envir)
      tkrreplot(img,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))})
    tkadd(graphicMenu,"command",label="R vs I",command = function() {
      assign("graphic.type", 2, envir = envir)
      tkrreplot(img,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))})
    tkadd(graphicMenu,"command",label="M vs A",command = function() {
      assign("graphic.type", 3, envir = envir)
      tkrreplot(img,fun=function()graphic.choose(get("a.spot", envir = envir), get("graphic.type", envir = envir)))})
    tkadd(topMenu,"cascade",label="Graphics",menu =graphicMenu)
    
    optionsMenu <- tkmenu(topMenu,tearoff=FALSE)
    make.file.name <- function(file.type, ext){
      switch(as.integer(tclvalue(dist)),
             suggested.name <- paste(spot.name[1], "_lower_1.", ext, sep = ""),
             suggested.name <- paste(spot.name[1], "_1-1_5.", ext, sep = ""),
             suggested.name <- paste(spot.name[1], "_1_5-2.", ext, sep = ""),
             suggested.name <- paste(spot.name[1], "_greater_1_5.", ext, sep = ""),
             suggested.name <- paste(spot.name[1], "_greater_2.", ext, sep = ""),
             suggested.name <- paste(spot.name[1], "_all.", ext, sep = ""),
             suggested.name <- paste(spot.name[1], "_", unlist(strsplit(as.character(spots[7,2]), "\\."))[1],".", ext, sep = ""),
             suggested.name <- paste(spot.name[1], "_",unlist(strsplit(as.character(spots[8,2]), "\\."))[1], ".",ext, sep = ""),
             suggested.name <- paste(spot.name[1], "_",unlist(strsplit(as.character(spots[9,2]), "\\."))[1], ".",ext, sep = ""),
             suggested.name <- paste(spot.name[1], "_",unlist(strsplit(as.character(spots[10,2]), "\\."))[1], ".",ext, sep = ""),
             suggested.name <- paste(spot.name[1], "_",unlist(strsplit(as.character(spots[11,2]), "\\."))[1], ".",ext, sep = ""),
             )
      tclvalue(tkgetSaveFile(initialdir = paste(noext,.Platform$file.sep,graphics.file,sep=""),
                             initialfile = suggested.name, filetypes = file.type))
    }
    tkadd(optionsMenu,"command",label="Save graphic as PDF",command = function() {
      name <- make.file.name(file.type = "{{PDF Files} {.pdf}} {{All files} *}", ext ="pdf")
      if (!nchar(name))
       tkmessageBox(parent = tt,  message= "You must write a name of file!", icon = "error", default = "ok")
      else{
        name <- unlist(strsplit(name, "\\."))[1]
        pdf(paste(name,".pdf",sep=""), horiz = FALSE, height = 8, width = 8, title = name)
        switch(as.integer(tclvalue(dist)),
               Zscore.plot(get("Zscore.spot", envir = envir), all = FALSE, Zscore.max = 1, col = "green"),
               Zscore.plot(get("Zscore.spot", envir = envir), Zscore.max = 1.5, Zscore.min = 1, all = FALSE, col = "blue"),
               Zscore.plot(get("Zscore.spot", envir = envir), Zscore.max = 2, Zscore.min = 1.5, all = FALSE, col = "cyan"),
               Zscore.plot(get("Zscore.spot", envir = envir), Zscore.min = 1.5, all = FALSE, col = "yellow"),
               Zscore.plot(get("Zscore.spot", envir = envir), Zscore.min = 2, all = FALSE, col = "snow"),
               Zscore.plot(get("Zscore.spot", envir = envir)),
               ri.plot(get("a.spot", envir = envir)),
               ri.plot(get("a.spot", envir = envir)),
               ri.plot(get("a.spot", envir = envir)),
               ri.plot(get("a.spot", envir = envir)),
               ri.plot(get("a.spot", envir = envir)),
               )
        dev.off(dev.cur())
      }
    })
#    tkadd(optionsMenu,"command",label="Notes",command = function()note(envir))
    tkadd(topMenu,"cascade",label="Options", menu =optionsMenu)
    
    helpMenu <- tkmenu(topMenu,tearoff=FALSE)
    tkadd(helpMenu,"command",label="About genArise...", command=function() help())#{
    
    tkadd(topMenu,"cascade",label="Help",menu = helpMenu)
    
    frame.label <- tkframe(tt,relief="groove",borderwidth=2)
    tkfocus(tt)
    tkselect(radio.original)
    tkgrid(img,frameOverall, padx = "10", pady = "10",sticky="w")
    tkgrid(frameimage, padx = "90", pady = "10",sticky="w")
    tkgrid(frameFeatures)
    tkgrid(frame1, pady = "10")
    tkgrid(frame2, pady = "10")
    tkgrid(upper.frame, pady = "10")
    area.frame <- tkframe(tt,relief="groove",borderwidth=2)
    tkgrid(area.frame, padx = "10")
     }
}

Zscore <- function(spot.object, type = "ri", window.size = 50){
	spot.data <- attr(spot.object, "spotData")
        if(type == "ri"){
          x.axis <- log(spot.data$Cy3, 10) + log(spot.data$Cy5, 10)
        }else{
          if(type == "ma"){
            x.axis <- (log(spot.data$Cy3, 2) + log(spot.data$Cy5, 2))/2
          }else{
            stop("Type argument must be ri or ma")
          }
        }
        resultado <- list()
        y.axis <- log(spot.data$Cy5,2) - log(spot.data$Cy3,2)
        
	sort.index <- sort.list(x.axis)        
        y.axis <- y.axis[sort.index]
#        cy5 <- spot.data$Cy5[sort.index]
 #       cy3 <- spot.data$Cy3[sort.index]
  #      id <- spot.data$Id[sort.index]
	x.axis <- x.axis[sort.index]
        
        spot.data$Cy3 <- spot.data$Cy3[sort.index]
        spot.data$Cy5 <- spot.data$Cy5[sort.index]
        spot.data$Id <- spot.data$Id[sort.index]

        if(!is.null(spot.data$Symdesc))
   #       symdesc <- spot.data$Symdesc[sort.index]
          spot.data$Symdesc <- spot.data$Symdesc[sort.index]
        

	n.datos <- length(y.axis)
	inicio.y.axis <- unlist(sapply(1:as.integer(window.size/2), function(x) y.axis[x]))
        tmp <-  unlist(sapply(1:window.size, function(x) y.axis[x]))
        media <- mean(tmp,na.rm=TRUE)
	std <- sd(tmp,na.rm=TRUE)
        inicio.Zscore <- (inicio.y.axis - media)/ std
	resultados <- inicio.Zscore
        
	final.y.axis  <- unlist(sapply((n.datos - as.integer(window.size/2)):n.datos, function(x) y.axis[x]))
        tmp <-  unlist(sapply((n.datos - window.size):window.size, function(x) y.axis[x]))
	media <- mean(tmp,na.rm=TRUE)
	std <- sd(tmp,na.rm=TRUE)
	final.Zscore <- (final.y.axis - media) / std

	for( i in 26:(n.datos-26)){
		y.axis.temp <- unlist(sapply((i-as.integer(window.size/2)):(i+as.integer(window.size/2)), function(x) y.axis[x]))
		media <- mean(y.axis.temp,na.rm=TRUE)
 		std <- sd(y.axis.temp,na.rm=TRUE)
		resultados <- c(resultados, (y.axis[i] - media)/std, recursive = TRUE) 
	}
	resultados <- c(resultados, final.Zscore, recursive = TRUE)
        if(is.null(spot.data$Symdesc))
          new("DataSet", name = attr(spot.object, "name"),
              dataSets = list(Cy3 = spot.data$Cy3, Cy5 = spot.data$Cy5, Id = spot.data$Id, Zscore = resultados), type = type)
        else
          new("DataSet", name = attr(spot.object, "name"),
              dataSets = list(Cy3 = spot.data$Cy3, Cy5 = spot.data$Cy5, Id = spot.data$Id, Symdesc = spot.data$Symdesc, Zscore = resultados), type = type) 
      }

get.values <- function(list.values, genes.values, up.down, min.val, max.val){
  if( !(up.down == "up" || up.down == "down"))
    stop("The value for the  up.down argument must be up or down")
  if(is.null(min.val) && is.null(max.val))
    stop("You must select a range")
  if(up.down == "up"){
    if(!is.null(max.val) && !is.null(min.val)){
      indices <- (list.values > min.val) & (list.values <= max.val)
    }else{
      if(is.null(max.val)){
        indices <- list.values >= min.val
      }else{
        indices <- list.values <= max.val
      }
    }      
  }else{
    if(!is.null(max.val) && !is.null(min.val)){
      indices <- (list.values < -min.val) & (list.values >= -max.val)
    }else{
      if(is.null(max.val)){
        indices <- list.values < -min.val
      }else{
        indices <- list.values >= -max.val
      }
    }      
  }
  toupper(genes.values[indices])
}

post.analysis <-  function(values, min.val, max.val, up.down, output){
   if(length(values) <= 1)
    stop("You must at least two files to perform the post-analysis")
  myHash <- new.env(hash = TRUE)
  for(i in 1 :length(values)){
    results.file <- read.table(values[i], sep ="\t", header = FALSE)[1,1]
    file.data <- read.csv(file.path(unlist( strsplit(values[i], "\\.prj"))[1],
                                      results.file, "zscore.txt"),sep = "\t", header = FALSE)
    file.data <- file.data[-1,]
    if(length(file.data[1,])==5)
      assign(values[i], get.values(as.vector(file.data[,5]), as.vector(file.data[,3]), up.down, min.val, max.val), envir = myHash)
    else
      assign(values[i], get.values(as.vector(file.data[,4]), as.vector(file.data[,3]), up.down, min.val, max.val), envir = myHash)
  }


  intersect.sets <-  function(hash,archivos, binario){
    archivos.int <- archivos[binario == 1]
    archivos.dif <- archivos[binario == 0]

    interseccion <- get(archivos.int[1], envir = hash)

    if(length(archivos.int) > 1){
      for(i in 2:length(archivos.int)){
        ids <-  get(archivos.int[i], envir = hash)
        interseccion <- intersect(interseccion, ids)
      }
    }
    if(length(archivos.dif) > 0){
      for(i in 1:length(archivos.dif)){
        ids <-  get(archivos.dif[i], envir = hash)
        interseccion <- setdiff(interseccion, ids)
      }
    }
    interseccion
  }
  
  nextNumber <- function(binario, pos1){
    while(pos1 > 0){
      
      if(binario[pos1] == 0){
        binario[pos1] <- 1
        break
      }else{
        binario[pos1] <- 0
        pos1 <- pos1 - 1
        nextNumber(binario, pos1)
      }
    }
    binario
  }
  
  binary.add <- function(lista, cantidad){
    for(i in 0:cantidad){
      nextNumber(lista, length(lista))
    }
  }

  final <- function(lista){
    
    contador.binario <- rep(0, times = length(lista))
    if(file.exists(output)){
      files.list <- list.files(output)
      for(i in 1:length(files.list))
        file.remove(paste(output,files.list[i],sep=.Platform$file.sep))
    }
    else
      dir.create(output)
    write.table(lista, file.path(output, "order.txt"), sep = "\t", quote = FALSE, row.names = FALSE, col.names = FALSE)
    cat("The order of the files is important for the right interpretation of the data.", file = file.path(output, "order.txt"),
        append = TRUE)

    for(i in 1:((2^length(lista))-1)){
      contador.binario <- binary.add(contador.binario,1)
      datos <- intersect.sets(myHash, lista, contador.binario)
      output.file <- file.path(output, paste(paste(contador.binario, collapse= ""), ".set", sep = ""))
      unlink(output.file)
      objeto <- file(output.file, "w")
      write.table(datos, output.file, col.names = FALSE, row.names = FALSE,quote = FALSE, sep = "\t")
      close(objeto)
    }
  }
  
  final(values)

 }

require(methods) # In case R version is earlier than 1.7.0

# Class to define a microarray
# Contains 2 attributes:
# name: The name of the experiment
# spotData: The data of the experiment. Cy3, Cy5, BgCy3, BgCy5, Id
setClass("Spot",
	  representation(name ="character", spotData ="list"), where = .GlobalEnv)

# Some checking to get a correct object
setValidity("Spot",function(object){

	spotName <- as.character(attr(object, "name"))
	spotData <- attr(object, "spotData")

	Cy3 <- spotData$Cy3
	Cy5 <-spotData$Cy5
	BgCy3 <- spotData$BgCy3
	BgCy5 <-spotData$BgCy5
	Id <- spotData$Id
        Symdesc <- spotData$Symdesc
        
	if(is.null(Cy3) || is.null(Cy5)) return("Element R or G missing")
	if(!is.numeric(Cy3) || !is.numeric(Cy5)) return("R or G contain non-numeric elements")
	if(length(dim(Cy3)) > 2) return("R and G have more than two dimensions")
})

# Class to define the Z-score for the data set.
# Contains 3 attributes:
# name: The name of the experiment
# dataSet: The data of the experiment. Cy3, Cy5, Id
# type: The analysis wll be done R vs I or M vs A
setClass("DataSet", representation(name = "character", dataSets = "list", type = "character"), where = .GlobalEnv)

# Some checking to get a correct object 
setValidity("DataSet", function(object){
  spot.name <- attr(object, "name")
  dataSets <- attr(object, "dataSets")
  type <- attr(object, "type")
  Cy3 <-  dataSets$Cy3
  Cy5 <-  dataSets$Cy5
  Id  <-  dataSets$Id
  Symdesc <- dataSets$Symdesc
  Zscore <- dataSets$Zscore
  if(!(type == "ri" || type == "ma")) return ("Type argument must be ri or ma")
  if(is.null(Cy3) || is.null(Cy5) || is.null(Id) || is.null(Zscore)) return("Some element is missing")
  if(!is.numeric(Cy3) || !is.numeric(Cy5) || !is.numeric(Zscore)) return("There is a non-numeric element")
})

%\VignetteIndexEntry{genAriseGUI Vignette}
%\VignetteDepends{}
%\VignetteKeywords{microarray analyzer GUI}
%\VignettePackage{genAriseGUI}
\documentclass[12pt]{article}
\topmargin=0in
\oddsidemargin 0.0in
\evensidemargin 0.0in
\usepackage{url}
\usepackage{graphicx}
\usepackage{hyperref}
\usepackage{fancyhdr}
\pagestyle{fancy}
\headheight 35pt 
\fancyhead[R]{\Large{\bf{genArise}}} %Especifico

\title{The genArise Package}
\author{Ana Patricia G\'omez May\'en\\ Gustavo Corral Guill\'e\\ Lina Riego Ruiz\\ Gerardo Coello Couti\~no}

\begin{document}
\setkeys{Gin}{width=0.5\textwidth}
\maketitle

\section{Introduction}

genArise is a package that contains specific functions to perform an analysis of microarray obtained data to select genes that are significantly differentially expressed between classes of samples. Before this analysis, genArise carry out a number of transformations on the data to eliminate low-quality measurements and to adjust the measured intensities to facilitate comparisons.\\

First, you must install the R system. The current released version is 2.2.0 and the binary can be obtained via CRAN, a collection of sites which carry identical material, consisting of the R distribution(s), the contributed extensions, documentation for R, and binaries. The CRAN master site can be found at the URL \url{http://cran.R-project.org/}. \\

Then, you need install the packages \emph{tkrplot}, \emph{locfit} and \emph{xtable} in order to be able to use this package. This can be done as follows:
\begin{Scode}
> install.packages(c("locfit","tkrplot", "xtable"))
\end{Scode}

This R function automatically download and install the three packages from CRAN, but you must be connected to the network. In other case, you must download the three packages from CRAN for the correct O.S. (Windows, Linux) and install from the terminal these add-on packages with the script \emph{R CMD INSTALL package\_name} for Linux or \emph{Rcmd INSTALL package\_name} for Windows. With the same script you can install genArise too. You can read the instalation manual in the URL: \url{http://www.ifc.unam.mx/genarise}.\\

genArise GUI uses widgets from the BWidget package and from the Img package, neither of which come bundled with the default installation of R. If you are using Windows (and not using the installation wizard), you need to install ActiveTcl 8.4.x from \url{http://aspn.activestate.com/ASPN/Downloads/ActiveTcl/}. After install ActiveTcl, you must set the environment variable MY\_TCLTK to a non-empty value, for example: \texttt{MY\_TCLTK=Yes} and it is assumed that you want to use a different Tcl/Tk 8.4.x installation, and that this is set up correctly. You must too set the environment variable TCL\_LIBRARY with the path where are the DLLs of Tcl in your machine, for example: \texttt{TCL\_LIBRARY=C:/Tcl/lib/tcl8.4}.\\

If using Linux, install the Tcl and Tk 8.3 rpms from CD, e.g. into /usr/lib, then download BWidget from \url{http://sourceforge.net/projects/tcllib/}.\\
You must gunzip and untar this file into /usr/lib. Download too tkimg1.3 (this is the version for Tcl and Tk 8.3) which you can search in \url{http://sourceforge.net} and gunzip it and untar it into /usr/lib. Then you probably must copy the directory /usr/lib/tkimg1.3/Img/exec\_prefix/lib/Img/ to /usr/lib.\\

Finally, genArise can be loaded with the next instruction:\\
<<>>=
library(genArise)
@
Once loaded, you can use any function of genArise and proceed with the analysis of the microarray data. genArise is provided with functions that can be applied from R prompt in every step of analysis; however, there is also a graphical user interface to facilitate the use of all the functions in the package.

\section{The genArise Package's Functions\label{sec1}}
To start the analysis you must read first the data contained in the input file. To read this file, genArise provide the function \texttt{read.spot}. This function returns an object of type Spot (a table structure with the name of the file without extension and the data for the analysis: Cy3 intensity, Cy5 intensity, Cy3 Background, Cy5 Background, Ids and maybe an optional column for an extra identifier or gene symbol) because many other functions of genArise that carry out transformations on the data need a spot object as argument. The syntax for this function is:

\begin{Scode}
> my.spot <- read.spot( file.name, cy3 = 1, cy5 = 2, 
  bg.cy3 = 3, bg.cy5 = 4, ids = 5, header = FALSE, 
  sep = "\t", is.ifc = FALSE)	
\end{Scode}

\begin{Soutput}
Output: an object of class Spot called my.spot
\end{Soutput}

You can see the details and the meaning of each one of the arguments of this and all the other functions described bellow in the manual genArise.pdf and you can download it from this address \url{http://www.ifc.unam.mx/genarise} in the \texttt{Package GenArise \( API \)} link\\

We are not to give an example from this function, but you just have to follow the sintax of the \texttt{read.spot} function described in the manual to create a spot object. In this manual we use a subset of a data set called Simon for the next examples. Simon, as we have said, is a spot object that was created with the \texttt{read.spot} function and you can load this object as follow:

<<>>=
data(Simon)
ls()
@

\begin{Soutput}
You have created an object of the class Spot in the current
environment so you will be able to apply to that object any 
function of genArise that receives a spot as argument.
\end{Soutput}

\subsection{Diagnostic plots}
Previous to any kind of transformation on the data once loaded the input, you are able to visualize it using the function \texttt{imageLimma}. This function constructs an image plot in a green to red scale representing the $log_2$ intensity ratio for each spot on the array. See Figure \ref{fig1}.\\

 The imageLimma function receives several arguments depending on the data that is going to be analyzed. imageLimma is completly based in the imageplot function from the limma package.\footnote{\textit{Gordon K. Smyth (2004) \textquotedblleft Linear Models and Empirical Bayes Methods for Assessing Differential Expression in Microarray Experiments \textquotedblright, Statistical Applications in Genetics and Molecular Biology: Vol. 3: No. 1, Article 3. \url{http://www.bepress.com/sagmb/vol3/iss1/art3}}} \\

\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(Simon)
datos <- attr(Simon, "spotData")
M <- log(datos$Cy3, 2) - log(datos$Cy5, 2)
imageLimma(M, 23, 24, 2, 2)
@
\caption{Image plot in a green to red scale.\label{fig1}}
\end{center}
\end{figure}
\begin{Scode}
> data(Simon)
> datos <- attr(Simon, "spotData")# Extract spot data
> M <- log(datos$Cy3, 2) - log(datos$Cy5, 2)
> imageLimma(z = M, row = 23, column = 24, meta.row = 2, 
  meta.column = 2, low = NULL, high = NULL)
\end{Scode}

\begin{Soutput}
Output: Plot the intensity values See Figure 1
\end{Soutput}

In the same way that you can plot the $log_2$ intensity ratio for each spot, you can also plot the background value that correspond to each one of the intensities. See Figure \ref{fig2} and \ref{fig3}.\\

For example, the next code generates the plot for Cy3 Bg intensities shown in Figure \ref{fig2}.
\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(Simon)
datos <- attr(Simon, "spotData")
R <- log(datos$BgCy3, 2)
imageLimma(R, 23, 24, 2, 2, high = "red", low = "white")
@
\caption{Cy3 intensity background preview. \label{fig2}}
\end{center}
\end{figure}
\begin{Scode}
> data(Simon)
> datos <- attr(Simon, "spotData")# Extract spot data
> R <- log(datos$BgCy3, 2)
> imageLimma(z = R, row = 23, column = 24, meta.row = 2, 
  meta.column = 2, low = "white", high = "red")
\end{Scode}

\begin{Soutput}
Output: Cy3 intensity background plot. See Figure 2
\end{Soutput}
And this is the code to generate the plot for Cy5 Bg intensities shown in Figure \ref{fig3}..
\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(Simon)
datos <- attr(Simon, "spotData")
G <- log(datos$BgCy5, 2)
imageLimma(G, 23, 24, 2, 2, low = "white", high = "green")
@
\caption{Cy5 intensity background preview. \label{fig3}}	
\end{center}
\end{figure}
\begin{Scode}
> data(Simon)
> datos <- attr(Simon, "spotData")# Extract spot data
> G <- log(datos$BgCy5, 2)
> imageLimma(z = G, row = 23, column = 24, meta.row = 2, 
  meta.column = 2, low = "white", high = "green")
\end{Scode}

\begin{Soutput}
Output: Cy5 intensity Background plot. See Figure 3
\end{Soutput}

With the previous plots, you can see a preview of the raw data (without background correction and normalization). It is important to clarify that these image plot does not replace the original TIFF image from the microarray experiment.\\

In order to be able to plot a spot after apply any operation, genArise provides some functions for this purpose. For example, you can plot the values R vs I, M vs A and Cy5 vs Cy3. This functions receive an object of the class Spot.
\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(Simon)
ri.plot(Simon)
@
\caption{R vs I plot. \label{fig4}}	
\end{center}
\end{figure}
\begin{Scode}
> data(Simon)
> ri.plot(Simon)
\end{Scode}
\begin{Soutput}
Output: R vs I plot. See Figure 4.
\end{Soutput}
\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(Simon)
ma.plot(Simon)
@
\caption{M vs A plot. \label{fig5}}
\end{center}
\end{figure}
\begin{Scode}
> data(Simon)
> ma.plot(Simon)
\end{Scode}
\begin{Soutput}
Output: M vs A plot. See Figure 5.
\end{Soutput}
\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(Simon)
cys.plot(Simon)
@
\caption{log2(Cy3) vs log2(Cy5) plot. \label{fig6}}
\end{center}
\end{figure}
\begin{Scode}
> data(Simon)
> cys.plot(Simon)
\end{Scode}
\begin{Soutput}
Output: plot the log2(Cy3) and log2(Cy5) values. See Figure 6.
\end{Soutput}

\subsection{Transformation functions}

At this moment, you can proceed with the data analysis. There are different functions for this purpose and all this functions will be described in this section.

\subsubsection{Background Correction}

 The first of this functions makes the background correction; that is a subtraction Cy3 - BckgCy3 and Cy5 - BckgCy5 for each spot in the microarray. The purpose of this function is to obtain the value of the real signal of Cy3 and Cy5 from the reported values in the data file. See Figure \ref{fig7}. The name of this function is \texttt{bg.correct}.

\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(Simon)
c.spot <- bg.correct(Simon)
ri.plot(c.spot)
@
\caption{R vs I plot of background corrected data. \label{fig7}}
\end{center}
\end{figure}
\begin{Scode}
> data(Simon)
> c.spot <- bg.correct(Simon)
> ri.plot(c.spot)
\end{Scode}

\begin{Soutput}
Input: The argument of this function is a spot object 
before background correction.
Output: A spot object after background correction. See Figure 7
\end{Soutput}

\subsubsection{Data Normalization}

Data normalization can be done by each subgrids (By grid) or in a global way over all the microarray data, and each method may returns different results for the same spot object. We must remark that the grid normalization can just be applied to the complete data set, so you should not eliminate spots in order to avoid errors executing this function.  In the normalization procedure any observation in which the R value is zero will be eliminated.\\

Grid normalization is executed with the function \texttt{grid.norm} and is mandatory to indicate the dimensions of the subgrid, so, you must indicate the number of rows and columns within each subgrid. See Figure \ref{fig8}.

\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(Simon)
n.spot <- grid.norm(mySpot = Simon, nr = 23, nc = 24)
ri.plot(n.spot)
@
\caption{R vs I plot of grid normalized data.\label{fig8}}	
\end{center}
\end{figure}
\begin{Scode}
> data(Simon)
> n.spot <- grid.norm(mySpot = Simon, nr = 23, nc = 24)
> ri.plot(n.spot)
\end{Scode}

\begin{Soutput}
Input: The argument of this function is a spot object
       and the number of rows (nr) and columns (nc) 
       of each subgrid.
Output: A normalized spot object. See Figure 8
\end{Soutput}

On the other hand, the global normalization is executed with the function \texttt{global.norm} and it only requires as argument an object of the class Spot. See Figure \ref{fig9}.

\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(Simon)
n.spot <- global.norm(mySpot = Simon)
ri.plot(n.spot)
@
\caption{R vs I plot of data after global normalization. \label{fig9}}
\end{center}
\end{figure}
\begin{Scode}
> data(Simon)
> n.spot <- global.norm(mySpot = Simon)
> ri.plot(n.spot)
\end{Scode}
\begin{Soutput}
Input: The argument of this function is the spot object.
Output: A normalized (global normalization) spot object. See Figure 9
\end{Soutput}

As we said previously both functions returns different results and for this reason you must choose the suitable function of normalization to the analysis that you want to do. It for any reason spots are eliminated before normalization procedure (applying operations as filtering or duplicates elimination) the global normalization is the only normalization way.\\

In this example, the number of data in the spot is small and for this reason the obtained values with both normalization functions could seems very similar in the plots, however this is not the same in all the cases.

\subsubsection{Data Filtering}

Data filtering is an important step in the data analysis. genArise provide the intensity-based filtering algorithm described by John Quackenbush\footnote[2]{ John Quackenbush \textquotedblleft Microarray data normalization and transformation\textquotedblright. Nature Genetics. Vol.32 supplement pp 496-501 (2002)}. The name of the function is \texttt{filter.spot}. After this filtering you keep only array elements with intensities that are statistically significantly different from background. See Figure \ref{fig10}.\\

\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(Simon)
f.spot <- filter.spot(mySpot = Simon)
ri.plot(f.spot)
@
\caption{Filtered data, R vs I plot. \label{fig10}}	
\end{center}
\end{figure}

\begin{Scode}
> data(Simon)
> f.spot <- filter.spot(mySpot = Simon)
> ri.plot(f.spot)
\end{Scode}

\begin{Soutput}
Input: The argument of this function is a spot object.
Output: A filtered spot object. See Figure 10
\end{Soutput}

\subsubsection{Replicates Filtering}

Another step is the replicates filtering, and in this step, a lot of observations are eliminated. One of the function for this purpose is called \texttt{spotUnique} and this function compute the geometric mean for the replicates to average our measurements, and the adjusted average measures for each gene can then be used to carry out further analyses. See Figure \ref{fig11}.\\

\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(Simon)
u.spot <- spotUnique(mySpot = Simon)
ri.plot(u.spot)
@
\caption{R vs I plot of data after apply \texttt{spotUnique} function. \label{fig11}}
\end{center}
\end{figure}
\begin{Scode}
> data(Simon)
> u.spot <- spotUnique(mySpot = Simon)
> ri.plot(u.spot)
\end{Scode}

\begin{Soutput}
Input: The argument of this function is a spot object.
Output: A spot object without duplicates observations. See Figure 11
\end{Soutput}

However, genArise offers other functions for the same purpose, but with a different approach. We mean the function \texttt{alter.unique}. This function takes the R value of each one of the duplicated observations but just keep that observation with the extremest R value. So, if both of them are positives this function keeps the greater one, if both of them are negatives it keep the lower one and if there is one positive and one negative, both observations are eliminated. It is clear that with this function a greater number of observations is conserved compared to those that are obtained with \texttt{spotUnique} function. By this way you will probably have at the final of the analysis a greater number of observations in the upper-expressed and lower-expressed because you keep the extreme values. See Figure \ref{fig12}.\\

\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(Simon)
u.spot <- alter.unique(mySpot = Simon)
ri.plot(u.spot)
@
\caption{R vs I plot of data after apply \texttt{alter.unique} function.\label{fig12}}
\end{center}
\end{figure}
\begin{Scode}
> data(Simon)
> u.spot <- alter.unique(mySpot = Simon)
> ri.plot(u.spot)
\end{Scode}

\begin{Soutput}
Input: The argument of this function is a spot object.
Output: A spot object without duplicates observations. See Figure 12
\end{Soutput}

A third function compute the mean for the duplicates. This function keeps not just one element for each id, but also this function eliminates those points where the diference between the R value of the duplicated observations is bigger than 20\% of one of them. The name of this function is \texttt{meanUnique}. See Figure \ref{fig13}.

\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(Simon)
u.spot <- meanUnique(mySpot = Simon)
ri.plot(u.spot)
@
\caption{R vs I plot of data after apply \texttt{meanUnique} function.\label{fig13}}
\end{center}
\end{figure}
\begin{Scode}
> data(Simon)
> u.spot <- meanUnique(mySpot = Simon)
> ri.plot(u.spot)
\end{Scode}

\begin{Soutput}
Input: The argument of this function is a spot object
Output: A spot object without duplicates observations. See Figure 13
\end{Soutput}

\subsection{Identifying differentially expressed genes}

The approach we use involves calculating the mean and standard deviation of the distribution of $log_2$(ratio) values and defining a global fold-change difference and confidence; for this purpose you must compute a Z-score for the data set. In an R-I plot, that would be represented as lines following the structure of the data; genes outside of those lines would be the differentially expressed\footnotemark[2]. See Figure \ref{fig14}.\\

As a matter of fact genArise offers two options for this analysis. You just need to specify in the type argument on the \texttt{Zscore} function if you want a R-I or a M-A analysis. This function receive as argument an object of the class Spot and returns an object of other class called \emph{DataSet} that includes as one of their values the Z-score for the data set.\\

This is the example for \texttt{Zscore} using  R-I values.

\begin{Scode}
> data(Simon)
> s.spot <- Zscore(Simon, type="ri")
\end{Scode}

\begin{Soutput}
Input: The argument of this function is a spot object
Output: An object of the class DataSet
\end{Soutput}

And this is the example for \texttt{Zscore} using \texttt{M-A} values

\begin{Scode}
> data(Simon)
> s.spot <- Zscore(Simon, type="ma")
\end{Scode}

\begin{Soutput}
Input: The argument of this function is a spot object
Output: An object of the class DataSet
\end{Soutput}

\subsection{Plotting DataSets}

Since the objects of the \texttt{DataSet} class are different from the objects of the class \texttt{Spot} you need a function to plot them. For this purpose exists a function called \texttt{Zscore.plot}. So, in a R-I plot, array elements are color-coded depending on wether they are less than 1 standard deviation from the mean (green), between 1 and 1.5 standard deviations (blue), between 1.5 and 2 standard deviations (cyan), more than 1.5 standard deviations (yellow), or more than 2 standard deviations from the mean (white). We use a subset of a data set called WT for the next examples. WT is a DataSet object that we have previously created.

\begin{figure}[h]
\begin{center}
<<fig=TRUE, echo=FALSE>>=
data(WT.dataset)
Zscore.plot(WT.dataset, Zscore.min=1)
@
\caption{Data after Z-score Analysis. \label{fig14}}
\end{center}
\end{figure}
\begin{Scode}
> data(WT.dataset)
> Zscore.plot(WT.dataset)
\end{Scode}

\begin{Soutput}
Input: An object of the class DataSet
Output: Plot for identify differential expression. See Figure 14
\end{Soutput}

\subsection{genMerge}

After you finish your slice analysis with function \texttt{Zscore} you get an up-regulated and down-regulated set. This will be the set of study genes for genMerge. Given this set, the function \texttt{genMerge} retrieves functional genomic data for each gene and provides statistical rank scores for over-representation of particular functions in the dataset. The \texttt{genMerge} function is completly based on GeneMerge from Cristian I. Castillo-Davis and Daniel L. Hartl \footnote[3]{Cristian I. Castillo-Davis, Department of Statistics, Harvard University \url{http://www.oeb.harvard.edu/hartl/lab/publications/GeneMerge}
}.\\

Given a set of genes (for example the upper-expressed), genMerge write in files the data of an statistic analysis of the upper-representation of functions or particular categories on the set of data upper-expressed.\\

You must take care in the format of the input file, the file must not have a header row. This function requires four data files and the file name where will be save the obtained information. The main file that the function needs are:\\\\

\textbf{Association file}. This file contains two columns splited by tab characters. The first column corresponds to the gene identifier in the microarray and the second one corresponds to a list of associated ids for the geneontology database to this gen (GO). Each GO in this list is separted by ";" and is necessary to consult a database to locate the associated GOs to each gen (see http://www.geneontology.org)

Example:

\begin{Scode}
YAL001C GO:0003709;
YAL002W GO:0005554;
YAL003W GO:0003746;
YAL005C GO:0003754;GO:0003773;GO:0004002;
YAL007C GO:0005554;
\end{Scode}

\textbf{Description file}. This file also contains two columns splited by tab characters, the first one corresponds to the name of some GOs and the second one corresponds to the description associated to each GO in the database.

Example:

\begin{Scode}
GO:0000005      ribosomal chaperone activity
GO:0000006      high affinity zinc uptake transporter activity
GO:0000007      low-affinity zinc ion transporter activity
GO:0000008      thioredoxin
GO:0000009      alpha-1,6-mannosyltransferase activity
GO:0000010      trans-hexaprenyltranstransferase activity
GO:0000014      single-stranded DNA specific endodeoxyribonuclease activity
GO:0000016      lactase activity
\end{Scode}

\textbf{All genes (population.genes)}. This file should only contain one column, the spot identifiers or those of the input file not analyzed yet are the column information, each identifier is a row file.

\begin{Scode}
YAL001C
YAL002W
YAL003W
YAL004W
YAL005C
YAL007C
YAL008W
YAL009W
YAL010C
YAL011W
\end{Scode}

\textbf{The genes to study}. Like the preceding described file, this file only contains one column and each row corresponds to the name of one of the genes in the set that will be studied (upper-expressed or lower-expressed). 

\subsection{Using genMerge}

As the first step is create a spot object in the way described in the previous sections, the you must write the ids in a file, so you can do something like this:\\

\begin{Scode}
> # Let's suppose that original spot is o.spot
> # To write the population file you can use write.table
> ids <- attr(o.spot, "spotData")$Ids
> ids <- unique(ids)
> write.table(ids, "population.genes")
\end{Scode}

\begin{Soutput}
Output: File named population.genes that contains just the ids
\end{Soutput}

In the same way, with the \texttt{write.table} function you can write the identifiers of the genes set that will be studied. Suppose there exist a slice Spot called s.spot wich have the results after slice analysis, if you wish that your set of study have the upper-expressed identifiers and the lower-expressed identifiers to write the corresponding file, you must follow the next code.

\begin{Scode}
> # To write the study genes file
> study.ids <- c(s.spot$Id.up, s.spot$Id.down)
> study.ids <- unique(study.ids) 
> write.table(study.ids, "study.genes")
\end{Scode}

If you only want to preserve the identifier of some set, say upper-expressed or lower-expressed type the next lines:

\begin{Scode}
> # Only write upper-expressed
> study.ids <- s.spot$Id.up # to write lower-expressed use s.spot$Id.down
> study.ids <- unique(study.ids)
> write.table(study.ids, "study.genes")
\end{Scode}

The other files should be constructed consulting a database or could be downloaded from the internet, this files should contain the format described above, so that the genMerge function operates correctly. The sintax of \texttt{genMerge} function is the next:   

\begin{Scode}
> genMerge(gene.association, description, population.genes,
  study.genes, output.file = "GeneMerge.txt"){
\end{Scode}

The results will be in the corresponding file, in the above example \texttt{GeneMerge.txt}.

\subsection{Post-analysis}

This function allows you to perform a set combinatorial analysis between the results previously obtained in different projects. This function is called \texttt{post.analysis} and it is mandatory that you have done the \texttt{Zscore} operation in all the selected projects. It is important to clarify that this function receives a list of files with extension \texttt{prj} as argument and for this reason you can't use it if the results to compare was not obtained by the genArise GUI.\\

The function \texttt{post.analysis} receives as argument a list of projects and look for the \texttt{zscore.txt} files for each project in the list. Once located each \texttt{zscore.txt} file the function get a list of ids for each project under the next criterion: all the ids must have a zscore value inside a range defined by the user.\\

Once obtained the ids list for each project a number of files with extension \texttt{set} are created in a directory. The name of this files consists in a sequence of 0 and 1. The number of digits in the file names is the same to the number of projects in the list passed as argument to the function. There is then, a relation between the number of digits in the file names and the projects. This relation is defined by the position specified in the file \texttt{order.txt} in the same directory you have passed as another argument in the function.\\

Each file will contain the ids that are present in all the sets (projects) in agreement to the file \texttt{order.txt} have 1 in the digit corresponding in the file name and in addition, this ids are not present in all the other sets (identified with a 0 in the filename).\\



\section{The genArise GUI}

To reduce the size of the genArise package, in the text bellow the screenshots were not included. You can find the complete manual in the site of genArise \url{http://www.ifc.unam.mx/genarise}. The title is $genArise_images.pdf$.\\

For making the analysis easier, genArise joins all functions in a graphic interface, to call this GUI, you must type the next line from the R prompt:

\begin{Scode}
> genArise()
\end{Scode}

Now a menu-bar with two options and some icons for specific functions is displayed. When the mouse is passed over any icon in the main menu bar, a label with a description of the corresponding function will be displayed in the right side. From now on every icon in this menu bar will be identified by this description.\\

The GUI use R TclTk and if you are using the RGui in Windows, you can quickly notices that the RGui main window frequently "gets in the way", as genArise windows like to hide behind it. To avoid this, you can minimize the RGui after you call \texttt{genArise()}.\\

\subsection{Create a new project}

You should create a New Project following the sequence  \textbf{File $\rightarrow$ Project $\rightarrow$ New Project} in the menu, or click the icon \textbf{Create a new Project}.\\

A new window containing two panels (Input and Output) will be displayed. In the first of them you must specify the properties of the file containing the data, for example: location, was the experiment performed in the IFC?, microarray dimensions in other case. In this way, when the experiment was performed in the IFC you must choose the option \textbf{IFC} and the microarray dimensions will be automatically obtained.\\

In the input field labeled \textbf{Cy3}, \textbf{Cy5}, \textbf{BgCy3}, \textbf{BgCy5} and \textbf{Id} you must type the number of the column in the file corresponding Cy3 intensity, Cy5 intensity, Cy3 background, Cy5 background and the gene identifiers, respectively. The input field labeled \textbf{Symbol} is optional and and here you can type the number of the column in the file with an extra identifier (for example: $Gene_Symbol$).

In the inferior panel, there is an input field labeled with \textbf{Project Name} where you must type the name of the project to create and a new file with prj extension, as well as a directory with the specified name will be created. The next two input fields you must specify the name of the subdirectories from this project where the results files will be placed \textbf{(Location for Results)} and the plots \textbf{(Location for Plots)}. You can change this names writing on the input field a new file name (without the path) or click the \textbf{browse} button.\\

Then, the window containing a preliminary view of the main grid is displayed as an image plot in a green to red scale representing the $log_2$ intensity ratio for each spot on the array. It is important to clarify that these image plot does not replace the original TIFF image from the microarray experiment.

You can also see the green and red levels by separated just selecting the corresponding option in the lower right part of the window.\\

In the upper right side panel of the window you can see the experiment geometry.\\

You can save the plot in a pdf file by selecting the option \textbf{Save as PDF} in the menu \textbf{Options} or clicking in the \textbf{Save graphic as PDF} icon. With the option \textbf{Note} of the same menu or clicking in the \textbf{Notes about the experiment} icon, a text editor will be launched where you can make some important notes through the analysis and save these in a file called notes.txt.\\

\subsubsection{Data Analisis \label{secda}}
At this moment you can start the analysis by clicking on the menu \textbf{Analyze} or clicking in the \textbf{Make analysis data} icon, that will display a dialog box. Choosing \textbf{Yes} the program makes the operations in the following sequential order: background correction, loess normalization, intensity-filter, replicates filtering. Once this analysis is over you see the next window. \\

This window shows the graphic R -vs- I of the data obtained after the complete analysis. In the right side of the same window there are five options corresponding to the results of the operations performed through the analysis, choosing some of them the plot of the results for this step of the analysis will be displayed in the window.\\

genArise allows you to choose between three different graphic types (R -vs- I, M -vs- A o Cy3 -vs- Cy5) and you can display each one of this graphics just choosing the right option from the \textbf{Graphics} menu in the menu bar.\\

There is a white component in the lower side of the window where you can find the information for each one of the operations done.\\

At the end, you find one file for each operation done in the \textbf{Location for Results} subdirectory. All this files have a default name and you can't modify in no way this file name because they are essencial to be able to open this project with the genArise GUI in the future. \\

If you decide not to follow the wizard only the original data (no one operation performed) is displayed in a different window. In this case, you can perform any operation selecting the wished option in the menu bar. Once an operation is made, a new reference with the label corresponding to the operation performed will be showed up in the right side of the window and the graphic of the results obtained with that operation will be displayed. Is important to note that the operations are performed on the selected spot. If you wish to make a sequential analysis you must take care of the selected option.

\subsubsection{Zscore Window}
Clicking the \textbf{Zscore} option menu you can perform the operation Zscore described in section \ref{sec1} and after select this option a new window will be displayed. You can display six plots for different z-score values in a range defined by the label of the corresponding option. You must select the option \textbf{Write as output file} to get a file for the up-regulated genes an a file for the down-regulated genes in the selected z-score range. Each file include a header indicating the corresponding values.

\subsection{Open Project}
You can open and consult the data of a previously done analysis with the option \textbf{Open Project} from the genArise main menu. After click this option you must select the prj file with the same name of the wished project from the file selector window. After this, a new window is displayed where you can check all plots from the results of the analyasis in that project. This plots are obtained from the data in the result files. 

\subsection{Post-analysis Window}
This window allows you to obtain all the essencial information for the function \texttt{post.analysis} previously described. The list of the projects are choosed in the left side panel of the window. You can select several files with extension \texttt{prj} one by one just clicking the \textbf{Add} button.\\

You must type and select the other arguments for the function: the directory that will contain all the output files, the range for the zscore and if the analysis will be done with "up" or "down" regulated.

\subsection{Swap Analysis}
If you have a dye-swapped experiment, you can apply the Swap Analysis to get better results than apply the single experiments separately.\\

You should create a New Swap Project following the sequence  \textbf{File $\rightarrow$ Swap Analysis} in the menu, or click the icon \textbf{Dye-Swap Analysis}. A new window will be displayed.

\subsubsection{Read swap dye data file pair}
The new window allows you to select input data file pair from the File selector browser opened when the \textbf{Add file} button is clicked. When this File selector browser is displayed.\\

 Find the directory where data files are stored, all data files with the specified format type (csv, txt or *) in this directory will be displayed. From the file list, select the first file of a swap dye pair (the original) and click OK. Click again the \textbf{Add file} button and repeat the operation to select the second file of the swap dye pair (the swap). The file pair will be displayed in the left panel. Multiple pairs selection is not implemented for now. If two files are paired up by mistake, you can delete the incorrect file just clicking the \textbf{Delete file} button.\\

The right side of the window contain two panels (Input and Output) will be displayed. In the first of them you must specify the properties of the file pair containing the data, for example: was the experiment performed in the IFC?, microarray dimensions in other case. In this way, when the experiment was performed in the IFC you must choose the option \textbf{IFC} and the microarray dimensions will be automatically obtained.\\

In the input field labeled \textbf{Cy3}, \textbf{Cy5}, \textbf{BgCy3}, \textbf{BgCy5} and \textbf{Id}, you must type the number of the column in the files corresponding Cy3 intensity, Cy5 intensity, Cy3 background, Cy5 background and the gene identifiers, respectively.\\

In the Output panel, there is an input field labeled with \textbf{Project Name} where you must type the name of the swap project to create, and a new file with prj extension, as well as a directory with the specified name will be created. The next two input fields you must specify the name of the subdirectories from this project where the results files will be placed \textbf{(Location for Results)} and the plots \textbf{(Location for Plots)}. You can change this names writing on the input field a new file name (without the path) or click the \textbf{browse} button.

\subsubsection{Swap Dye Diagnostic graph and swap dye consistency checking}
After create a new swap project, a new window is displayed where a Swap Dye Diagnostic graph is plotted based on $log_2 \frac{Cy5}{Cy3}$ of file 1 (original) -vs- $log_2 \frac{Cy3}{Cy5}$ of file 2 (swap). This kind of graph provides an good visual summary of how consistent the gene expressions are between a pair of Swap-dye experiment.\\

Swap dye consistency checking allows filtering out spots that show expression level inconsistency between a pair of flip-dye replicates. For this function the next steps are performed.\\

Calculate the cross-log-ratios of the data file pair for each spot.\\

\begin{center} $R = log_2 \frac{Cy5_1 * Cy3_2}{Cy3_1 * Cy5_2}$\end{center}
If any $Cy5_1, Cy3_2, Cy3_1, Cy5_2$ is 0, set R = 0.\\

Calculate the mean and standard deviation of all none-zero R.\\

For each spot:\\

A 2 SD cut is used, and the spots with theirs R values having $R \le mean(R) - \sigma(R) * 2$ or $R \ge mean(R) + \sigma(R) * 2$ are marked as swap-dye inconsistent  and will be excluded from following analysis.\\

For those spots passed the swap-dye consistency checking, the $Cy3$ and $Cy5$ intensities from both files will be merged. The merged intensities will be the geometric means of the swap-dye replicates. In the merged file, spot information other than $Cy3$ and $Cy5$ intensities will be the geometric mean of the data in the pair.\\

 Two factors are computed to describe the overall consistency level between the swap-dye pair and both values are shown in the lower-right panel of the window. The two factors are Confidence Factor and Dispersion Factor.

\begin{center}$Confidence Factor = \frac{NumberOfSpotsSatisfyTheConsistencyCriteria}{TotalNonZeroIntensities}$\end{center}
\begin{center}$Dispersion Factor = \frac{NumberOfSpotsWithInconsistentRatioBeyond2SD}{TotalNonZeroIntensities}$\end{center}

If the R values follow normal distribution (closely), the Confidence Factor for SD cut range of $|2SD|$ should be very close to 95\%.\\

\subsubsection{Data Analisis}
After this you can perform the analysis as described in the \textbf{section} \ref{secda}. The background correction is performed by default and for the other operations you can follow the wizard or customize the order of the operations selecting the wished option in the menu bar.

\subsubsection{Zscore Window}
Clicking the \textbf{Zscore} option menu you can perform the operation Zscore described in section \ref{sec1} and after select this option a new window will be displayed. You can display six plots for different z-score values in a range defined by the label of the corresponding option. You must select the option \textbf{Write as output file} to get a file for the up-regulated genes an a file for the down-regulated genes in the selected z-score range. For this dye-swapped experiment, the first column of the z-score files is the Id for aech spot, the second column is the Z-score for each cross-ratio (both files), the third column is the Zscore for the file 1 and the fourth column is the Zscore for the file 2. Where a zero appear in one of the last two columns, that means that this spot was eliminated in one of the preprocessing transformations before the Z-score analysis of the corresponding experiment. Each file include a header indicating the corresponding values.

\end{document}

\name{principal}
\alias{principal}
\title{Principal window of genAriseGUI}
\description{
  This function show a window with the information of experiment like
  name and dimensions, too plot an  image of colours representing the
  log intensity ratio for each spot on the array.
  This is just an auxiliar function for genAriseGUI, and can not beused
  in the command line.
}
\usage{
principal(envir, swap)
}
\arguments{
  \item{envir}{Environment where are the genArise variables}
  \item{swap}{Is this a swap analysis or an individual analysis}
  }
\value{
  tkwidget
}
\keyword{color}

\name{get.values}
\alias{get.values}
\title{Auxiliar function for post-analysis}
\description{
  This function get values from an DataSet object.
  
  This is just a function for the GUI, and can not be
  used in the command line.
}
\usage{
get.values(list.values, genes.values, up.down, min.val, max.val)
}
\arguments{
  \item{list.values}{Zscore values from DataSet object}
  \item{genes.values}{Ids values from DataSet object}
  \item{up.down}{If the analysis will be done with "up" or "down" regulated}
  \item{min.val}{Minimal value of the range}
  \item{max.val}{Maximal value of the range}
}
\value{
  An Ids list
 }
\keyword{arith}

\name{genArise.init}
\alias{genArise.init}
\title{genArise.init}
\description{
Auxiliar function of genArise GUI, this function show a principal menu
of genAriseGUI
}
\usage{
genArise.init(envir)
}
\arguments{
  \item{envir}{Environment where are the project variables}
}
\value{
  tkwidget
}
\keyword{math}

\name{write.zscore}
\alias{write.zscore}
\title{Write Z-score data}
\description{
  Write the values for observations of an object of DataSet class in an
  output file. This values are writen in columns tab separated with the
  follow order:
  Cy3, Cy5, Cy3 Background, Cy5 Background, Ids and finally the z-score
  value. The header of the output file is the selected type for the
  z-score (ri or ma).}
\usage{
write.zscore(dataSet.spot, fileName, sep = "\t")
}
\arguments{
\item{dataSet.spot}{An object of DataSet class}
\item{fileName}{The name of the output file where the data will be
  writen. This argument must be quoted.}
\item{sep}{Character to separate the columns in file. By default sep = "\\t".}
}
\examples{
data(WT.dataset)
write.zscore(dataSet.spot = WT.dataset, fileName = "Zscore.csv", sep =
"\t")
}
\keyword{file}

\name{alter.unique}
\alias{alter.unique}
\title{Remove Duplicates}
\description{
This function allows to remove from the spot repeated Id's. Before
moving one of the repeated Id's the function compute the log ratio of
both values with the same Id and delete the least absolute value if both
of them are positive or negative. In other case delete both observations. 
}
\usage{
alter.unique(mySpot)
}
\arguments{
\item{mySpot}{Spot object for one microarray.}
}
\value{
Spot object without duplicates.
}
\examples{
data(Simon)
## filter the spot and save it in f.spot
f.spot <- filter.spot(Simon)
## remove duplicates and save it in u.spot
u.spot <- alter.unique(f.spot)
}
\keyword{math}
\keyword{arith}

\name{write.dataSet}
\alias{write.dataSet}
\title{Write dataSet}
\description{
  Write the values for observations of an object of DataSet class in an
  output file. This values are writen in columns with the follow order:
  Cy3, Cy5, Cy3 Background, Cy5 Background, Ids and finally the Zscore
  value. By default this output file has no header.}
\usage{
write.dataSet(dataSet.spot, fileName, quote
= FALSE, col.names = FALSE, row.names = FALSE,
Zscore.min = NULL, Zscore.max = NULL, sep = "\t")
}
\arguments{
\item{dataSet.spot}{An object of DataSet class}
\item{fileName}{The name of the output file where the data will be
  writen. This argument must be quoted.}
\item{quote}{If quote = TRUE, all values in the file will be quoted.}
\item{col.names}{If col.names = TRUE, an integer is writen in every
  column as header. By default col.names = FALSE.}
\item{row.names}{If row.names = TRUE will be an extra column that
  numerates every rows in the file.}
\item{Zscore.min}{The lower value in a range, if Zscore.min = NULL then
  the file will contain all values bellow Zscore.max}
\item{Zscore.max}{The greater value in a range, if Zscore.max = NULL
  then file will be contain all values above Zscore.min. Both values,
  Zscore.min and Zscore.max can not be NULL}
\item{sep}{Character to separate the columns in file. By default sep
  = "\\t".}
}
\examples{
data(WT.dataset)
write.dataSet(dataSet.spot = WT.dataset, fileName = "Example.csv", Zscore.min = 1,
Zscore.max = 1.5, sep = "\t")
}
\keyword{file}

\name{Zscore.plot}
\alias{Zscore.plot}	
\title{Z-score Data Visualization: R vs I or M vs A}
\description{
This function allows to plot \bold{R-values} vs \bold{I-values} or
\bold{M-values} vs \bold{A-values} for identifying differential expression.}
\usage{
Zscore.plot(dataSet.spot, Zscore.min, Zscore.max, all, col)
}
\arguments{
  \item{dataSet.spot}{Spot Object}
  \item{Zscore.min}{The lower value in a range, if Zscore.min = NULL then
    the file will contain all values bellow Zscore.max}
  \item{Zscore.max}{The greater value in a range, if Zscore.max = NULL
    then file will be contain all values above Zscore.min. Both values,
    Zscore.min and Zscore.max can not be NULL}
  \item{all}{Plot all the observations in four sets: Z < 1, 1 < Z <
    1.5, 1.5 < Z < 2, Z > 2}
    \item{col}{ Color in which the pioints of the plot will be shown where
    only the points from center are plot. This argument must be quoted
    and the possible values it can take are the same from the colors
    function in the R base package.}
  }
\examples{
data(WT.dataset)
Zscore.plot(WT.dataset, Zscore.min = 1, Zscore.max = 2)
}
\seealso{colors()}
\keyword{aplot}

\name{make.swap}
\alias{make.swap}
\title{Swap analysis}
\description{
Read both files, but only extract the interested columns and create a Spot object.
}
\usage{
make.swap(spot1, spot2, Cy3, Cy5, BgCy3, BgCy5, Id, Symdesc, header = FALSE, is.ifc = FALSE,envir,nr,nc)
}
\arguments{
\item{spot1}{a connection or a character string giving the
  name of the file to read where each column represent the spot
  components.}
\item{spot2}{a connection or a character string giving the
  name of the file to read where each column represent the spot
  components.}
\item{Cy3}{column that represent Cy3.}
\item{Cy5}{column that represent Cy5.}
\item{BgCy3}{column that represent BgCy3.}
\item{BgCy5}{column that represent BgCy5.}
\item{Id}{column that represent Id.}
\item{Symdesc}{optional identifier besides the Id column.}
\item{header}{the logical value of the header input file}
\item{is.ifc}{If is.ifc = TRUE this experiment was done in the
  Unit of Microarray  from  Cellular Phisiology Institute.}
\item{envir}{Environment where are the genArise variables.}
\item{nr}{Total of meta-row.}
\item{nc}{Total of meta-column.}
}
\seealso{
\code{\link{write.spot}}.
}
\keyword{file}

\name{set.history.project}
\alias{set.history.project}
\title{Save the history of a project}
\description{
Save in the history file each operation performed while the
analysis. This is just to get the open this particular project in the
future. This is just an auxiliary function for the GUI, and can not be
used in the command line.
}
\usage{
set.history.project(history.file, id.name, data.file)
}
\arguments{
  \item{history.file}{The name of the prj history file.}
  \item{id.name}{The name of the operation.}
  \item{data.file}{The file with the results of the operation.}
}
\value{
  The history file with the new performed operation.
}
\keyword{file}

\name{project.select}
\alias{projects.select}
\title{File selector}
\description{
  Previous window to post-analysis. In this window you can select one or
  several files (projects) and arguments to be used by post analisis
  function.
  
  This is just an auxiliar function for genAriseGUI, and can not be used
  in the command line.
}
\usage{
projects.select(envir, nombre)
}
\arguments{
  \item{envir}{Environment where are the genArise variables}
  \item{nombre}{Name of directory where the post-analysis results will
    be placed.}
}
\value{
  tkwidget
}
\keyword{file}

\name{global.norm}
\alias{global.norm}
\title{Global Normalization of Spot}
\description{
  This function normalize R and I values and fit the value of Cy5
  from his argument. In this function the normalize algorithm will be
  applied to all observations to get the lowess factor and then fit
  Cy5 with this factor. The observations. The observations with values
  R = 0 are deleted because they have no change in their expression levels.}
\usage{
global.norm(mySpot)
}
\arguments{
\item{mySpot}{A spot object}
}
\value{
A new spot object but normalized, It means with a different Cy5 that is
the result of the fit with the lowess factor.}
\examples{
data(Simon)
# Background Correction
c.spot <- bg.correct(Simon)
#Normalized data
n.spot <- global.norm(c.spot)
}
\keyword{arith}
\keyword{math}

\name{Spot-class}
\docType{class}
\alias{Spot-class}
\title{Spot - class}

\description{
A simple list-based class for storing red and green channel foreground
and background intensities for a batch of spotted microarrays and the Ids.
}

\section{Creating Objects from the Class}{
Objects can be created by calls of the form \code{new("Spot",spot)} where \code{spot} is a list.
Objects are normally created by \code{\link{read.spot}}.
}

\section{Slots/List Components}{
This class contains no slots (other than \code{.Data}), but objects should contain the following list components:
\tabular{ll}{
  \code{Cy5}:\tab numeric matrix containing the red (cy5) foreground
  intensities. Rows correspond to spots and columns to arrays.\cr
  \code{Cy3}:\tab numeric matrix containing the green (cy3) foreground intensities.\cr
  \code{BgCy5}:\tab numeric matrix containing the red (cy5) background intensities.\cr
  \code{BgCy3}:\tab numeric matrix containing the green background intensities.\cr
  \code{Id}:\tab Ids from all the observations.
}
All of these matrices should have the same dimensions.
}

\section{Methods}{
This class inherits directly from class \code{list} so any operation appropriate for lists will work on objects of this class.
}

\keyword{classes}
\keyword{data}

\name{Zscore.points}
\alias{Zscore.points}	
\title{Z-score Window}
\description{
  This function display the window that show the results after the
  Z-score. This window allow:

  1. Show the plots of the up and down generated with the function Zscore.plot
  regulated spots in:
  Zscore < 1 sd
  1 sd < Zscore < 1.5 sd
  1.5 sd < Zscore < 2 sd
  Zscore > 2 sd
  and All the points

  2. Save the plots in pdf and save the results in an output file
  
  3. Gene annotations. Denote any gene information beyond the expression
  level data.

  This is just a function for the GUI, and can not be
  used in the command line.
}
\usage{
  Zscore.points(type,text,envir, swap)
}
\arguments{
  \item{type}{Type of analysis done: "ri" is for a R-I analysis and "ma" is
    for M-A analysis}
  \item{text}{The text for the text area of the history of the project}
  \item{envir}{Environment where the variables are stored}
  \item{swap}{Is this a swap analysis or an individual analysis}
  }
\keyword{aplot}

\name{swap.select}
\alias{swap.select}
\title{Dye swap files selector}
\description{
  This is just an auxiliar function for genAriseGUI, and can not be used
  in the command line.
}
\usage{
swap.select(envir)
}
\arguments{
  \item{envir}{Environment where are the genArise variables}
}
\value{
  tkwidget
}
\keyword{file}

\name{a.arise}
\alias{a.arise}
\title{A Arise}
\description{
Extract \bold{A values} from a Spot.
}
\usage{
a.arise(mySpot)
}
\arguments{
\item{mySpot}{Spot object for one microarray.}
}
\value{
List of A-values. (log(cy3, 2) + log(cy5, 2))/2
}
\seealso{
\code{\link{m.arise}}.
}
\examples{
## read the spot from a file and save it in spot
data(Simon)
## Extract A from spot and save in a
a <- a.arise(mySpot = Simon)
}
\keyword{math}
\keyword{arith}

\name{spotUnique}
\alias{spotUnique}
\title{Replicate filtering}
\description{
We consider replicate measures of two samples and adjust the
log(ratio,2) measures for each gene so that the transformed values are
equal. To do this we take the geometric mean.\\
This procedure can be extended to averaging over \code{n} replicates.
}
\usage{
spotUnique(mySpot)
}
\arguments{
\item{mySpot}{Spot object for one microarray.}
}
\value{
Spot object without duplicates
}
\examples{
data(Simon)
c.spot <- bg.correct(Simon)
f.spot <- filter.spot(c.spot)
spotUnique(mySpot = f.spot)
}
\keyword{math}

\name{genMerge}
\alias{genMerge}
\title{genMerge: Post-Genomic Analysis}
\description{
  After we finished our slice analysis we get a up-regulated and
  down-regulated set. This will be the set of study genes for genMege.
  Given this set, genMerge retrieves functional genomic data for each
  gene and provides statistical rank scores for over-representation of
  particular functions in the dataset.
}
\usage{
genMerge(gene.association, description, population.genes,
study.genes, output.file = "GenMerge.txt")
}
\arguments{
  \item{gene.association}{The gene-association file links gene names
    with a particular datum of information using a shorthand of
    gene-association IDS}
  \item{description}{The description file contains human-readable
    description of gene-association IDS}
  \item{population.genes}{Set of all genes detected on a array}
  \item{study.genes}{Set of genes may be those that are up-regulated or
    down-regulated or both of them.}
  \item{output.file}{The name of output file that includes all results
    obtained after this analisys.}
}
\note{This function is completly based on GeneMerge from Cristian
  I. Castillo-Davis and Daniel L. Hartl}
\references{
  Cristian I. Castillo-Davis
  Department of Statistics
  Harvard University
  \url{http://www.oeb.harvard.edu/hartl/lab/publications/GeneMerge}
}
\keyword{math}

\name{old.project}
\alias{old.project}
\title{Open previous project}
\description{
  Show the information that was obtained from the analysis of a
  previous project. This is just an auxiliar function for genAriseGUI,
  and can not be used in the command line.
}
\usage{
old.project(project.name,envir, parent)
}
\arguments{
  \item{project.name}{path of project file (PRJ)}
  \item{envir}{Environment where are the genArise variables}
  \item{parent}{The parent widget}
}
\value{
  tkwidget
}
\keyword{file}

\name{reset.history}
\alias{reset.history}
\title{Reset the prj history file}
\description{
Clean all the operations saved in the prj history file.
}
\usage{
reset.history(history.file, text)
}
\arguments{
  \item{history.file}{The name of the prj history file.}
  \item{text}{The new content of the prj history file.}
}
\value{
  The history file without operations.
}
\keyword{file}

\name{set.project.properties}
\alias{set.project.properties}
\title{set.project.properties}
\description{
Auxiliar function for genAriseGUI
}
\usage{
set.project.properties(envir)
}
\arguments{
\item{envir}{Environment where are the experiment variables}
}
\keyword{arith}

\name{graphic.choose}
\alias{graphic.choose}
\title{Graphic choose}
\description{
  This function show the plot of an spot sobject. This plot are identify
  with the graphic.type.value
}
\usage{
graphic.choose(spot.object, graphic.type)
}
\arguments{
  \item{spot.object}{An object ob Spot class}
  \item{graphic.type}{representative integer of type graphic will be plot}
}
\value{
  Plot device
}
\keyword{math}

\name{back.gui}
\alias{back.gui}
\title{Return to the last window}
\description{
Auxiliar function of genArise GUI.
}
\usage{
back.gui(envir)
}
\arguments{
  \item{envir}{Environment where are the project variables}
}
\value{
  tkwidget
  }
\keyword{IO}

\name{meanUnique}
\alias{meanUnique}
\title{Remove Duplicates}
\description{
  This function allows to remove from the spot repeated Id's. Before
  moving one of the repeated Id's the function compute the average of
  Cy3 intensity and Cy5 intensity.
  
}
\usage{
meanUnique(mySpot)
}
\arguments{
\item{mySpot}{Spot object for one microarray.}
}
\value{
Spot object without duplicates
}
\examples{
data(Simon)
c.spot <- bg.correct(Simon)
n.spot <- global.norm(c.spot)
f.spot <- filter.spot(n.spot)
meanUnique(f.spot)
}
\keyword{math}

\name{cys.plot}
\alias{cys.plot}
\title{Data Visualization: log2(Cy3) vs log2(Cy5)}
\description{
  This function shows the plot of the values from the log Cy3 against
  log Cy5 intensities that belongs to an object of the Spot class.}
\usage{
cys.plot(mySpot, col = "green")
}
\arguments{
  \item{mySpot}{An Spot object}
  \item{col}{Color in which the points of the plot will be shown. This
    argument must be quoted and the possible values it can take are the
    same from the color funcion in the R base.}
}
\examples{
data(Simon)
cys.plot(Simon)
}
\keyword{graphs}
\keyword{aplot}

\name{set.grid.properties}
\alias{set.grid.properties}
\title{set.grid.properties}
\description{
Auxiliary function for genAriseGUI
}
\usage{
set.grid.properties(envir, name, nr, nc, nmr, nmc)
}
\arguments{
  \item{envir}{Environment where the variables are stored}
  \item{name}{The name of the experiment}
  \item{nr}{Total rows in the array (each row represent a spot)}
  \item{nc}{Total columns in the array}
  \item{nmr}{Total of meta-rows}
  \item{nmc}{Total of meta-columns}
}
\keyword{IO}

\name{grid.norm}
\alias{grid.norm}
\title{Normalization by grid of Spot}
\description{
This function normalize R and I values and fit the value of Cy5 for each
grid in the spot that it receives as argument. In this function the
dimension of grid is (meta-row * meta-column).}
\usage{
grid.norm(mySpot, nr, nc)
}
\arguments{
\item{mySpot}{Spot object for one microarray.}
\item{nr}{Total of meta-row.}
\item{nc}{Total of meta-column.}
}
\value{
Spot object with the grid normalization done. 
}
\examples{
data(Simon)
## background correction and save it in c.spot
c.spot <- bg.correct(Simon)
## normalization and save it in n.spot
n.spot <- grid.norm(c.spot, 23, 24)
}
\keyword{arith}
\keyword{math}

\name{filter.spot}
\alias{filter.spot}
\title{Intensity-based filtering of array elements}
\description{
  This function keep only array elements with intensities that are 2
  standard deviation above background.
}
\usage{
filter.spot(mySpot)
}
\arguments{
  \item{mySpot}{Spot object for one microarray.}
}
\value{
  Array elements with intensities that are 2 standard deviation above
  background.
}
\references{
  John Quackenbush "Microarray data normalization and transformation".
  Nature Genetics. Vol.32 supplement pp496-501 (2002)
  }
\examples{
data(Simon)
## background correction and save it in c.spot
c.spot <- bg.correct(Simon)
## normalize spot
n.spot <- grid.norm(c.spot, nr = 23, nc = 24)
## filtering the spot
filter.spot(n.spot)
}
\keyword{arith}
\keyword{math}

\name{note}
\alias{note}
\title{note}
\description{
Call a editor for note about actual experiment
}
\usage{
note(envir)
}
\arguments{
\item{envir}{Environment where are the experiment variables}
}
\keyword{arith}

\name{read.dataset}
\alias{read.dataset}
\title{Read Dataset from File}
\description{
Read all file and extract the interested columns to create a DataSet
object (this file contain the zscore with all the genes after the
duplicates filtering and makes not distinction between up-regulated
and down-regulated. If you want to make this distinction you must write
the data with the function write.dataSet, but there is no way to read
this files with this function).
}
\usage{
read.dataset(file.name, cy3 = 1, cy5 = 2, ids = 3, symdesc = NULL,
zscore = 4, type = 6, header = FALSE, sep = "\t")
}
\arguments{
\item{file.name}{a connection or a character string giving the
name of the file to read where each column represent the dataset components.}
\item{cy3}{column that represent Cy3.}
\item{cy5}{column that represent Cy5.}
\item{ids}{column that represent Id.}
\item{symdesc}{optional identifier besides Id column.}
\item{zscore}{column that represent the zscore value.}
\item{type}{column that represent if the experiment was performed as R
  vs I or M vs A.}
\item{header}{the logical value of the header input file}
\item{sep}{the separator in the inputfile}
}
\seealso{
\code{\link{write.zscore}}.
}
\keyword{file}

\name{annotations}
\alias{annotations}
\title{Gene Annotations}
\description{
  Performed an HTML file
}
\usage{
annotations(specie.data, specie, column, symbol,
output.file = "annotations.html")
}
\arguments{
  \item{specie.data}{A data frame}
  \item{specie}{Name of specie}
  \item{column}{Number of column where are the gene name in the data
    frame}
  \item{symbol}{An optional symbol besides GenBank ID}
  \item{output.file}{Name of output file }
}
\value{
  HTML file with link for each spot in data frame
}
\keyword{math}

\name{bg.correct}
\alias{bg.correct}
\title{Background Correction}
\description{
This function use the background data to eliminate unwanted effects in signal.
The background correction establish the new Cy3 signal as Cy3 - BgCy3 and 
the new Cy5 as Cy5 - BgCy5.
}
\usage{
bg.correct(mySpot)
}
\arguments{
\item{mySpot}{Spot object for one microarray.}
}
\value{
Spot object with the background correction done. 
}
\examples{
data(Simon)
## background correction and save it in c.spot
c.spot <- bg.correct(Simon)
}
\keyword{math}

\name{analysis.window}
\alias{analysis.window}
\title{Analysis.window}
\description{
Auxiliar function of genArise GUI, in this window you can apply
operations to original data.
}
\usage{
analysis.window(texto, follow.wizard = FALSE, envir, swap)
}
\arguments{
  \item{texto}{Historial project string}
  \item{follow.wizard}{Boolean value, if this argument is TRUE, an data
    analysis are performed}
  \item{envir}{Environment where are the project variables}
  \item{swap}{Is this a swap analysis or an individual analysis}
}
\value{
  tkwidget
  }
\keyword{math}

\name{help}
\alias{help}
\title{Help of genArise}
\description{
  Display the help of genArise in the GUI. This is just a function for the GUI, and can not be
  used in the command line.
}
\usage{
help()
}
\keyword{documentation}

\name{imageLimma}
\alias{imageLimma}
\title{Image Plot of Microarray}
\description{
  Plot an image of colours representing the log intensity ratio for each 
  spot on the array. This function can be used to explore whether there
  are any spatial effects in the data.
}
\usage{
imageLimma(z, row, column, meta.row, meta.column,
low = NULL, high = NULL)
}
\arguments{
  \item{z}{numeric vector or array. This vector can contain any spot
    statistics, such as log intensity ratios, spot sizes or
    shapes, or t-statistics. Missing values  are allowed and will
    result in blank spots on the image }
  \item{row}{rows in the microarray }
  \item{column}{columns in the microarray }
  \item{meta.row}{ metarows in the microarray}
  \item{meta.column}{ metacolumns in the microarray}
  \item{low}{color associated with low values of 'z'. May be specified as
    a character string  such as '"green"', '"white"' etc, or as a
    rgb vector in which 'c(1,0,0)' is red,  'c(0,1,0)' is green
    and 'c(0,0,1)' is blue. The default value is '"green"' if
    'zerocenter=T' or '"white"' if 'zerocenter=F'.}
  \item{high}{color associated with high values of 'z'. The default value
    is '"red"' if 'zerocenter=T' or '"blue"' if 'zerocenter=F'.}
}
\note{This function is based in the imageplot function from limma package.}
\references{
  Gordon K. Smyth (2004) "Linear Models and Empirical Bayes Methods for
  Assessing Differential Expression in Microarray Experiments",
  Statistical Applications in Genetics and Molecular Biology: Vol. 3:
  No. 1, Article 3.
  \url{http://www.bepress.com/sagmb/vol3/iss1/art3}
}
\examples{
data(Simon)
spot.data <- attr(Simon, "spotData")
M <- log(spot.data$Cy5, 2) - log(spot.data$Cy3, 2)
imageLimma(z = M, row = 23, column = 24, meta.row = 2, meta.column = 2,
low = NULL, high = NULL)
}
\keyword{color}

\name{single.norm}
\alias{single.norm}
\title{Swap from Files}
\description{
Read both files, but only extract the interested columns and create a Spot object.
}
\usage{
single.norm(envir)
}
\arguments{
\item{envir}{Environment where are the genArise variables.}
}
\seealso{
\code{\link{write.spot}}.
}
\keyword{file}

\name{Zscore}
\alias{Zscore}
\title{Z-scores for identifying differential expression}
\description{
  This function identify differential expressed genes by calculating an
  intensity-dependent Z-score. This function use a sliding window to
  calculate the mean and standard deviation within a window surrounding
  each data point, and define a Z-score where Z measures the number of
  standard deviations a data point is from the mean.}
\usage{
Zscore(spot.object,type,window.size)
}
\arguments{
  \item{spot.object}{A spot object}
  \item{type}{Type of analysis: "ri" is for a R-I analysis and "ma" is
    for M-A analysis}
  \item{window.size}{Size of the sliding window}
}
\value{
A dataSet object with attributes Cy3, Cy5, Id, Z-score.}
\examples{
data(Simon)
# Background Correction
c.spot <- bg.correct(Simon)
#Normalized data
n.spot <- grid.norm(c.spot,23,24)
#Filter spot
f.spot <- filter.spot(n.spot)
#Replicate filtering
u.spot <- spotUnique(f.spot)
#Zscore analysis
s.spot <- Zscore(u.spot)
}
\keyword{arith}
\keyword{math}

\name{m.arise}
\alias{m.arise}
\title{M Arise}
\description{
Extract \bold{M values} from a Spot.
}
\usage{
m.arise(mySpot)
}
\arguments{
\item{mySpot}{Spot object for one microarray.}
}
\value{
List of M-values
}
\seealso{
\code{\link{a.arise}}.
}
\examples{
data(Simon)
## Extract M from spot and save in m
m <- m.arise(Simon)
}
\keyword{math}
\keyword{arith}

\name{write.spot}
\alias{write.spot}
\title{Write Spot}
\description{
  Write the values for observations of an object of Spot class in an
  output file. This values are writen in columns with the follow order:
  Cy3, Cy5, Cy3 Background, Cy5 Background and finally Ids. By default
  this file has no header.}
\usage{
write.spot(spot, fileName, quote = FALSE,sep = "\t",
col.names = FALSE, row.names = FALSE)
}
\arguments{
\item{spot}{An object of Spot class}
\item{fileName}{The name of the output file where the data will be
  writen. This argument must be quoted.}
\item{quote}{If quote = TRUE, all values in the file will be quoted.}
\item{sep}{Character to separate the columns in file. By default sep = "\\t".}
\item{col.names}{If col.names = TRUE, an integer is writen in every
  column as header. By default col.names = FALSE.}
\item{row.names}{If row.names = TRUE will be an extra column that
  numerates every rows in the file.}
\code{\link{read.spot}}.
}
\examples{
data(Simon)
write.spot(spot = Simon, fileName = "Example.csv", quote = FALSE, sep =
"\t", col.names = FALSE, row.names = FALSE)
}
\keyword{file}

\name{create.project}
\alias{create.project}

\title{Create directorie for the project and its results and graphics}
\description{
Auxiliar function for genAriseGUI. Create the directory's hierarchy of
the project.
}
\usage{
create.project(project.name, results.file = "Results",
               graphics.file = "Graphics")
}
\arguments{
  \item{project.name}{Project directory name.}
  \item{results.file}{Filename of the project result.}
  \item{graphics.file}{Filename of the project graphics.}
}
\keyword{arith}

\name{Simon}
\alias{Simon}
\title{Dataset: Little fragment of a microarray from IFC UNAM}
\description{
  This structure is a data fragment of a yeast microarray from the
  Microarrays Unit in IFC UNAM. The original microarray contains 6
  meta-rows and 4 meta-columns, however this data just belongs to the
  first meta-row order in a way of 2 meta-rows and 2 meta-columns.} 
\usage{data(Simon)}
\format{A list that contains 1104 observations, because the dimensions
  of this example are: 2 meta-rows, 2 meta-columns, 23 rows, 24 columns.}
\examples{
data(Simon)
#A preview from the chip
datos <- attr(Simon, "spotData")
M <- log(datos$Cy3, 2) - log(datos$Cy5, 2)
imageLimma(M, 23, 24, 2, 2)
}
\keyword{datasets}

\name{WT.dataset}
\alias{WT.dataset}
\title{Microarray from the IFC}
\description{
This data set is a Microarray from the IFC.
}
\usage{data(WT.dataset)}
\format{A vector containing 4036 observations.}
\examples{
data(WT.dataset)
Zscore.plot(WT.dataset)
}
\keyword{datasets}

\name{set.path.project}
\alias{set.path.project}
\title{set.path.project}
\description{
Auxiliar function for genAriseGUI
}
\usage{
set.path.project(path, results.file, graphics.file, envir)
}
\arguments{
  \item{path}{Project path value}
  \item{results.file}{Name of directory where results file will be}
  \item{graphics.file}{Name of directory where pdf graphics will be}
  \item{envir}{Environment where are the experiment variables}
}
\keyword{arith}

\name{post.analysis}
\alias{post.analysis}
\title{Set-combinatorial analysis}
\description{
  This function allows you to perform a set combinatorial analysis
  between the results previously obtained in different projects.
  This function is called post.analysis and it is mandatory that
  you have done the Zscore operation in all the selected projects. It is
  important to clarify that this function receives a list of files with
  extension \code{prj} as argument and for this reason you can't use
  it if the results to compare was not obtained by the genArise GUI.
}
\usage{
post.analysis(values, min.val, max.val, up.down, output)
}
\arguments{
  \item{values}{A list of projects to compare}
  \item{min.val}{The minimal value of the range}
  \item{max.val}{The maximal value of the range}
  \item{up.down}{If the analysis will be done with "up" or "down" regulated}
  \item{output}{The directory that will contain all the output files}
  
  
}
\value{
  Once obtained the ids list for each project a number of files with
  extension set are created in a directory. The name of this files
  consists in a sequence of 0 and 1. The number of digits in the file
  names is the same to the number of projects in the list passed as
  argument to the function. There is then, a relation between the number
  of digits in the file names and the projects. This relation is defined
  by the position specified in the file order.txt in the same directory
  you have passed as another argument in the function.
}
\keyword{file}

\name{classes}
\alias{classes}
\title{Classes Defined by this Package}
\description{
This package defines the following data classes.
\describe{
\item{ \code{\link[genArise:Spot-class]{Spot}} }{
	A class used to store spot data with the following attributes:
	Cy3, Cy5, BgCy3, BgCy5, Ids as they are read by \code{read.spot}
      or obtained from a function that return a spot object.}
\item{ \code{\link[genArise:DataSet-class]{DataSet}} }{
  A class used to store spot data with the following attributes:
  Cy3, Cy5, Ids, Z-score.}
}}
\keyword{documentation}

\name{ri.plot}
\alias{ri.plot}	
\title{Data Visualization: R vs I}
\description{
This function allows to plot \bold{R-values} vs \bold{I-values}
\bold{I-value} from a Spot object}
\usage{
ri.plot(mySpot, col = "green")
}
\arguments{
  \item{mySpot}{Spot Object}
  \item{col}{ Color in which the pioints of the plot will be shown. This
    argment must be quoted and the possible values it can ake ares the
    same from the colors funcion in the R base package.}
  }
\examples{
data(Simon)
ri.plot(Simon)
}
\seealso{colors()}
\keyword{aplot}

\name{r.arise}
\alias{r.arise}
\title{Get R value}
\description{
Get the \bold{R values} from an object of the Spot class.}
\usage{
r.arise(mySpot)
}
\arguments{
\item{mySpot}{An object of the Spot class}
}
\value{
A vector containing the R value ( log(Cy5/Cy3)) for each observation of
the spot object.}
\seealso{
\code{\link{i.arise}}.
}
\examples{
data(Simon)
#Get R-value from an object of the Spot class and save the result
R <- r.arise(Simon)
#Show the R-values}
\keyword{math}
\keyword{arith}

\name{DataSet-class}
\docType{class}
\alias{DataSet-class}
\title{DataSet - class}

\description{
A simple list-based class for storing red and green channel foreground,
z-scores and the Ids.
}

\section{Creating Objects from the Class}{
Objects can be created by calls of the form \code{new("DataSet",sets, type)}
where \code{sets} is a list containing Cy3, Cy5, Id and Zscore and
\code{type} is "ri" or "ma".
Objects are normally created by \code{\link{read.spot}}.
}

\section{Slots/List Components}{
This class contains no slots (other than \code{.Data}), but objects should contain the following list components:
\tabular{ll}{
  \code{Cy5}:\tab numeric matrix containing the red (cy5) foreground
  intensities.  Rows correspond
  to spots and columns to arrays.\cr
  \code{Cy3}:\tab numeric matrix containing the green (cy3) foreground intensities.\cr
  \code{Id}:\tab Ids from all the observations.\cr
  \code{Zscore}:\tab The result of (R - mean) / sd that define an
  intensity-dependent Z-score threshold to identify differential expression.
}
All of these matrices should have the same dimensions.
}

\section{Methods}{
This class inherits directly from class \code{list} so any operation appropriate for lists will work on objects of this class.
}

\keyword{classes}
\keyword{data}

\name{trim}
\alias{trim}
\title{Trim}
\description{
  Extract white spaces at the begining or end of a word.
} 
\usage{
trim(word)
}
\arguments{
  \item{word}{A string of characters posibly with white spaces at
    the beging or end of the string.  }
}
\value{
 Returns a string of characters, with leading and trailing whitespace omitted.
}
\examples{

trim("        This is a String            ")
## return [1] "This is a String"

}
\keyword{character}

\name{i.arise}
\alias{i.arise}
\title{I Arise}
\description{
Extract \bold{I} from a Spot.
}
\usage{
i.arise(mySpot)
}
\arguments{
\item{mySpot}{Spot object for one microarray.}
}
\value{
List of I-values
}
\seealso{
\code{\link{r.arise}}.
}
\examples{
data(Simon)
## Extract I from spot and save in i
i.arise(Simon)
}
\keyword{math}

\name{ma.plot}
\alias{ma.plot}
\title{Data Visualization: M vs. A plot}
\description{
This function allows to plot \bold{M -vs- A} in spot.
}
\usage{
ma.plot(mySpot, col = "green")
}
\arguments{
  \item{mySpot}{Spot for one microarray.}
  \item{col}{color of points in graphic}
}
\examples{
data(Simon)
## plot the signals for spot.
ma.plot(Simon)
}
\keyword{aplot}
\keyword{arith}

\name{read.spot}
\alias{read.spot}
\title{Read Spot from File}
\description{
Read all file, but only extract the interested columns and create a Spot object.
}
\usage{
read.spot(file.name, cy3, cy5, bg.cy3, bg.cy5, ids, symdesc, header =
FALSE, sep = "\t", is.ifc = FALSE, envir)
}
\arguments{
\item{file.name}{a connection or a character string giving the
name of the file to read where each column represent the spot components.}
\item{cy3}{column that represent Cy3.}
\item{cy5}{column that represent Cy5.}
\item{bg.cy3}{column that represent BgCy3.}
\item{bg.cy5}{column that represent BgCy5.}
\item{ids}{column that represent Id.}
\item{symdesc}{(optional) identifier besides Id column.}
\item{header}{the logical value of the header input file}
\item{sep}{the separator in the inputfile}
\item{is.ifc}{If is.ifc = TRUE this experiment was done in the
  Unit of Microarray  from  Cellular Phisiology Institute.}
\item{envir}{Environment where are the genArise variables. You don't
  need to specify this argument.}
}
\seealso{
\code{\link{write.spot}}.
}
\keyword{file}

\name{get.Zscore}
\alias{get.Zscore}
\title{Swap from Files}
\description{
Read both files, but only extract the interested columns and create a Spot object.
}
\usage{
get.Zscore( spot, name, Zscore.min=NULL, Zscore.max=NULL, all=FALSE, envir)
}
\arguments{
\item{spot}{a connection or a character string giving the
  name of the file to read where each column represent the spot
  components.}
\item{name}{a connection or a character string giving the
  name of the file to read where each column represent the spot
  components.}
\item{Zscore.min}{column that represent Cy3.}
\item{Zscore.max}{column that represent Cy5.}
\item{all}{column that represent BgCy3.}
\item{envir}{Environment where are the genArise variables.}
}
\seealso{
\code{\link{write.spot}}.
}
\keyword{file}
